/*
 * Copyright (C) 2001 Lexra, Inc
 */

#include <memory_map.h>

#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/mm.h>
#include <linux/swap.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/timex.h>
#include <asm/bootinfo.h>
#include <asm/page.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/processor.h>
#include <asm/ptrace.h>
#include <asm/reboot.h>
#include <asm/mc146818rtc.h>
#include <linux/version.h>
#include <linux/bootmem.h>
#include <asm/irq.h>

#include <linux/mc146818rtc.h>

#ifdef ISHONI_ENGINE
#include <asm/ibe/ibeirq.h>
#include <asm/ibe/bootcommon.h>
#include <drv_rtcpcf8583.h>
#include "bootinit.h"

#ifndef LINUX_BOOT_LOADER
#include <asm/ibe/ibe_reset.h>
#endif

#else
#include <asm/lexra/lxirq.h>
#endif /* ISHONI_ENGINE */

#include <asm/lexra/lx_defs.h>

#ifdef ISHONI_ENGINE
#include <asm/mipsregs.h>
#endif
/**
 * The control register addresses of the Lexra BUSCLK timer.
 */
#define BUSCLK_CTRL_ADDR	0xbc000010
/**
 * The control register addresses of the Lexra SYSCLK timer.
 */
#define SYSCLK_CTRL_ADDR	0xbc000014
/**
 * The control register addresses of the Lexra UARTCLK timer.
 */
#define UARTCLK_CTRL_ADDR	0xbc000018

/**
 * Sys_clock is a variable used to store the system clock frequency
 * chosen during kernel configuration.
 * We default to using UART clock as the system timer. This was chosen
 * as the default because the UART clock crystal is the most likely
 * to be available and have a known frequency of 18.432 MHz.
 */
#define Sys_clock (18.432 * 1000000)

/**
 * Sys_clock_ctrl is a variable used to store the control register address
 * of the clock chosen to provide the system time during kernel configuration.
 * We default to using UART clock as the system timer. This was chosen
 * as the default because the UART clock crystal is the most likely
 * to be available and have a known frequency of 18.432 MHz.
 */
#define Sys_clock_ctrl UARTCLK_CTRL_ADDR

/**
 * The rate of the crystal used for the sytem clock must be defined
 * as a configuration option.
 */
#ifdef CONFIG_SYSCLK_18
#define Sys_clock (18.432 * 1000000)
#endif
#ifdef CONFIG_SYSCLK_32
#define Sys_clock (32 * 1000000)
#endif
#ifdef CONFIG_SYSCLK_20
#define Sys_clock (20 * 1000000)	//20 MHz
#endif
#ifdef CONFIG_SYSCLK_33
#define Sys_clock (33 * 1000000)	// 33 MHz
#endif
#ifdef CONFIG_SYSCLK_25
#define Sys_clock (25 * 1000000)	// 25 MHz
#endif

/**
 * This series of defines determine which clock net was chosen as the
 * time base at configuration time. The Lexra development boards
 * generally have three up-down counter timers to choose from.
 */
#ifdef CONFIG_LX_BUSCLK
#ifdef ISHONI_ENGINE
#undef Sys_clock_ctrl
#endif
#define Sys_clock_ctrl BUSCLK_CTRL_ADDR
#endif
#ifdef CONFIG_LX_SYSCLK
#ifdef ISHONI_ENGINE
#undef Sys_clock_ctrl
#endif
#define Sys_clock_ctrl SYSCLK_CTRL_ADDR
#endif
#ifdef CONFIG_LX_UARTCLK
#ifdef ISHONI_ENGINE
#undef Sys_clock_ctrl
#endif
#define Sys_clock_ctrl UARTCLK_CTRL_ADDR
#endif

/**
 * Variable for storage of the clock decrement value
 */
int LX_Dec_Value;

/**
 * Function stub. Function not required for Lexra implementation of Linux.
 */
void page_is_ram(void) {};

/**
 * Function stub. Function not required for Lexra implementation of Linux.
 */
void prom_free_prom_memory(void) {};

/**
 * A structure to store parameters to be passed from the boot loader
 * to the kernel.
 */
struct bootParams lxBootParams;

/**
 * Macro to get address of next page.
 * @param x Address of current page.
 */
#define PFN_UP(x)       (((x) + PAGE_SIZE-1) >> PAGE_SHIFT)
/**
 * Macro to align the supplied address to the next page boundary
 * @param x Address to align.
 */
#define PFN_ALIGN(x)    (((unsigned long)(x) + (PAGE_SIZE - 1)) & PAGE_MASK)

/**
 * Variable containing the address of the end of the kernel data
 */
extern int _end;

/**
 * Variable to store the size of the RAM.
 */
unsigned long mem_size;

/**
 * Boolean variable to indicate if the Lexra processor has DSP features.
 */
unsigned int dspProduct;

/**
 * The xtime clock lock used to lock the xtime structure while it is being updated.
 */
extern rwlock_t xtime_lock;

#ifdef ISHONI_ENGINE

spinlock_t rtc_lock = SPIN_LOCK_UNLOCKED; 
btParamStruct  flashParamStruct;
int saveFlashParamsFlag;
int cpuBootWord;

#ifdef NEW_FEATURES
flashMapStructure flashPartitionTblStruct;
int saveFlashPartitionTblFlag;

#ifdef BOOT_TIME_DEFAULTS
/**
 * A variable to store the number of times this board has been soft-resetted!
 * This variable will be used to dbDeclare the count when the database is up
 */
unsigned int resetCount;
#endif /* BOOT_TIME_DEFAULTS */


#ifdef BOOT_TIME_DEFAULTS
/**
 * A variable to store the number of times this board has been soft-resetted!
 * This variable will be used to dbDeclare the count when the database is up
 */
unsigned int resetCount;
#endif /* BOOT_TIME_DEFAULTS */

#endif /* ISHONI_ENGINE */

extern asmlinkage void ibe_handle_int(void);

extern asmlinkage void ibe_nvPicHandler(void);
void __init ibe_init_interrupts(void);

decint_t ibe_interrupt[NR_INTS];

void start_ibe_timer(void);

extern asmlinkage void handle_ri(void);
extern char except_vec3_generic;  /* for reserved instruction exception */ 
#else
/**
 * Declaration of the MIPS standard interrupt dispatch function.
 * This function is defined in int-handler.S.
 */
extern asmlinkage void lx_handle_int(void);

/**
 * Declaration of the Lexra vectored interrupt dispatch function.
 * This function is defined in int-handler.S.
 */
extern asmlinkage void lx_vec_handle_int(void);
extern asmlinkage void handle_ri(void);
extern char except_vec3_generic;  /* for reserved instruction exception */
void lx_init_lxPB20K(void);

/**
 * An array of lxint_t structures which store the masking information
 * for each of the 32 Linux interrupts. This data is used to enable/disable
 * either a MIPS, Lexra vectored or PCI interrupt.
 */
lxint_t lx_interrupt[NR_INTS];

#endif
#ifdef ISHONI_ENGINE
void ibeEnableNetPICinterrupt();
#endif

#ifndef ISHONI_ENGINE 
/**
 * Unimplemented power control function.
 */
extern void lx_machine_restart(char *command);
/**
 * Unimplemented power control function.
 */
extern void lx_machine_halt(void);
/**
 * Unimplemented power control function.
 */
extern void lx_machine_power_off(void);
#endif

extern void wbflush_setup(void);

/**
 * External function defined in irq.c.
 */
extern int setup_lx_irq(int, struct irqaction *);

/**
 * The irqaction structure for the system timer.
 */
extern void (*board_timer_setup) (struct irqaction * irq);

extern struct rtc_ops lx45xxx_rtc_ops;

/**
 * Pointer to the system clock control register.
 */
volatile unsigned int* Sys_clock_ctrl_reg;

/**
 * The default UART channel, UARTA.
 */
 #define channel 1

/**
 * External function to send a character to the UART.
 */
extern void serial_putc(int chan, int c);

/**
 * External function used to set an interger value to the UART as
 * a hexidecimal string.
 */
extern void int2hex(int i);

/**
 * Variable to store the limits of the initial ramdisk.
 */
extern unsigned long initrd_start,initrd_end;
extern void * __rd_start, * __rd_end;

/**
 * This funtion initializes kernel variables with data from the boot
 * loader.
 */
int prom_init(int argc, char **argv, char **envp, int *prom_vec)
{
#ifdef BOOT_TIME_DEFAULTS
    int *resetCntPtr;
#endif /* BOOT_TIME_DEFAULTS */

#ifdef CONFIG_BLK_DEV_INITRD
	ROOT_DEV=MKDEV(RAMDISK_MAJOR, 0);
	initrd_start = (unsigned long)&__rd_start;
	initrd_end = (unsigned long)&__rd_end;
#endif
#if defined(ISHONI_ENGINE) && !defined(LINUX_BOOT_LOADER)
    {
        unsigned int *memptr = (unsigned int *)(BOOT_MSG_BOX + 4);
        int i, j = 1;
        cpuBootWord = 0;
        for (i=0; i<=3; i++) {
            if(*memptr != NOBOOT_WAIT_VEC) {
                cpuBootWord |= j;
            }
            j <<= 1;
            memptr += 2;
        }
    }

#ifdef BOOT_TIME_DEFAULTS
    printk("Board has been soft reset:%d times\n",
            ((struct bootParams *)prom_vec)->resetCount);

    if (((struct bootParams *)prom_vec)->resetCount > 0) {
        resetCount = ((struct bootParams *)prom_vec)->resetCount;
    }

    /*
     * kernel has booted. Reset the count back to zero.
     * Hope the kernel hasn't overwritten the location with its own data!
     */
     resetCntPtr = (unsigned int *)(RESET_COUNT_VALUE_ADDR);
    *resetCntPtr = 0;

#endif /* BOOT_TIME_DEFAULTS */

    if (!((int)prom_vec & 0x3) && 
            (((struct bootParams *)prom_vec)->bootParamMagicNumber == BOOT_PARAM_MAGIC_NUMBER)) {
	    memcpy(&lxBootParams.cmd_line, 
	       ((struct bootParams *)prom_vec)->cmd_line,
	       ((struct bootParams *)prom_vec)->cmd_line_size);
	    lxBootParams.cmd_line_size =
	       ((struct bootParams *)prom_vec)->cmd_line_size;
        ((struct bootParams *)prom_vec)->bootParamMagicNumber = 0;
        if (((struct bootParams *)prom_vec)->saveFlashParamsFlag == 1) {
            saveFlashParamsFlag = 1;
	        memcpy(&flashParamStruct, (unsigned int *)(LINUX_KERNEL_BOOTPARAM_ADDR + sizeof(struct bootParams)), 
                    sizeof(btParamStruct));
        }
#ifdef NEW_FEATURES
        if (((struct bootParams *)prom_vec)->saveFlashPartitionTblFlag == 1) {
            saveFlashPartitionTblFlag = 1;
	        memcpy(&flashPartitionTblStruct, (unsigned int *)(LINUX_KERNEL_BOOTPARAM_ADDR + 
                    sizeof(struct bootParams) + sizeof(btParamStruct)), sizeof(flashMapStructure));
        }
#endif
    }
#endif

    mips_machgroup = MACH_GROUP_LEXRA;
	mips_machtype = MACH_LXPB20K;

#ifdef ISHONI_ENGINE
	#ifndef LINUX_BOOT_LOADER
		#ifdef SW_MEMMAP_32MB_VER1
			#ifndef EMUL_BRD
				/* #if (!defined CBUS_4MB_ADDRESS_USED) || (!defined INCLUDE_ISHONI_WIRELESS) */
				#ifdef BOARD_WIRELESS_TYPHOON 
					mem_size = 32  << 20; 
				#else
					mem_size = 32  << 20; /* changed from 16 MB to 48 MB.*/
				#endif
			#else
				mem_size = 32  << 20; /* changed from 16 MB to 48 MB.*/
			#endif
        #elif defined(SW_MEMMAP_16MB_VER2) 
            mem_size = LX_KERNEL_MEM_SIZE;
        #else
			/* changed from 14 MB to 16 MB. Giving 1 MB to second block of shared memory.*/
			mem_size = 16  << 20; 
		#endif
	#else
		#ifdef SW_MEMMAP_32MB_VER1
			mem_size = 14  << 20; /* changed to 14 MB. Linux BL can use 5M,the first 9M vacant.*/
		#else
			mem_size = 7  << 20;  /* changed to 7 MB. Linux BL can use 5M,the first 2M vacant.*/
		#endif
	#endif /* LINUX_BOOT_LOADER */
#else
	mem_size = 64 << 20;
#endif

	printk("%d MB SDRAM.\n", (int)(mem_size >> 20));
	add_memory_region(0, mem_size, BOOT_MEM_RAM);
#if defined(SW_MEMMAP_16MB_VER2) 
	add_memory_region(mem_size, LX_KERNEL_RES_MEM_SIZE, BOOT_MEM_RESERVED);
#endif

	mips_io_port_base = LX_PORT_BASE;

	/* Redefine the end of ioport resource memory space */
	ioport_resource.end = 0xbcffffff;
        return 0;

};

/**
 * Variable to store the time of the last clock update.
 */
static long last_rtc_update;

/**
 * This function stub allows the Lexra board to run Linux without
 * a real time clock.
 * @param nowtime Unused.
 */
static int set_rtc_mmss(unsigned long nowtime)
{

#ifndef EMUL_BRD /* We don't have RTC on emulation board.*/
#ifdef LINUX_BOOT_LOADER
    return 0;
#else
   /* put ishoni stuff here.*/
    int retval = 0;
    int real_seconds, real_minutes;
    int rtc_seconds, rtc_minutes; 
                
    printk("\n time.c: set_rtc_mmss() .");

    /* gets recalled with irq locally disabled */
    spin_lock(&rtc_lock);

    /* we're only adjusting minutes and seconds.*/
    real_seconds = nowtime % 60;
    real_minutes = nowtime / 60;

    /* Read minute and seconds from RTC.*/
    getTime8583(0,0,0, 0, &rtc_minutes, &rtc_seconds, 0, 0);

    if (((abs(real_minutes - rtc_minutes) + 15)/30) & 1)
       real_minutes += 30; /* correct for half hour time zone */
    real_minutes %= 60;

    if (abs(real_minutes - rtc_minutes) < 30) 
       setTime8583(-1,-1,-1, -1, real_minutes, real_seconds, -1, -1);
    else { 
        printk("set_rtc_mmss: can't update from %d to %d\n", rtc_minutes, real_minutes);
        retval = -1;
    } 

    spin_unlock(&rtc_lock);
    return retval;
#endif /* LINUX_BOOT_LOADER */

#else
	return 0;
#endif
}

/**
 * Threshold to avoid timer interupt latency problems.
 */
#define TICK_THRESHOLD	(LX_Dec_Value >> 2)

#ifdef ISHONI_ENGINE
/* We cannot work out with the lx_timer_interrupt,
 * because its very much specific for Lexra board.
 * We need write a different routine for this.
 */
static void inline
ibe_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
        /* printk("\n Timer interrupt. Tick = %d", jiffies); */

    /* Is this the right place ? */

    /* Clear the Timer Interrupt here. */
        CLEAR_TIMER_INTERRUPT(MY_TIMER_ADDRESS);

        if (!user_mode(regs)) {
                if (prof_buffer && current->pid) {
                        extern int _stext;
                        unsigned long pc = regs->cp0_epc;

                        pc -= (unsigned long) &_stext;
                        pc >>= prof_shift;
                        /*
                         * Dont ignore out-of-bounds pc values silently,
                         * put them into the last histogram slot, so if
                         * present, they will show up as a sharp peak.
                         */
                        if (pc > prof_len - 1)
                                pc = prof_len - 1;
                        atomic_inc((atomic_t *) & prof_buffer[pc]);
                }
        }
        do_timer(regs);

        /*
         * If we have an externally synchronized Linux clock, then update
         * CMOS clock accordingly every ~11 minutes. Set_rtc_mmss() has to be
         * called as close as possible to 500 ms before the new second starts.
         */
        read_lock(&xtime_lock);
        if ((time_status & STA_UNSYNC) == 0
            && xtime.tv_sec > last_rtc_update + 660
            && xtime.tv_usec >= 500000 - tick / 2
            && xtime.tv_usec <= 500000 + tick / 2) {
                if (set_rtc_mmss(xtime.tv_sec) == 0)
                        last_rtc_update = xtime.tv_sec;
                else
                        /* do it again in 60 s */
                        last_rtc_update = xtime.tv_sec - 600;
        }
        /* As we return to user mode fire off the other CPU schedulers.. this is
           basically because we don't yet share IRQ's around. This message is
           rigged to be safe on the 386 - basically it's a hack, so don't look
           closely for now.. */
        /*smp_message_pass(MSG_ALL_BUT_SELF, MSG_RESCHEDULE, 0L, 0); */
       read_unlock(&xtime_lock);

}

#else
/**
 * The sytem timer interrupt routine. This is the only time base
 * for Lexra boards, there is no real time clock. The routine reads
 * the value of the clock decrementer and uses the negative value to
 * the reset decrementer value to adjust for the interrupt latency.
 * Subsequently the Linux do_timer() routine is called to handle
 * the general Linux timing functions.
 * Additional code exists for updating profiling structures and the
 * system time.
 * @param irq The timer irq number.
 * @param dev_id Device id, NULL for timer interrupt.
 * @param regs Pointer to stack passed through by interrupt handler.
 * @return void
 */
static void inline
lx_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	volatile int dummy, dummy1;

	dummy = *Sys_clock_ctrl_reg;		/* read counter*/

	/* Case where sys clock has *REALLY* rolled over */
	if (dummy > 0) {
	        dummy1 = (1<<29) - 1 - dummy;
	        jiffies += (dummy1/LX_Dec_Value);
		dummy = -(dummy1%LX_Dec_Value);	
 	}
	/**
	  *
	  * Calculate what the new clock value should be.
	  * This is the "normal processing. We are not late and we don't
	  * have to add extra ticks. (do_timer increments jiffies)
	  *
	  **/
	dummy = (dummy + LX_Dec_Value);

	/**
	  * If we've rolled too far negative, compensate for it.
	  *
	  * We keep adding time (and ticking) until we will go to sleep for
	  * a significant period, to minimise the overhead of an interrupt
	  * immediately after we service this one
	  *
	  **/

	while (dummy <= TICK_THRESHOLD) {
		jiffies++;
		dummy += LX_Dec_Value;
	}

	/**
	  * dummy is a reasonable value, we've tried to compensate jiffies
	  * for anything missed
	  **/

	/* Set counter */
	*Sys_clock_ctrl_reg = (dummy | 0x40000000);

	if (!user_mode(regs)) {
		if (prof_buffer && current->pid) {
			extern int _stext;
			unsigned long pc = regs->cp0_epc;

			pc -= (unsigned long) &_stext;
			pc >>= prof_shift;
			/*
			 * Dont ignore out-of-bounds pc values silently,
			 * put them into the last histogram slot, so if
			 * present, they will show up as a sharp peak.
			 */
			if (pc > prof_len - 1)
				pc = prof_len - 1;
			atomic_inc((atomic_t *) & prof_buffer[pc]);
		}
	}

	do_timer(regs);

	/*
	 * If we have an externally synchronized Linux clock, then update
	 * CMOS clock accordingly every ~11 minutes. Set_rtc_mmss() has to be
	 * called as close as possible to 500 ms before the new second starts.
	 */
	read_lock(&xtime_lock);
	if ((time_status & STA_UNSYNC) == 0
	    && xtime.tv_sec > last_rtc_update + 660
	    && xtime.tv_usec >= 500000 - tick / 2
	    && xtime.tv_usec <= 500000 + tick / 2) {
		if (set_rtc_mmss(xtime.tv_sec) == 0)
			last_rtc_update = xtime.tv_sec;
		else
			/* do it again in 60 s */
			last_rtc_update = xtime.tv_sec - 600;
	}
	/* As we return to user mode fire off the other CPU schedulers.. this is
	   basically because we don't yet share IRQ's around. This message is
	   rigged to be safe on the 386 - basically it's a hack, so don't look
	   closely for now.. */
	/*smp_message_pass(MSG_ALL_BUT_SELF, MSG_RESCHEDULE, 0L, 0); */
	read_unlock(&xtime_lock);
}
#endif /* ISHONI_ENGINE */

/**
 * Function to initialize the Lexra timer. This initialization includes
 * calculating the appropriate decrementer value based on the configured
 * clock rate, requesting and registering the interrupt and starting the
 * timer.
 * @param irq Pointer to the irqaction structure for the timer. This structure
 * stores information required by the do_IRQ() function which dispatches interrupts.
 */
static void __init lx_time_init(struct irqaction *irq)
{
#ifndef ISHONI_ENGINE
	/* Instruction order is important */
	__asm__ __volatile__(".set\tnoreorder\n\t");

	/* setup ptr to contrl reg */
	Sys_clock_ctrl_reg = (unsigned int *)Sys_clock_ctrl;	
	*Sys_clock_ctrl_reg = 0xa0000000;	/* reset clock */

	irq->handler = lx_timer_interrupt;
	irq->flags = 0;
	irq->mask = 0;
	irq->name = "System Clock";
	irq->next = NULL;
	irq->dev_id = NULL;

	/* must have user setup clock frequencies */
	/* Lx_Dec_Value = # of sysclk counter ticks per 10 ms */
	LX_Dec_Value = Sys_clock / HZ;
	switch(Sys_clock_ctrl){
	case BUSCLK_CTRL_ADDR:
		setup_lx_irq(BUSCLK, irq);
		break;
	case SYSCLK_CTRL_ADDR:
		setup_lx_irq(SYSCLK, irq);
		break;
	default:
		setup_lx_irq(UARTCLK, irq);
		break;
	}

	/* start the counter with initial (calculated) value */
	*Sys_clock_ctrl_reg = 0x40000000 | LX_Dec_Value;

	__asm__ __volatile__(".set\treorder");
#else
    /* ISHONI Changes here. */
    /* We should make this as a separate function. Nazim .*/
  
    irq->handler = ibe_timer_interrupt;
    irq->flags = SA_INTERRUPT;
    irq->mask = 0;
    irq->name = "System Clock";
    irq->next = NULL;
    irq->dev_id = NULL;

    setup_lx_irq(TIMER0_IRQ, irq);

    /* Program IBE Timer here. */
    start_ibe_timer();

#endif /* ifndef ISHONI_ENGINE */
}


#ifdef ISHONI_ENGINE
void ibeEnableNetPICinterrupt(void)
{
    unsigned int s;

    cli();

        /* Where it enables IEc Bit ib SR register. Lets do it here.*/
        s = read_32bit_cp0_register(CP0_STATUS);
/*      s |= (0x0000f000 | ST0_IE); */
        s |= (0x00004000);
        write_32bit_cp0_register(CP0_STATUS, s);

    sti();
    return;
}
#endif 
/**
 * Lexra specific interrupt setup function. This function calls the
 * routine which initializes the Lexra interrupt handling structures.
 * Following this the MIPS interrupt exception vector is installed.
 */
void __init lx_irq_setup(void)
{
#ifdef ISHONI_ENGINE

    switch (mips_machgroup) {
    case MACH_GROUP_LEXRA:
        ibe_init_interrupts();
        ibeEnableNetPICinterrupt();
        break;
    default:
     panic("Don't know how to set this up!");
    }
    /* setupt the interrupt exception vector to dispatch MIPS type interrupts */
    set_except_vector(0, ibe_handle_int);

#else
	int i;
    switch (mips_machgroup) {
		case MACH_GROUP_LEXRA:
		lx_init_lxPB20K();
		break;
	}
	/* setupt the interrupt exception vector to dispatch 
	 * MIPS type interrupts 
	 */
	set_except_vector(0, lx_handle_int);
	for (i=0; i<NR_INTS; i++) {
		disable_irq(i);
	}
#endif /* ifndef ISHONI_ENGINE */
}

/**
 * Stores the command line entered at the boot prompt.
 */
char arcs_cmdline[COMMAND_LINE_SIZE] = {0, };

/**
 * Lexra machine specific setup routine. This routine sets up function
 * pointers used by the kernel intitialization code to intialize
 * various hardware specific operations like interrupts, timers and
 * io space.
 */
void lexra_setup(void)
{
	/**
	 * LX4xxx/5xxx specific RTC routines
	 * this is a structure containing pointers to RTC functions
	 * we don't have an RTC so stubs must be defined
	 */
	rtc_ops = &lx45xxx_rtc_ops;

/*	irq_setup = lx_irq_setup; */
	board_timer_setup = lx_time_init;

	memcpy(&arcs_cmdline, &(lxBootParams.cmd_line),
	       lxBootParams.cmd_line_size);

	mips_io_port_base = KSEG1;

	/*  trap MAC instructions before trap_init is called in Linux kernel,
	    it will be override later in trap_init */
	set_except_vector(10, handle_ri);

/* This is already handled in the trap_init() */
#ifndef ISHONI_ENGINE
	memcpy((void *)(KSEG0 + 0x80), &except_vec3_generic, 0x80);
#endif

#ifdef ISHONI_ENGINE
#ifndef LINUX_BOOT_LOADER 
    /* Reboot hooks for IBE System.*/ 
    _machine_restart = ibeBoardRestart;
    _machine_halt = ibeBoardHalt;
    _machine_power_off = ibeBoardPowerOff;
#endif
#endif

};

#ifdef ISHONI_ENGINE

void __init ibe_init_interrupts(void)
{

#ifdef CHECK_THIS /* NAZIM */
    /*
     * Setup some memory addresses.
     */
    dec_rtc_base = (char *) KN02_RTC_BASE;

#endif

    /* IBE Netvoice PIC Controller setup.*/


    /* Set up NetPIC HW interrupt. */
    cpu_ivec_tbl[0] = (unsigned int)ibe_nvPicHandler;

        /* Why are we putting this entry at index 0?
     * cpu_ivec_table[] defines the prioroties of CPU interrupts.
     * First entry in this table has the highest priority.
     * So we are giving highest priority to NetPIC. 
     */

        /* Important: If this is a NETPIC interrupt, we give -1 so that
     * in the interrupt handler code "ibe_handle_int"
     * we can distinguish between other hardware interrupts and
     * NetPIC interrupts.
     */
    cpu_irq_nr[0] = -1;
    cpu_mask_tbl[0] = IE_IRQ4;


        /* Set up Ishoni timer interrupt.*/
    ibe_interrupt[TIMER0_IRQ].cpu_mask = IE_IRQ4;
    ibe_interrupt[TIMER0_IRQ].iemask = (1 << PIC_INT(TIMER0_IRQ));
    netpic_irq_nr[PIC_INT(TIMER0_IRQ)] = TIMER0_IRQ;

    /* Set up host interrupt.*/
    ibe_interrupt[HOST_IRQ].cpu_mask = IE_IRQ4;
    ibe_interrupt[HOST_IRQ].iemask = (1 << PIC_INT(HOST_IRQ));
    netpic_irq_nr[PIC_INT(HOST_IRQ)] = HOST_IRQ;

    /* set up the SPI interrupt */
    ibe_interrupt[SPI_IRQ].cpu_mask = IE_IRQ4;
    ibe_interrupt[SPI_IRQ].iemask = (1 << PIC_INT(SPI_IRQ));
    netpic_irq_nr[PIC_INT(SPI_IRQ)] = SPI_IRQ;

    /* set up the PIO interrupt */
    ibe_interrupt[PIO_IRQ].cpu_mask = IE_IRQ4;
    ibe_interrupt[PIO_IRQ].iemask = (1 << PIC_INT(PIO_IRQ));
    netpic_irq_nr[PIC_INT(PIO_IRQ)] = PIO_IRQ;

#ifdef LINUX_BOOT_LOADER
    /* set up the ETH_CORE0_IRQ interrupt */
    ibe_interrupt[ETH_CORE0_IRQ].cpu_mask = IE_IRQ4;
    ibe_interrupt[ETH_CORE0_IRQ].iemask = (1 << PIC_INT(ETH_CORE0_IRQ));
    netpic_irq_nr[PIC_INT(ETH_CORE0_IRQ)] = ETH_CORE0_IRQ;

    /* set up the BM ether interrupt */
    ibe_interrupt[ETH_BM0_IRQ].cpu_mask = IE_IRQ4;
    ibe_interrupt[ETH_BM0_IRQ].iemask = (1 << PIC_INT(ETH_BM0_IRQ));
    netpic_irq_nr[PIC_INT(ETH_BM0_IRQ)] = ETH_BM0_IRQ;

    /* set up the ETH_CORE1_IRQ interrupt */
    ibe_interrupt[ETH_CORE1_IRQ].cpu_mask = IE_IRQ4;
    ibe_interrupt[ETH_CORE1_IRQ].iemask = (1 << PIC_INT(ETH_CORE1_IRQ));
    netpic_irq_nr[PIC_INT(ETH_CORE1_IRQ)] = ETH_CORE1_IRQ;

    /* set up the BM ether interrupt */
    ibe_interrupt[ETH_BM1_IRQ].cpu_mask = IE_IRQ4;
    ibe_interrupt[ETH_BM1_IRQ].iemask = (1 << PIC_INT(ETH_BM1_IRQ));
    netpic_irq_nr[PIC_INT(ETH_BM1_IRQ)] = ETH_BM1_IRQ;
#endif

    /* Add other entries here. */

    return;

}                               /* ibe_init_interrupts */

#else /* ISHONI_ENGINE */

/**
 * Lexra specific initialization  of interrupts.
 * Support is provided for the MIPS 6 hardware and 2 software interrups,
 * the 8 Lexra vectored interrupts and 16 available PCI interrupts.
 * The interrupts are prioritized through the used of the mask and irq_nr
 * tables. This allows for simple modification of the interrupt priority
 * by changing the relative location of the interrupt in these tables.
 */
void __init lx_init_lxPB20K(void)
{
	unsigned int index;

	/**
	* Setup interrupt structure
	*/

	lx_interrupt[I2S].cpu_mask = 0;
	lx_interrupt[I2S].LX_mask = LX_IRQ11;
	lx_mask_tbl[4] = LX_IRQ11;
	lx_irq_nr[4] = I2S;


	lx_interrupt[PCI_ABCD].cpu_mask = IE_IRQ0;
	lx_interrupt[PCI_ABCD].LX_mask = 0;
	cpu_mask_tbl[2] = IE_IRQ0;
	cpu_irq_nr[2] = PCI_ABCD;

	lx_interrupt[UART12TXRX].cpu_mask = IE_IRQ1;
	lx_interrupt[UART12TXRX].LX_mask = 0;
	cpu_mask_tbl[3] = IE_IRQ1;
	cpu_irq_nr[3] = UART12TXRX;

	lx_interrupt[BUSCLK].cpu_mask = IE_IRQ2;
	lx_interrupt[BUSCLK].LX_mask = 0;
	cpu_mask_tbl[4] = IE_IRQ2;
	cpu_irq_nr[4] = BUSCLK;

	lx_interrupt[SYSCLK].cpu_mask = IE_IRQ3;
	lx_interrupt[SYSCLK].LX_mask = 0;
	cpu_mask_tbl[5] = IE_IRQ3;
	cpu_irq_nr[5] = SYSCLK;

	lx_interrupt[UARTCLK].cpu_mask = IE_IRQ4;
	lx_interrupt[UARTCLK].LX_mask = 0;
	cpu_mask_tbl[6] = IE_IRQ4;
	cpu_irq_nr[6] = UARTCLK;

	lx_interrupt[SYSTIMER].cpu_mask = IE_IRQ5;
	lx_interrupt[SYSTIMER].LX_mask = 0;
	cpu_mask_tbl[7] = IE_IRQ5;
	cpu_irq_nr[7] = SYSTIMER;

}/* lx_init_lxBP20K */
#endif /* ISHONI_ENGINE */ 
#ifdef ISHONI_ENGINE
/*
* msecCount - return 1 msec count of system clock. 
*/
int  msecCount(void);
extern void getSysClkConfig(int *, char *);
int  msecCount()
{
    int freq;
    getSysClkConfig(&freq,NULL);
    return ( ( (freq * 1000000) / 16 )/1000) ;
}
        
void start_ibe_timer(void)
{
#ifdef EMUL_BRD
        /* changed the timer frequency 25MHz */
        SET_TIMER_CONSTANT(MY_TIMER_ADDRESS, 15000); /*TBD: timer calc*/
#else
#define ADJUST_TIMER_CONSTANT
#ifdef ADJUST_TIMER_CONSTANT
        /* This is a hack to temp fix the counter  in the software. 
         * This value 83332 is calculated on the typhoon board running at 133Mhz 
         * DO NOT use this on the other board running at other frequency.
         */  
        SET_TIMER_CONSTANT(MY_TIMER_ADDRESS, 83332); /*60000 for 100 MHz*/
#else        
        SET_TIMER_CONSTANT(MY_TIMER_ADDRESS, msecCount() * (1000 / HZ) ); /*60000 for 100 MHz*/
#endif        
        
#endif
        /* SET_TIMER_CONSTANT(MY_TIMER_ADDRESS, 1000);*/

        SET_TIMER_CONTROL(MY_TIMER_ADDRESS,
                        TM_CNTL_RELOAD_CONSTANT_START << TM_CNTL_START_SHF |
                        TM_CNTL_ENABLE_INTERRUPT << TM_CNTL_INT_EN_SHF |
                        TM_CNTL_AUTO_RELOAD_ON << TM_CNTL_AUTO_RELOAD_SHF);

        /* SET_PIC_IEN(1 << 16); */
}

#endif /* ISHONI_ENGINE */ 
