/*
 * THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
 * ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 * Copyright (C) NEC Corporation 2001.
 */
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/pci.h>
#include <linux/ioport.h>

#include <asm/types.h>
#include <asm/vr4122/vr4122.h>
#include <asm/vr4122/eagle.h>

#ifdef CONFIG_EAGLE_REV1
#define EAGLE_REV1_ON	{						\
		unsigned short val;					\
		val = readw(VR4122_GIUPIODL);				\
		val |= VR4122_GIUPIODL_GPIO9;				\
		writew(val, VR4122_GIUPIODL); }
#define EAGLE_REV1_OFF	 {						\
		unsigned short val;					\
		val = readw(VR4122_GIUPIODL);				\
		val &= VR4122_GIUPIODL_GPIO9;				\
		writew(val, VR4122_GIUPIODL); }
#else
#define EAGLE_REV1_ON
#define EAGLE_REV1_OFF
#endif

#define MAX_PCI_DEV	64

extern struct pci_ops vr4122_pci_ops;
extern void vr4122_pcibios_init(void);

static unsigned long memstart = 0x10200000;
static unsigned long memlimit = 0x15ffffff;
static unsigned long iostart =  0x00010000;
static unsigned long iolimit =  0x01ffffff;

static int pcidevs = 0;

static unsigned long  ReadConfig32(int bus, int dev, int func, int reg)
{
	unsigned long config;

	if (bus == 0) {
		/* type 0 configuration */
		if (dev < 11 || dev >= 32 || func >= 8 || reg >= 256)
			return 0xffffffff;
		EAGLE_REV1_ON;
		writel((reg & 0xfc) | (func << 8) | (1 << dev), VR4122_PCICONFAREG);
	}
	else {
		/* type 1 configuration */
		if (bus >= 256 || dev >= 32 || func >= 8 || reg >= 256)
			return 0xffffffff;
		EAGLE_REV1_OFF;
		writel(1 | (reg & 0xfc) | (func << 8) | (dev << 11) | (bus << 16),
		       VR4122_PCICONFAREG);
	}

 
	config = readl(VR4122_PCICONFDREG);
	EAGLE_REV1_ON;
	return config;
}

static void WriteConfig32(int bus, int dev, int func, int reg, unsigned long data)
{
	if (bus == 0) {
		/* type 0 configuration */
		if (dev<11 || dev>=32 || func>=8 || reg>=256)
			return;
		EAGLE_REV1_ON;
		writel((reg & 0xfc) | (func << 8) | (1 << dev),VR4122_PCICONFAREG);
	}
	else {
		/* type 1 configuration */
		if (bus>=256 || dev>=32 || func>=8 || reg>=256)
			return;
		EAGLE_REV1_OFF;
		writel(1 | (reg & 0xfc) | (func << 8) | (dev << 11) | (bus << 16),
		       VR4122_PCICONFAREG);
	}

	writel(data, VR4122_PCICONFDREG);
	EAGLE_REV1_ON;
}

static void map_resource(int bus, int dev, int func, int nbase)
{
	unsigned long val, reg, next, req, base, size;

	/* Scan all base registers */
	next = 0x10;
	while (next < 0x10 + nbase * 4) {
		reg = next;
		next = reg + 4;
		WriteConfig32(bus, dev, func, reg, 0xFFFFFFFF);
		val = ReadConfig32(bus, dev, func, reg);
		if (val & 1) {
			req = ~(val & 0xFFFFFFFC)+1;
			if (req & (req - 1))
				continue;
			if (req == 0)
				continue;
			size = req > 0x1000 ? req : 0x1000;
			base = (iostart + size - 1) & ~(size - 1);
			if ((base + size - 1) > iolimit)
				continue;
			iostart = base + size;
		}
		else {
			req = ~(val & 0xFFFFFFF0) + 1;
			if (req & (req - 1))
				continue;
			if (req == 0)
				continue;
			switch (val & 0x00000006) {
			case 0:
				break;
			case 2:
				continue;
			case 4:
				WriteConfig32(bus, dev, func, reg + 4, 0);
				next += 4;
				break;
			case 6:
				continue;
			}
			size = req > 0x1000 ? req : 0x1000;
			base = (memstart + size - 1) & ~(size - 1);
			if ((base + size - 1) > memlimit)
				continue;
			memstart = base + size;
		}
		WriteConfig32(bus, dev, func, reg, base);
	}
}

static int __init eagle_pcidev_init(int bus, int dev, int func)
{
	u32 id, status, bhlc;
	int nbase = 0;

	/* Probe device */
	writel(0xff00ff00, VR4122_INTCNTSTAREG);
	id = ReadConfig32(bus, dev, func, PCI_VENDOR_ID);
	status = readl(VR4122_INTCNTSTAREG);
	writel(0xff00ff00, VR4122_INTCNTSTAREG);

	if (status & 0x000000ff)
		return 1;
	if ((id & 0xffff) == 0xffff)
		return 1;
	if (pcidevs >= MAX_PCI_DEV)
		return 1;
	pcidevs++;

	bhlc = ReadConfig32(bus, dev, func, PCI_CACHE_LINE_SIZE);
	switch ((bhlc >> 16) & 0x7f) {
	case 0:
		nbase = 6;
		break;
	case 1:
		nbase = 2;
		break;
	case 2:
		nbase = 1;
		break;
	default:
	}

	map_resource(bus, dev, func, nbase);

	/* Set master latency timer and cache line size */
	WriteConfig32(bus, dev, func, PCI_CACHE_LINE_SIZE, 0x8004);

	/* Enable device and clear all status */
	WriteConfig32(bus, dev, func, PCI_COMMAND,
	              ReadConfig32(bus, dev, func, PCI_COMMAND) | 0xffff0000 |
	              PCI_COMMAND_IO | PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER |
	              PCI_COMMAND_SERR);

	return 0;
}

static void __init eagle_pcibus_init(int bus)
{
	int dev, func;
	unsigned long type;

	for (dev = 0; dev < 32; dev++) {
		for (func = 0; func < 8; func++) {
			if (eagle_pcidev_init(bus, dev, func)) {
				type = ReadConfig32(bus, dev, func,
				                    PCI_CACHE_LINE_SIZE);
				if ((type & 0x00800000) == 0)
					break;
			}
		}
	}
}

void __init pcibios_update_irq(struct pci_dev *dev, int irq)
{
	/* assume it's a generic PCI interrupt */
	dev->irq = VR4122_IRQ_PCI;

	/* if it's wired differently, correct the assumption */
	switch (dev->vendor) {
	case PCI_VENDOR_ID_NEC:
		switch (dev->device) {
		case PCI_DEVICE_ID_NEC_VRC4173_BCU:
		case PCI_DEVICE_ID_NEC_VRC4173_AC97:
		case PCI_DEVICE_ID_NEC_VRC4173_CARDU:
		case PCI_DEVICE_ID_NEC_VRC4173_USB: /* all three share the same IRQ */
			dev->irq = VR4122_IRQ_VRC4173;
			break;
		}
		break;
	case  PCI_VENDOR_ID_MEDIAQ:
		if (dev->device == PCI_DEVICE_ID_MEDIAQ_MQ200)
			dev->irq = VR4122_IRQ_MQ200;
		break;
	}
}

static void __init pcibios_fixup_irqs(void)
{
	struct pci_dev *dev;
	int slot_num;
	int cnt = 0;

	pci_for_each_dev(dev) {
		slot_num = PCI_SLOT(dev->devfn);
		pcibios_update_irq(dev, cnt++);
	}
}

void __init pcibios_fixup_bus(struct pci_bus *b)
{
}

void pcibios_update_resource(struct pci_dev *dev, struct resource *root,
                             struct resource *res, int resource)
{
}

void __init pcibios_init(void)
{
	vr4122_pcibios_init();

	ioport_resource.start = 0; 
	ioport_resource.end = VR4122_PCI_IO_SIZE;
	iomem_resource.start = VR4122_PCI_MEM_BASE;
	iomem_resource.end = VR4122_PCI_MEM_BASE + VR4122_PCI_MEM_SIZE;

	eagle_pcibus_init(0);

	(void)pci_scan_bus(0, &vr4122_pci_ops, NULL);
	pcibios_fixup_irqs();
}

int __init pcibios_enable_device(struct pci_dev *dev)
{
	/* Not needed, sine we enable all devices at startup */
	return 0;
}

void __init pcibios_align_resource(void *data, struct resource *res,
                                   unsigned long size)
{
	/* Nothing to do. */
}

char * __init pcibios_setup(char *str)
{
	/* Nothing to do for now. */
	return str;
}

struct pci_fixup pcibios_fixups[] = { { 0 } };
