/*
 * BRIEF MODULE DESCRIPTION
 *	Setup for NEC Eagle board.
 *
 * Copyright 2001 MontaVista Software Inc.
 * Author: Yoichi Yuasa
 *		yyuasa@mvista.com or source@mvista.com
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/init.h>
#include <linux/console.h>

#include <asm/addrspace.h>
#include <asm/io.h>
#include <asm/reboot.h>
#include <asm/time.h>
#include <asm/vr4122/vr4122.h>
#include <asm/vr4122/eagle.h>

extern void vr4122_restart(char *command);
extern void vr4122_halt(void);
extern void vr4122_power_off(void);

extern void vr4122_time_init(void);
extern void vr4122_timer_setup(struct irqaction *irq);

void __init nec_eagle_setup(void)
{
	unsigned long clock;
	unsigned short val;

	val = readw(VR4122_CLKSPEEDREG);
	clock = (18432000 * 98) / (val & 0x1f);
	printk("PClock: %ldHz\n", clock);

	clock = clock / ((val & 0x700) >> 8);
	printk("VTClock: %ldHz\n", clock);

	clock = clock / (2 << ((val & 0x1000) >> 12));
	printk("TClock: %ldHz\n", clock);

        mips_counter_frequency = clock / 4;

	mips_io_port_base = VR4122_IO_PORT_BASE;

	_machine_restart = vr4122_restart;
	_machine_halt = vr4122_halt;
	_machine_power_off = vr4122_power_off;

	board_time_init = vr4122_time_init;
	board_timer_setup = vr4122_timer_setup;

#ifdef CONFIG_FB
	conswitchp = &dummy_con;
#endif

	/* Select RS-232C */
	val = readw(VR4122_SIUIRSEL);
	val &= ~VR4122_SIUIRSEL_SIRSEL;
	writew(val, VR4122_SIUIRSEL);

	/* Supply DSIU and SIU clock */
	vr4122_clock_supply(VR4122_CMUCLKMSK_MSKSIU);
	vr4122_clock_supply(VR4122_CMUCLKMSK_MSKDSIU);
	vr4122_clock_supply(VR4122_CMUCLKMSK_MSKSSIU);
}
