/*
 * Copyright 1999, 2000 Zereau Ltd.
 * Author: Alex Holden <alex@linuxhacker.org>
 */

#define TFTP_SERVPORT	69		/* Tftp server port no. */
#define TFTP_XFER	"octet"		/* Transfer mode to request */

/* Timeouts: */
/* NB. These values were pretty much picked using nothing more than educated
   guesswork and experimentation. They seem to work Ok for me but YMMV. */
#define TFTP_RRQ_TIMEOUT (HZ * 5)	/* Wait 5 seconds for server response */
#define TFTP_PACKET_TIMEOUT (HZ)	/* 1 second before resend ack */
#ifdef ISHONI_ENGINE
#define TFTP_RRQ_MAX	5		/* Retry 10 times before giving up */
#else
#define TFTP_RRQ_MAX	10		/* Retry 10 times before giving up */
#endif
#define TFTP_RESEND_MAX	30		/* Resend 30 times before giving up */

/* Packet types: */
#define TFTP_RRQ	1		/* Read Request */
#define TFTP_WRQ	2		/* Write Request */
#define TFTP_DATA	3		/* Data */
#define TFTP_ACK	4		/* Acknowledgement */
#define TFTP_ERROR	5		/* Error */

/* Error codes: */
#define TFTP_EUNDEF	0	/* Not defined, see error message (if any) */
#define	TFTP_ENOTFOUND	1	/* File Not Found */
#define TFTP_EACCESS	2	/* Access violation */
#define TFTP_ENOSPACE	3	/* Disk full or allocation exceeded */
#define TFTP_EBADOP	4	/* Illegal TFTP operation */
#define TFTP_EBADID	5	/* Unknown transfer ID */
#define TFTP_EEXISTS	6	/* File already exists */
#define TFTP_ENOUSER	7	/* No such user */

/* C structure representation of various tftp headers: */
struct tftp_rqhdr {
	u16 opcode __attribute__ ((packed));
	u8 data[0];
};
struct tftp_datahdr {
	u16 opcode __attribute__ ((packed));
	u16 seqno __attribute__ ((packed));
	u8 data[0];
};
struct tftp_ackhdr {
	u16 opcode __attribute__ ((packed));
	u16 seqno __attribute__ ((packed));
};
struct tftp_errhdr {
	u16 opcode __attribute__ ((packed));
	u16 errno __attribute__ ((packed));
	char msg[0];
};

/* Array sizes: */
#define TFTP_INBUFLEN	516	/* The size of tftpstate->inbuf */
#define TFTP_OUTBUFLEN	29	/* The size of tftpstate->outbuf */
#ifdef ISHONI_ENGINE
#define TFTP_FILELEN	128	/* The maximum length of the filename string */
#else
#define TFTP_FILELEN	65	/* The maximum length of the filename string */
#endif
#define TFTP_SERVERNAMELEN 16	/* The maximum length of the server IP string */

/* Tftp state structure: */
struct initrd_tftp_state {
	u8 inbuf[TFTP_INBUFLEN];	/* Input buffer space */
	u8 outbuf[TFTP_OUTBUFLEN];	/* Output buffer space */
#ifndef ISHONI_ENGINE
	int retries;			/* How many times to retry */
#else
	unsigned int retries;   /* How many times to retry */
#endif
	u32 serverip;			/* Server IP in network form */
	int inbuflen;			/* The number of bytes in inbuf */
	int outbuflen;			/* The number of bytes in outbuf */
	int finished;			/* "finished receiving data" flag */
	int errno;			/* The last error No. we got */
	struct file ramdisk;		/* Used for writing to the ramdisk */
#ifdef ISHONI_ENGINE
#ifdef LINUX_BOOT_LOADER
	int ramfsFd;		/* File descriptor for  ram file */
#endif
#endif
	struct inode *ramdisk_inode;	/* Ditto */
	struct dentry ramdisk_dentry;	/* Ditto */
	struct socket *sock;		/* The socket we converse over */
	struct timer_list timer;	/* The timeout timer */
	u16 serverport;			/* The server port number (net order) */
	u16 seqno;			/* Where we are in the sequence */
	u16 lastport;			/* The port the last packet came from */
#ifdef CONFIG_BLK_DEV_INITRD_TFTP_GZIP
	int gzipped;			/* Whether we need to gunzip the file */
#endif
};

/* Function prototypes: */
static int bindsocket(struct initrd_tftp_state *tftpstate) __init;
static void handleerror(struct initrd_tftp_state *tftpstate) __init;
static void timeout(unsigned long pid) __init;
static int sendpacket(struct initrd_tftp_state *tftpstate) __init;
static int sendrequest(struct initrd_tftp_state *tftpstate) __init;
static int recvpacket(struct initrd_tftp_state *tftpstate) __init;
static int sendack(struct initrd_tftp_state *tftpstate) __init;
static int senderr(struct initrd_tftp_state *tftpstate) __init;
static void handleerrorpacket(struct initrd_tftp_state *tftpstate) __init;
static int handledatapacket(struct initrd_tftp_state *tftpstate) __init;
static int handlepacket(struct initrd_tftp_state *tftpstate) __init;
static int getpacket(struct initrd_tftp_state *tftpstate) __init;
static int waitforresponse(struct initrd_tftp_state *tftpstate) __init;
static int check_for_gzip(struct initrd_tftp_state *tftpstate) __init;
static int mainreceiveloop(struct initrd_tftp_state *tftpstate) __init;
static int openramdisk(struct initrd_tftp_state *tftpstate) __init;
#ifndef ISHONI_ENGINE
int initrd_tftp_load(void) __init;
#else
#ifdef LINUX_BOOT_LOADER
int initrd_tftp_load(char *file) __init;
#endif
#endif
int initrd_tftp_setup(char *str) __init;
#ifdef CONFIG_BLK_DEV_INITRD_TFTP_GZIP
static void *malloc(int size) __init;
static void free(void *where) __init;
static void gzip_mark(void **) __init;
static void gzip_release(void **) __init;
static int fill_inbuf(void) __init;
static void flush_window(void) __init;
static void error(char *msg) __init;
static int start_gunzip(struct initrd_tftp_state *tftpstate) __init;
#endif
