/*
 * cpx1024.h
 *
 * Driver to control access to the hardware monitor on the CPX1024 Quad
 * Xeon processor machine built by Diversified Technology Inc.
 * 
 * For information on programming this driver, refer to
 * Documentation/cpx1024.txt in the Linux kernel source code.
 *
 * Author: Johnnie Peters
 *         jpeters@mvista.com
 *
 * Copyright 2000 MontaVista Softare Inc.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR   IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* ISA Command and Data port values */
#define DTIMON_DATA 		0x132
#define DTIMON_CMD  		0x133

/* Monitor Status definitions */
#define DTIMON_STATUS_OBF	  0x1	/* Status Output Buffer Full */
#define DTIMON_STATUS_IBE	  0x2	/* Status Input Buffer Empty */
#define DTIMON_STATUS_CBE	  0x4	/* Status Command Buffer Empty */
#define DTIMON_STATUS_INTP	  0x80	/* Status Interrupt Pending */

/* Command Values
 * These Values come directly from the users manual for the hardware monitor
 * on the cpx1024. 
 */
#define DTIMON_CMD_GET_STATUS	(unsigned char)0x00 /* System Status */
#define 	DTIMON_STATUS_VOLT		(unsigned char)0x01	/* Volt Alarm */
#define 	DTIMON_STATUS_ONBT		(unsigned char)0x02	/* On Board Temp */
#define 	DTIMON_STATUS_OFFBT		(unsigned char)0x04	/* Off Board Temp */
#define 	DTIMON_STATUS_FANS		(unsigned char)0x08	/* Fans Alarm */
#define 	DTIMON_STATUS_GPI		(unsigned char)0x10	/* GPI Alarm */
#define 	DTIMON_STATUS_CPU		(unsigned char)0x20	/* CPU or Power */
#define 	DTIMON_STATUS_MINOR		(unsigned char)0x40	/* Minor Indicat */
#define 	DTIMON_STATUS_MAJOR		(unsigned char)0x80	/* Major Indicat */
#define DTIMON_CMD_GET_VOLTAGE	(unsigned char)0x01 /* Voltage Status */
#define 	DTIMON_VOLTS_PLUS5		(unsigned char)0x01	/* +5 Alarm */
#define 	DTIMON_VOLTS_PLUS2_5	(unsigned char)0x02	/* +2.5 Alarm */
#define 	DTIMON_VOLTS_PLUS12		(unsigned char)0x04	/* +12 Alarm */
#define 	DTIMON_VOLTS_MINUS12	(unsigned char)0x08	/* -12 Alarm */
#define 	DTIMON_VOLTS_PLUS3_3	(unsigned char)0x10	/* +3.3 Alarm */
#define 	DTIMON_VOLTS_CPU		(unsigned char)0x20	/* CPU Alarm */
#define 	DTIMON_VOLTS_VTT		(unsigned char)0x40	/* VTT Module */
#define 	DTIMON_VOLTS_CPU1		(unsigned char)0x01	/* Alarm on CPU1 */
#define 	DTIMON_VOLTS_CPU2		(unsigned char)0x02	/* Alarm on CPU2 */
#define 	DTIMON_VOLTS_CPU3		(unsigned char)0x04	/* Alarm on CPU3 */
#define 	DTIMON_VOLTS_CPU4		(unsigned char)0x08	/* Alarm on CPU4 */
#define DTIMON_CMD_GET_FAN		(unsigned char)0x02 /* Fan Status */
#define 	DTIMON_FAN_1			(unsigned char)0x01	/* Fan 1 failed */
#define 	DTIMON_FAN_2			(unsigned char)0x02	/* Fan 2 failed */
#define 	DTIMON_FAN_3			(unsigned char)0x04	/* Fan 3 failed */
#define 	DTIMON_FAN_4			(unsigned char)0x08	/* Fan 4 failed */
#define 	DTIMON_FAN_5			(unsigned char)0x10	/* Fan 5 failed */
#define 	DTIMON_RAN_6			(unsigned char)0x20	/* Fan 6 failed */
#define DTIMON_CMD_GET_GPI		(unsigned char)0x03 /* GPI Status */
#define 	DTIMON_GPI_1			(unsigned char)0x01	/* GPI 1 failed */
#define 	DTIMON_GPI_2			(unsigned char)0x03	/* GPI 2 failed */
#define 	DTIMON_GPI_3			(unsigned char)0x04	/* GPI 3 failed */
#define 	DTIMON_GPI_4			(unsigned char)0x08	/* GPI 4 failed */
#define 	DTIMON_GPI_5			(unsigned char)0x10	/* GPI 5 failed */
#define 	DTIMON_GPI_6			(unsigned char)0x20	/* GPI 6 failed */
#define 	DTIMON_GPI_7			(unsigned char)0x40	/* GPI 7 failed */
#define 	DTIMON_GPI_8			(unsigned char)0x80	/* GPI 8 failed */
#define DTIMON_CMD_GET_TEMP		(unsigned char)0x04 /* Temperature Status */
#define 	DTIMON_TEMP_ALARMB		(unsigned char)0x01	/* Alarm Board */
#define 	DTIMON_TEMP_CPU1		(unsigned char)0x01	/* CPU1 temp */
#define 	DTIMON_TEMP_CPU2		(unsigned char)0x04	/* CPU2 temp */
#define 	DTIMON_TEMP_CPU3		(unsigned char)0x08	/* CPU3 temp */
#define 	DTIMON_TEMP_CPU4		(unsigned char)0x10	/* CPU4 temp */
#define 	DTIMON_TEMP_IOB			(unsigned char)0x20	/* I/O Board temp */
#define 	DTIMON_TEMP_SYSTEM		(unsigned char)0x40	/* System temp */
#define 	DTIMON_TEMP_SUPPLY		(unsigned char)0x80	/* Supply temp */
#define 	DTIMON_TEMP_POWER		(unsigned char)0x01	/* Power supply */
#define 	DTIMON_TEMP_REMOTE1		(unsigned char)0x02	/* Remote 1 temp */
#define 	DTIMON_TEMP_REMOTE2		(unsigned char)0x04	/* Remote 2 temp */
#define 	DTIMON_TEMP_REMOTE3		(unsigned char)0x08	/* Remote 3 temp */
#define DTIMON_CMD_SET_PASSWD	(unsigned char)0x05 /* Set Password */
#define DTIMON_CMD_SET_UNITID	(unsigned char)0x07 /* Set Uit ID */
#define DTIMON_CMD_SERIAL_ECHO	(unsigned char)0x08 /* Toggle Serial Echo */
#define 	DTIMON_ECHO_OFF		        (unsigned char)0x00	/* echo off */
#define 	DTIMON_ECHO_ON		        (unsigned char)0x01	/* echo on */
#define DTIMON_CMD_VERBOSE	(unsigned char)0x09 /* Toggle Non Verbos Mode */
#define 	DTIMON_VERBOSE_OFF	        (unsigned char)0x00	/* echo off */
#define 	DTIMON_VERBOSE_ON	        (unsigned char)0x01	/* echo on */
#define DTIMON_CMD_WATCHDOG	(unsigned char)0x0a /* Enable Watchdog */
#define DTIMON_CMD_WWARNCOUNT	(unsigned char)0x0b /* Watchdog Warning Count */
#define DTIMON_CMD_STROBEWATCH	(unsigned char)0x0c /* Strobe Watchdog */
#define DTIMON_CMD_SET_BIOSFAIL	(unsigned char)0x0d /* Set Bios Fail Def */
#define DTIMON_CMD_GET_BIOSFAIL	(unsigned char)0x0e /* Get Bios Fail Def */
#define DTIMON_CMD_LAST_POST	(unsigned char)0x0f /* Get Last Post Code */
#define DTIMON_CMD_SET_LOWTEMP	(unsigned char)0x10 /* Set Low Boundary Temp */
#define 	DTIMON_SGT_ALARMB		(unsigned char)0x00	/* Alarm Board */
#define 	DTIMON_SGT_CPU1			(unsigned char)0x01	/* CPU Temp 1 */
#define 	DTIMON_SGT_CPU2			(unsigned char)0x02	/* CPU Temp 2 */
#define 	DTIMON_SGT_CPU3			(unsigned char)0x03	/* CPU Temp 3 */
#define 	DTIMON_SGT_CPU4			(unsigned char)0x04	/* CPU Temp 4 */
#define 	DTIMON_SGT_IOB			(unsigned char)0x05	/* I/O Board */
#define 	DTIMON_SGT_SYSTEM1		(unsigned char)0x06	/* System Temp 1 */
#define 	DTIMON_SGT_SYSTEM2		(unsigned char)0x07	/* System Temp 2 */
#define 	DTIMON_SGT_POWER		(unsigned char)0x08	/* Power Supply */
#define 	DTIMON_SGT_REM1			(unsigned char)0x09	/* Remote Temp 1 */
#define 	DTIMON_SGT_REM2			(unsigned char)0x0a	/* Remote Temp 2 */
#define 	DTIMON_SGT_REM3			(unsigned char)0x0b	/* Remote Temp 3 */
#define DTIMON_CMD_GET_LOWTEMP	(unsigned char)0x11 /* Get Low Boundary Temp */
	    /* Second byte on DTIMON_CMD_GET_LOWTEMP use DTIMON_CMD_SET_LOWTEMP */
#define DTIMON_CMD_SET_HIGHTEMP	(unsigned char)0x12 /* Set High Boundary Temp */
	    /* Second byte on DTIMON_CMD_SET_HIGHTEMP use DTIMON_CMD_SET_LOWTEMP */
#define DTIMON_CMD_GET_HIGHTEMP	(unsigned char)0x13 /* Soft Reset of Monitor */
	    /* Second byte on DTIMON_CMD_GET_HIGHTEMP use DTIMON_CMD_SET_LOWTEMP */
#define DTIMON_CMD_GET_TEMPVAL	(unsigned char)0x14 /* Soft Reset of Monitor */
	    /* Second byte on DTIMON_CMD_GET_TEMP use DTIMON_CMD_SET_LOWTEMP */
#define DTIMON_CMD_SET_LEDFLASH (unsigned char)0x16 /* Set Led Flash Rate */
#define 	DTIMON_LED_ACTIVE		(unsigned char)0x00	/* Active Service */
#define 	DTIMON_LED_OOS			(unsigned char)0x01	/* Out of Service */
#define DTIMON_CMD_CPU_OVERHEAT	(unsigned char)0x17 /* CPU Overheat Signals */
#define 	DTIMON_OH_CPU1			(unsigned char)0x01	/* CPU1 Signal */
#define 	DTIMON_OH_CPU2			(unsigned char)0x02	/* CPU1 Signal */
#define 	DTIMON_OH_CPU3			(unsigned char)0x04	/* CPU1 Signal */
#define 	DTIMON_OH_CPU4			(unsigned char)0x08	/* CPU1 Signal */
#define DTIMON_CMD_SOFTRESET	(unsigned char)0x18 /* Soft Reset of Monitor */
#define DTIMON_CMD_SET_7SEGLED	(unsigned char)0x19 /* Set 7 Segment LED Disp */
#define DTIMON_CMD_ENABLE_TTM	(unsigned char)0x1a /* Enable Term Tran Mode */
#define DTIMON_CMD_DISABLE_TTM	(unsigned char)0x1b /* Disable Term Tran Mode */
#define DTIMON_CMD_FORCE_GPO	(unsigned char)0x1c /* Force Gen Purpose Outp */
#define 	DTIMON_FGPO_LOW			(unsigned char)0x00	/* Low State */
#define 	DTIMON_FGPO_HIGH		(unsigned char)0x01	/* High State */
#define DTIMON_CMD_ACTIVE_LED	(unsigned char)0x1d /* Set Active Servce Led */
#define 	DTIMON_LED_OFF			(unsigned char)0x00	/* Led turned off */
#define 	DTIMON_LED_ON			(unsigned char)0x01	/* Led turned on */
#define 	DTIMON_LED_FLASH		(unsigned char)0x03	/* Led flashing */
#define DTIMON_CMD_OOS_LED		(unsigned char)0x1e /* Set Out of Service Led */
		/* Second byte on DTIMON_CMD_OOS_LED use DTIMON_CMD_ACTIVE_LED values */
#define DTIMON_CMD_POWERDOWN	(unsigned char)0x1f /* Power Down System */
#define DTIMON_CMD_GET_VOLT		(unsigned char)0x20 /* Get Voltage for Chan */
#define 	DTIMON_GVOLT_PLUS5		(unsigned char)0x00	/* +5 Voltage */
#define 	DTIMON_GVOLT_PLUS2_5	(unsigned char)0x01	/* +5 Voltage */
#define 	DTIMON_GVOLT_PLUS12		(unsigned char)0x02	/* +2.5 Voltage */
#define 	DTIMON_GVOLT_MINUS12	(unsigned char)0x03	/* +12 Voltage */
#define 	DTIMON_GVOLT_PLUS3_3	(unsigned char)0x04	/* -12 Voltage */
#define 	DTIMON_GVOLT_VTT		(unsigned char)0x05	/* VTT Voltage */
#define 	DTIMON_GVOLT_CPU1_CORE	(unsigned char)0x06	/* CPU1 Core */
#define 	DTIMON_GVOLT_CPU2_CORE	(unsigned char)0x07	/* CPU2 Core */
#define 	DTIMON_GVOLT_CPU3_CORE	(unsigned char)0x08	/* CPU3 Core */
#define 	DTIMON_GVOLT_CPU4_CORE	(unsigned char)0x09	/* CPU4 Core */
#define 	DTIMON_GVOLT_CPU1_LOG	(unsigned char)0x0a	/* CPU1 Logic */
#define 	DTIMON_GVOLT_CPU2_LOG	(unsigned char)0x0b	/* CPU2 Logic */
#define 	DTIMON_GVOLT_CPU3_LOG	(unsigned char)0x0c	/* CPU3 Logic */
#define 	DTIMON_GVOLT_CPU4_LOG	(unsigned char)0x0d	/* CPU4 Logic */
#define DTIMON_CMD_GET_VTOLER	(unsigned char)0x21 /* Get Voltage Tolerance */
#define 	DTIMON_TOLER_PLUS5		(unsigned char)0x00	/* +5 Voltage */
#define 	DTIMON_TOLER_PLUS2_5	(unsigned char)0x01	/* +2.5 Voltage */
#define 	DTIMON_TOLER_PLUS12		(unsigned char)0x02	/* +12 Voltage */
#define 	DTIMON_TOLER_MINUS12	(unsigned char)0x03	/* -12 Voltage */
#define 	DTIMON_TOLER_PLUS3_3	(unsigned char)0x04	/* +3.3 Voltage */
#define 	DTIMON_TOLER_VTT		(unsigned char)0x05	/* +CPU or VTT */
#define DTIMON_CMD_SET_VTOLER	(unsigned char)0x22 /* Set Voltage Tolerance */
		/* Second byte on DTIMON_CMD_SET_VTOLER use DTIMON_CMD_GET_VTOLER val */
#define DTIMON_CMD_VRM_STATUS	(unsigned char)0x23 /* VRM Module Status */
#define DTIMON_CMD_VRM_IDBITS	(unsigned char)0x24 /* VRM Voltage ID Bits */
#define DTIMON_CMD_SET_FANMODE	(unsigned char)0x25 /* Set Fan Mode */
#define 	DTIMON_GSFAN_1			(unsigned char)0x00	/* Fan #1 */
#define 	DTIMON_GSFAN_2			(unsigned char)0x01	/* Fan #2 */
#define 	DTIMON_GSFAN_3			(unsigned char)0x02	/* Fan #3 */
#define 	DTIMON_GSFAN_4			(unsigned char)0x03	/* Fan #4 */
#define 	DTIMON_GSFAN_5			(unsigned char)0x04	/* Fan #5 */
#define 	DTIMON_GSFAN_6			(unsigned char)0x05	/* Fan #6 */
#define 	DTIMON_GSFAN_STROBE		(unsigned char)0x00	/* Strobe mode */
#define 	DTIMON_GSFAN_LEVEL		(unsigned char)0x01	/* Level mode */
#define 	DTIMON_GSFAN_DISABLE		(unsigned char)0x02	/* Fan disabled */
#define DTIMON_CMD_SET_MINRPS	(unsigned char)0x26 /* Set Fan Min RPS val */
		/* Use fan number values from DTIMON_CMD_SET_FANMODE for MINRPS */
#define 	DTIMON_FAN_LOW			(unsigned char)0x00	/* Fan On Low */
#define 	DTIMON_FAN_HIGH			(unsigned char)0x01	/* Fan at High */
#define DTIMON_CMD_GET_MINRPS	(unsigned char)0x27 /* Get Fan Min RPS val */
		/* Use fan number values from DTIMON_CMD_SET_FANMODE for MINRPS */
#define DTIMON_CMD_GET_FANSTAT	(unsigned char)0x28 /* Get Fan status */
		/* Use values above */
#define DTIMON_CMD_GET_FANMODE	(unsigned char)0x29 /* Get Fan status */
		/* Use values from DTIMON_CMD_SET_FANMODE */
#define DTIMON_CMD_SET_GPIPOL	(unsigned char)0x2a /* Set GPI Polarity */
#define 	DTIMON_SGGPI_0			(unsigned char)0x00	/* GPI Number 1 */
#define 	DTIMON_SGGPI_1			(unsigned char)0x01	/* GPI Number 2 */
#define 	DTIMON_SGGPI_2			(unsigned char)0x02	/* GPI Number 3 */
#define 	DTIMON_SGGPI_3			(unsigned char)0x03	/* GPI Number 4 */
#define 	DTIMON_SGGPI_4			(unsigned char)0x04	/* GPI Number 5 */
#define 	DTIMON_SGGPI_5			(unsigned char)0x05	/* GPI Number 6 */
#define 	DTIMON_SGGPI_6			(unsigned char)0x06	/* GPI Number 7 */
#define 	DTIMON_SGGPI_7			(unsigned char)0x07	/* GPI Number 8 */
#define 	DTIMON_SGGPI_LOW		(unsigned char)0x00	/* GPI Low */
#define 	DTIMON_SGGPI_HIGH		(unsigned char)0x01	/* GPI High */
#define 	DTIMON_SGGPI_DIABLE	(unsigned char)0x02	/* GPI Disable */
#define DTIMON_CMD_GET_GPIPOL	(unsigned char)0x2b /* Set GPI Polarity */
		/* Use values from DTIMON_CMD_SET_GPIPOL */
#define DTIMON_CMD_GET_GPIX		(unsigned char)0x2c /* Set GPI Polarity */
		/* Use values from DTIMON_CMD_SET_GPIPOL */
#define DTIMON_CMD_SET_GPOPOL	(unsigned char)0x2d /* Set GPO Polarity */
#define 	DTIMON_GPO_0			(unsigned char)0x00	/* GPO Number 1 */
#define 	DTIMON_GPO_1			(unsigned char)0x01	/* GPO Number 2 */
#define 	DTIMON_GPO_2			(unsigned char)0x02	/* GPO Number 3 */
#define 	DTIMON_GPO_3			(unsigned char)0x03	/* GPO Number 4 */
#define 	DTIMON_GPO_LOW			(unsigned char)0x00	/* Normally Low */
#define 	DTIMON_GPO_HIGH			(unsigned char)0x01	/* Normally High */
#define 	DTIMON_GPO_FLOW			(unsigned char)0x02	/* Forced Low */
#define 	DTIMON_GPO_FHIGH		(unsigned char)0x03	/* Forced High */
#define DTIMON_CMD_GET_GPOPOL	(unsigned char)0x2e /* Get GPO Polarity */
		/* Use values from DTIMON_CMD_SET_GPOPOL */
#define DTIMON_CMD_MODIFY_POST	(unsigned char)0x2f /* Post Code En/Disable */
#define DTIMON_CMD_SET_DELIVERY	(unsigned char)0x30 /* Set Delivery Byte */
#define 	DTIMON_DELIV_VOLTAGE	(unsigned char)0x00	/* Voltage Deliv */
#define 	DTIMON_DELIV_WATCHD		(unsigned char)0x01	/* Watchdog */
#define 	DTIMON_DELIV_FAN		(unsigned char)0x02	/* Fan Deliv */
#define DTIMON_CMD_GET_DELIVERY	(unsigned char)0x31 /* Set Delivery Byte */
		/* Use values from DTIMON_CMD_SET_DELIVERY */
#define DTIMON_CMD_SET_GPIDEL	(unsigned char)0x32 /* Set GPI Delivery Byte */
		/* Use gpi numbergs from DTIMON_CMD_SET_GPIPOL */
#define DTIMON_CMD_GET_GPIDEL	(unsigned char)0x33 /* Get GPI Delivery Byte */
		/* Use gpi numbergs from DTIMON_CMD_SET_GPIPOL */
#define DTIMON_CMD_SET_TEMPDEL	(unsigned char)0x34 /* Set Temp Delivery Byte */
#define 	DTIMON_DELIV_ALARMB		(unsigned char)0x00	/* Alarm Board Temp */
#define 	DTIMON_DELIV_CPU		(unsigned char)0x01	/* CPU Temp */
#define 	DTIMON_DELIV_SYSTEM		(unsigned char)0x02	/* System Temp */
#define 	DTIMON_DELIV_REMOTE		(unsigned char)0x03	/* Remote Temp */
#define DTIMON_CMD_GET_TEMPDEL	(unsigned char)0x35 /* Get Temp Delivery Byte */
		/* Use values from DTIMON_CMD_SET_TEMPDEL */
#define DTIMON_CMD_SET_WDGRAN	(unsigned char)0x36 /* Set Watchd Granularity */
#define DTIMON_CMD_GET_WDGRAN	(unsigned char)0x37 /* Get Watchd Granularity */
#define DTIMON_CMD_GET_WDRESET	(unsigned char)0x38 /* Get Watchdog Reset */
#define DTIMON_CMD_CLR_WDRESET	(unsigned char)0x39 /* Clear Watchdog Reset */
#define DTIMON_CMD_BP_TEST		(unsigned char)0x3a /* Run Backplane Test */
#define DTIMON_CMD_SET_WDMAX	(unsigned char)0x3c /* Set Watchd Max Resets */
#define DTIMON_CMD_GET_WDMAX	(unsigned char)0x3d /* Get Watchd Max Resets */
#define DTIMON_CMD_GET_XEONROM	(unsigned char)0x40 /* Read Xeon Proc Rom */
#define DTIMON_CMD_SET_XEONROM	(unsigned char)0x41 /* Write Xeon Proc Rom */
#define DTIMON_CMD_GET_MONROM	(unsigned char)0x42 /* Read Monitor EEProm */
#define DTIMON_CMD_SET_MONROM	(unsigned char)0x43 /* Write Monitor EEProm */
#define DTIMON_CMD_GET_TIBM		(unsigned char)0x45 /* Temp Install Bit Mask */
#define 	DTIMON_TIBM_SEL0		(unsigned char)0x00	/* Section Selector 0 */
#define 	DTIMON_TIBM_SEL1		(unsigned char)0x01	/* Section Selector 1 */
#define 	DTIMON_TIBM_MEMB1		(unsigned char)0x01	/* Memory Board 1 */
#define 	DTIMON_TIBM_MEMB2		(unsigned char)0x02	/* Memory Board 2 */
#define 	DTIMON_TIBM_IOB			(unsigned char)0x04	/* I/O Board */
#define 	DTIMON_TIBM_ALARMB		(unsigned char)0x08	/* Alarm Board */
#define 	DTIMON_TIBM_POWER		(unsigned char)0x10	/* Power Supply */
#define 	DTIMON_TIBM_REM1		(unsigned char)0x20	/* Remote 1 */
#define 	DTIMON_TIBM_REM2		(unsigned char)0x40	/* Remote 2 */
#define 	DTIMON_TIBM_REM3		(unsigned char)0x80	/* Remote 3 */
#define 	DTIMON_TIBM_CPU1		(unsigned char)0x01	/* CPU 1 */
#define 	DTIMON_TIBM_CPU2		(unsigned char)0x02	/* CPU 2 */
#define 	DTIMON_TIBM_CPU3		(unsigned char)0x04	/* CPU 3 */
#define 	DTIMON_TIBM_CPU4		(unsigned char)0x08	/* CPU 4 */
#define DTIMON_CMD_SET_EEPROMM	(unsigned char)0x46 /* Set Changes In Prom */
#define DTIMON_CMD_GET_CPUMASK	(unsigned char)0x47 /* CPUs in system */
#define 	DTIMON_MASK_CPU1		(unsigned char)0x01	/* CPU 1 */
#define 	DTIMON_MASK_CPU2		(unsigned char)0x02	/* CPU 1 */
#define 	DTIMON_MASK_CPU3		(unsigned char)0x04	/* CPU 1 */
#define 	DTIMON_MASK_CPU4		(unsigned char)0x08	/* CPU 1 */
#define DTIMON_CMD_HARDRESET	(unsigned char)0x48 /* Hard Reset of Monitor */
#define DTIMON_CMD_GET_MONREV	(unsigned char)0x49 /* Monitor revison */
#define DTIMON_CMD_TEST_IRQ		(unsigned char)0x4b /* Test IRQ */
#define DTIMON_CMD_GET_IRQ		(unsigned char)0x4c /* Get Monitor IRQ */
#define DTIMON_CMD_SET_IRQ		(unsigned char)0x4d /* Set Monitor IRQ */
#define DTIMON_CMD_GET_AMPS		(unsigned char)0x4e /* Get Current Val */
#define 	DTIMON_AMPS_5_MOD1		(unsigned char)0x00	/* +5 Module 1 */
#define 	DTIMON_AMPS_5_MOD2		(unsigned char)0x01	/* +5 Module 2 */
#define 	DTIMON_AMPS_3_3			(unsigned char)0x02	/* +3.3 Module */
#define DTIMON_CMD_TOG_CURSCAL	(unsigned char)0x4f /* Set/Get Currnet Scalar */
#define DTIMON_CMD_GET_BIOSREV	(unsigned char)0x50 /* Get Bios Rev */
#define DTIMON_CMD_SET_BIOSREV	(unsigned char)0x51 /* Set Bios Rev */
#define DTIMON_CMD_GET_DIODE	(unsigned char)0x52 /* Diode Test Mask */
#define 	DTIMON_DIODE_CPU1		(unsigned char)0x01	/* Diode in CPU1 Det. */
#define 	DTIMON_DIODE_CPU2		(unsigned char)0x02	/* Diode in CPU1 Det. */
#define 	DTIMON_DIODE_CPU3		(unsigned char)0x04	/* Diode in CPU1 Det. */
#define 	DTIMON_DIODE_CPU4		(unsigned char)0x08	/* Diode in CPU1 Det. */
#define DTIMON_CMD_SET_AUTOWD	(unsigned char)0x53 /* Set Auto Watchd Params */
#define DTIMON_CMD_GET_AUTOWD	(unsigned char)0x54 /* Get Auto Watchd Params */

#define DTIMON_CPU_MAX             4
#define DTIMON_FAN_MAX             6
#define DTIMON_VOLT_BYTE1_MAX      8
#define DTIMON_VOLT_BYTE2_MAX      4
#define DTIMON_TEMP_BYTE1_MAX      8
#define DTIMON_TEMP_BYTE2_MAX      4

#define CPX1024_MAX_USERS          10
#define CPX1024_MESSAGE_CHUNK_SIZE 128

/* Internal driver data strcuture used to hole event messages */
struct cpx1024mon_message {
	int		          		  code;
	int                       val;
	struct cpx1024mon_message *next;
};

/* Internal driver data structure used to control user id information */
struct cpx1024mon_user {
	int                       pid;
	wait_queue_head_t         wq;
	struct cpx1024mon_user    *next;
	struct cpx1024mon_message *head;
	struct cpx1024mon_message *tail;
};

/*
 * Local variables:
 *  c-indent-level: 8
 *  c-basic-offset: 8
 *  tab-width: 8
 * End:
 */
