/*
 * $Id: physmap.c,v 1.10 2001/03/03 18:17:09 dwmw2 Exp $
 *
 * Normal mappings of chips in physical memory
 */

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <asm/io.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/config.h>
#include <linux/mtd/partitions.h>


#ifdef ISHONI_ENGINE
#include "asm/ibe/bootcommon.h"
#include "memory_map.h"
static struct mtd_partition myboard_partitions[] = {
#if ((ISHONI_FLASH == 4) || (ISHONI_FLASH == 6))
    {
        name: "flash bootloader",
        size: FLASH_BOOT_PART_SIZE,  /* 64k */
        offset: FLASH_BOOT_PART_OFFSET,
    },
    {
        name: "cramfs filesystem",
        size: CRAMFS_PART_SIZE,     /* WINDOW_SIZE - FLASH_BOOT_PART_SIZE - JFFS2_PART_SIZE */
        offset: CRAMFS_PART_OFFSET,
    },
    {
        name: "jffs2 partition",
        size: JFFS2_PART_SIZE,      /* 512K */
        offset: JFFS2_PART_OFFSET,
    },
    {
        name: "dummy flash partition",
        size: WINDOW_SIZE, 
        offset: FLASH_BOOT_PART_OFFSET,
    },
    {
        name:   "dynamic jffs2 partition",
        size: WINDOW_SIZE, 
        offset: FLASH_BOOT_PART_OFFSET,
    }

#elif ISHONI_FLASH == 5
    {
        name: "network bootloader",
        size: NW_BOOT_PART_SIZE,  /* 640k */
        offset: NW_BOOT_PART_OFFSET,
    },
    {
        name: "jffs2 second partition",
        size: JFFS2_SECOND_PART_SIZE, /* 4M - 640 k*/
        offset: JFFS2_SECOND_PART_OFFSET,
    },
    {
        name: "flash bootloader",
        size: FLASH_BOOT_PART_SIZE,  /* 64k */
        offset: FLASH_BOOT_PART_OFFSET,
    },
    {
        name: "cramfs filesystem",
        size: CRAMFS_PART_SIZE,  /* */
        offset: CRAMFS_PART_OFFSET,
    },
    {
        name: "jffs2 first partition",
        size: JFFS2_FIRST_PART_SIZE,  /* 4M - 64K - CRAMFS_PART_SIZE */
        offset: JFFS2_FIRST_PART_OFFSET,
    },
    {
       name: "dummy flash partition",
        size: FLASH_SIZE, /* 4M */
        offset: FLASH_BOOT_PART_OFFSET,
    },
    {
        name:   "dynamic jffs2 partition",
        size:   FLASH_SIZE, /* 4M */
        offset: FLASH_BOOT_PART_OFFSET,
    }
    
#else
    {
        name: "bootstrap and kernel",
        size: MTD_FIRST_PART,  /* 640k */
        offset: 0,
    },
    {
        name: "jffs2 filesystem",
        size: MTDPART_SIZ_FULL, /* 4M - 640 k*/
        offset: MTD_FIRST_PART,
    }
#endif
};

#else
#define WINDOW_ADDR CONFIG_MTD_PHYSMAP_START
#define WINDOW_SIZE CONFIG_MTD_PHYSMAP_LEN
#define BUSWIDTH CONFIG_MTD_PHYSMAP_BUSWIDTH
#endif /* ISHONI_ENGINE */

static struct mtd_info *mymtd;

__u8 physmap_read8(struct map_info *map, unsigned long ofs)
{
	return __raw_readb(map->map_priv_1 + ofs);
}

__u16 physmap_read16(struct map_info *map, unsigned long ofs)
{
	return __raw_readw(map->map_priv_1 + ofs);
}

__u32 physmap_read32(struct map_info *map, unsigned long ofs)
{
	return __raw_readl(map->map_priv_1 + ofs);
}

void physmap_copy_from(struct map_info *map, void *to, unsigned long from, ssize_t len)
{
	memcpy_fromio(to, map->map_priv_1 + from, len);
}

void physmap_write8(struct map_info *map, __u8 d, unsigned long adr)
{
	__raw_writeb(d, map->map_priv_1 + adr);
	mb();
}

void physmap_write16(struct map_info *map, __u16 d, unsigned long adr)
{
	__raw_writew(d, map->map_priv_1 + adr);
	mb();
}

void physmap_write32(struct map_info *map, __u32 d, unsigned long adr)
{
	__raw_writel(d, map->map_priv_1 + adr);
	mb();
}

void physmap_copy_to(struct map_info *map, unsigned long to, const void *from, ssize_t len)
{
	memcpy_toio(map->map_priv_1 + to, from, len);
}

struct map_info physmap_map = {
	name: "Physically mapped flash",
	size: WINDOW_SIZE,
	buswidth: BUSWIDTH,
	read8: physmap_read8,
	read16: physmap_read16,
	read32: physmap_read32,
	copy_from: physmap_copy_from,
	write8: physmap_write8,
	write16: physmap_write16,
	write32: physmap_write32,
	copy_to: physmap_copy_to
};

#if LINUX_VERSION_CODE < 0x20212 && defined(MODULE)
#define init_physmap init_module
#define cleanup_physmap cleanup_module
#endif

mod_init_t init_physmap(void)
{
       	printk(KERN_NOTICE "physmap flash device: %x at %x\n", WINDOW_SIZE, WINDOW_ADDR);
#ifndef ISHONI_ENGINE
	physmap_map.map_priv_1 = (unsigned long)ioremap(WINDOW_ADDR, WINDOW_SIZE);
#else
	physmap_map.map_priv_1 = (unsigned long)(WINDOW_ADDR);
#endif

	if (!physmap_map.map_priv_1) {
		printk("Failed to ioremap\n");
		return -EIO;
	}
#ifndef ISHONI_ENGINE
	mymtd = do_cfi_probe(&physmap_map);
#else
	mymtd = do_map_probe("cfi_probe", &physmap_map);
#endif

	if (mymtd) {
#ifdef MODULE
		mymtd->module = &__this_module;
#endif
#ifndef ISHONI_ENGINE
		add_mtd_device(mymtd);
#else
        /* Create MTD devices for each partition. */
        add_mtd_partitions( mymtd, myboard_partitions, NUM_PARTITIONS );
#endif

		return 0;
	}

	iounmap((void *)physmap_map.map_priv_1);
	return -ENXIO;
}

mod_exit_t cleanup_physmap(void)
{
	if (mymtd) {
		del_mtd_device(mymtd);
		map_destroy(mymtd);
	}
	if (physmap_map.map_priv_1) {
		iounmap((void *)physmap_map.map_priv_1);
		physmap_map.map_priv_1 = 0;
	}
}

int get_partition_no(const char *partition_name)
{
    int i;
    for(i = 0; i < NUM_PARTITIONS; i++){
       if (!strcmp(partition_name, myboard_partitions[i].name))
            return i;
    }
    return -1;
}

int resize_partition(const char *partition_name, int offset, int size)
{
    int partition_no;
    printk("part number %s, offset %x, size %x\n", partition_name, offset, size);
    if ((partition_no = get_partition_no(partition_name)) == -1)
    {
        printk("Invalid partition name %s\n", partition_name);
        return -1;
    }
    del_mtd_partitions(mymtd);
    /* Just change the my partition offset and size for the particular partition */ 
    myboard_partitions[partition_no].size = size;
    myboard_partitions[partition_no].offset = offset;
    add_mtd_partitions(mymtd, myboard_partitions, NUM_PARTITIONS );
    return 0;
} 

int get_jffs2_file_size(const char *partition_name)
{
    int partition_no;
    if ((partition_no = get_partition_no(partition_name)) == -1)
    {
        printk("Invalid partition name %s\n", partition_name);
        return -1;
    }
    return (myboard_partitions[partition_no].size);
}
module_init(init_physmap);
module_exit(cleanup_physmap);

