/*
 * $Id: physmap.c,v 1.10 2001/03/03 18:17:09 dwmw2 Exp $
 *
 * Normal mappings of chips in physical memory
 */

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <asm/io.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/config.h>
#include <linux/mtd/partitions.h>

#define WINDOW_ADDR     0xbf800000 
#define WINDOW_SIZE     0x800000 
#define CHIP_SIZE       0x400000
#define BUSWIDTH CONFIG_MTD_PHYSMAP_BUSWIDTH

extern struct mtd_info *cfi_probe(struct map_info *);

static struct mtd_info *mymtd;
static int probe_done = 0;
#ifdef ISHONI_ENGINE
#define MTD_FIRST_PART          0xA0000 /* 640 k */

static struct mtd_partition myboard_partitions[] = {
    {
        name: "bootstrap and kernel",
        size: MTD_FIRST_PART,  /* 640k */
        offset: WINDOW_SIZE - MTD_FIRST_PART,
    },
    {
        name: "jffs2 filesystem",
        size: WINDOW_SIZE - MTD_FIRST_PART, /* 8M - 640k*/
        offset: 0,
    }

};

#define NUM_PARTITIONS 2
#endif /* ISHONI_ENGINE */

__u8 physmap_read8(struct map_info *map, unsigned long ofs)
{
    if(probe_done == 0)
	    return __raw_readb(map->map_priv_1 + ofs);

    if(ofs < CHIP_SIZE) 
    {
	    return __raw_readb(map->map_priv_1 + ofs);
    }
    else 
    {
        if(ofs < (WINDOW_SIZE - MTD_FIRST_PART))
	        return __raw_readb(map->map_priv_1 + ofs + MTD_FIRST_PART);
        else 
            return __raw_readb(map->map_priv_1 + ofs - (CHIP_SIZE - MTD_FIRST_PART));
    }
}

__u16 physmap_read16(struct map_info *map, unsigned long ofs)
{
    if(probe_done == 0)
        return __raw_readw(map->map_priv_1 + ofs);
    if(ofs < CHIP_SIZE)
    {
        return __raw_readw(map->map_priv_1 + ofs);
    }
    else
    {
        if(ofs < (WINDOW_SIZE - MTD_FIRST_PART))
            return __raw_readw(map->map_priv_1 + ofs  + MTD_FIRST_PART);
        else
            return __raw_readw(map->map_priv_1 + ofs - (CHIP_SIZE - MTD_FIRST_PART));
    }
}

__u32 physmap_read32(struct map_info *map, unsigned long ofs)
{
    if(probe_done == 0)
        return __raw_readl(map->map_priv_1 + ofs);
    if(ofs < CHIP_SIZE)
    {
        return __raw_readl(map->map_priv_1 + ofs);
    }
    else
    {
        if(ofs < (WINDOW_SIZE - MTD_FIRST_PART))
            return __raw_readl(map->map_priv_1 + ofs + MTD_FIRST_PART);
        else
            return __raw_readl(map->map_priv_1 + ofs  - (CHIP_SIZE - MTD_FIRST_PART));
    }

}


void physmap_copy_from(struct map_info *map, void *to, unsigned long from, ssize_t len)
{
	if(probe_done == 0)
        memcpy_fromio(to, map->map_priv_1 + from, len);
    else if(from < CHIP_SIZE)
    {
        memcpy_fromio(to, map->map_priv_1 + from, len);
    }
    else
    {
        if(from < (WINDOW_SIZE - MTD_FIRST_PART))
            memcpy_fromio(to, map->map_priv_1 + from + MTD_FIRST_PART, len);
        else
            memcpy_fromio(to, map->map_priv_1 + from - (CHIP_SIZE - MTD_FIRST_PART), len);
    }
}

void physmap_write8(struct map_info *map, __u8 d, unsigned long adr)
{
    if(probe_done == 0)
    {
	    __raw_writeb(d, map->map_priv_1 + adr);
    }
    else if(adr < CHIP_SIZE)
    {
        __raw_writeb(d, map->map_priv_1 + adr);
    }
    else
    {
        if(adr < (WINDOW_SIZE - MTD_FIRST_PART))
            __raw_writeb(d, map->map_priv_1 + adr + MTD_FIRST_PART);
        else
            __raw_writeb(d, map->map_priv_1 + adr - (CHIP_SIZE - MTD_FIRST_PART));
    }
 
	mb();
}

void physmap_write16(struct map_info *map, __u16 d, unsigned long adr)
{
    if(probe_done == 0)
    {
        __raw_writew(d, map->map_priv_1 + adr);
    }
    else if(adr < CHIP_SIZE)
    {
        __raw_writew(d, map->map_priv_1 + adr);
    }
    else
    {
        if(adr < (WINDOW_SIZE - MTD_FIRST_PART))
            __raw_writew(d, map->map_priv_1 + adr + MTD_FIRST_PART);
        else
            __raw_writew(d, map->map_priv_1 + adr - (CHIP_SIZE - MTD_FIRST_PART));
    }

    mb();

}

void physmap_write32(struct map_info *map, __u32 d, unsigned long adr)
{
    if(probe_done == 0)
    {
        __raw_writel(d, map->map_priv_1 + adr);
    }
    else if(adr < CHIP_SIZE)
    {
        __raw_writel(d, map->map_priv_1 + adr);
    }
    else
    {
        if(adr < (WINDOW_SIZE - MTD_FIRST_PART))
            __raw_writel(d, map->map_priv_1 + adr + MTD_FIRST_PART);
        else
            __raw_writel(d, map->map_priv_1 + adr - (CHIP_SIZE - MTD_FIRST_PART));
    }

    mb();
}

void physmap_copy_to(struct map_info *map, unsigned long to, const void *from, ssize_t len)
{
    if(probe_done == 0)
    {
	    memcpy_toio(map->map_priv_1 + to, from, len);
    }
    else if(to < CHIP_SIZE)
    {
        memcpy_toio(map->map_priv_1 + to, from, len);
    }
    else
    {
        if(to < (WINDOW_SIZE - MTD_FIRST_PART))
            memcpy_toio(map->map_priv_1 + to + MTD_FIRST_PART, from, len);
        else
            memcpy_toio(map->map_priv_1 + to - (CHIP_SIZE - MTD_FIRST_PART), from, len);
    }

}

struct map_info physmap_map = {
	name: "Physically mapped flash",
	size: WINDOW_SIZE,
	buswidth: BUSWIDTH,
	read8: physmap_read8,
	read16: physmap_read16,
	read32: physmap_read32,
	copy_from: physmap_copy_from,
	write8: physmap_write8,
	write16: physmap_write16,
	write32: physmap_write32,
	copy_to: physmap_copy_to,
};

#if LINUX_VERSION_CODE < 0x20212 && defined(MODULE)
#define init_physmap init_module
#define cleanup_physmap cleanup_module
#endif

mod_init_t init_physmap(void)
{
       	printk(KERN_NOTICE "physmap flash device: %x at %x\n", WINDOW_SIZE, WINDOW_ADDR);
#ifndef ISHONI_ENGINE
	physmap_map.map_priv_1 = (unsigned long)ioremap(WINDOW_ADDR, WINDOW_SIZE);
#else
	physmap_map.map_priv_1 = (unsigned long)(WINDOW_ADDR);
#endif

	if (!physmap_map.map_priv_1) {
		printk("Failed to ioremap\n");
		return -EIO;
	}
	mymtd = (struct mtd_info *)do_cfi_probe(&physmap_map);
    probe_done = 1;
	if (mymtd) {
#ifdef MODULE
		mymtd->module = &__this_module;
#endif
#ifndef ISHONI_ENGINE
		add_mtd_device(mymtd);
#else
        /* Create MTD devices for each partition. */
        add_mtd_partitions( mymtd, myboard_partitions, NUM_PARTITIONS );
#endif

		return 0;
	}

	iounmap((void *)physmap_map.map_priv_1);
	return -ENXIO;
}

mod_exit_t cleanup_physmap(void)
{
	if (mymtd) {
		del_mtd_device(mymtd);
		map_destroy(mymtd);
	}
	if (physmap_map.map_priv_1) {
		iounmap((void *)physmap_map.map_priv_1);
		physmap_map.map_priv_1 = 0;
	}
}

module_init(init_physmap);
module_exit(cleanup_physmap);

