/*
 * linux/include/asm-arm/arch-xscale/time.h
 * 
 * Author:  Nicolas Pitre
 * Copyright:   (C) 2001 MontaVista Software Inc.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */


#if 1  /* IQ80310 timer */

static void iq80310_write_timer (u_long val)
{
	volatile u_char *la0 = (volatile u_char *)IQ80310_TIMER_LA0;
	volatile u_char *la1 = (volatile u_char *)IQ80310_TIMER_LA1;
	volatile u_char *la2 = (volatile u_char *)IQ80310_TIMER_LA2;
	
	*la0 = val;
	*la1 = val >> 8;
	*la2 = (val >> 16) & 0x3f;
}
	
static u_long iq80310_read_timer (void)
{
	volatile u_char *la0 = (volatile u_char *)IQ80310_TIMER_LA0;
	volatile u_char *la1 = (volatile u_char *)IQ80310_TIMER_LA1;
	volatile u_char *la2 = (volatile u_char *)IQ80310_TIMER_LA2;
	volatile u_char *la3 = (volatile u_char *)IQ80310_TIMER_LA3;
	u_long b0, b1, b2, b3, val;
	
	b0 = *la0; b1 = *la1; b2 = *la2; b3 = *la3;
	b0 = (((b0 & 0x20) >> 1) | (b0 & 0x1f));
	b1 = (((b1 & 0x20) >> 1) | (b1 & 0x1f));
	b2 = (((b2 & 0x20) >> 1) | (b2 & 0x1f));
	b3 = (b3 & 0x0f);
	val = ((b0 << 0) | (b1 << 6) | (b2 << 12) | (b3 << 18));
	return val;
}

/* IRQs are disabled before entering here from do_gettimeofday() */
static unsigned long iq80310_gettimeoffset (void)
{
	unsigned long elapsed, usec;

	/* We need elapsed timer ticks since last interrupt */
	elapsed = iq80310_read_timer();

	/* Now convert them to usec */
	usec = (unsigned long)(elapsed*tick)/LATCH;

	return usec;
}


static void iq80310_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	volatile u_char *timer_en = (volatile u_char *)IQ80310_TIMER_EN;
	
	/* clear timer interrupt */
	*timer_en &= ~2;
	*timer_en |= 2;

	do_leds();

	irq_exit(smp_processor_id(), irq);
	do_timer(regs);
	irq_enter(smp_processor_id(), irq);
}


inline void setup_timer (void)
{
	volatile u_char *timer_en = (volatile u_char *)IQ80310_TIMER_EN;

	gettimeoffset = iq80310_gettimeoffset;
	timer_irq.handler = iq80310_timer_interrupt;
	setup_arm_irq(IRQ_IQ80310_TIMER, &timer_irq);
	*timer_en = 0;
	iq80310_write_timer(LATCH);
	*timer_en |= 2;
	*timer_en |= 1;
#if 0
	{
		volatile u_char *irq_stat = (volatile u_char *)IQ80310_INT_STAT;
		u_char int1, int2;
		u_long intsrc1, intsrc2;
		int1 = *irq_stat;
		asm volatile ("mrc p13, 0, %0, c4, c0, 0" : "=r" (intsrc1));
		printk("initial int stat = %02x, %08x\n", int1, intsrc1);
		do {
			printk("-");
			int2 = *irq_stat;
		} while (int1 == int2);
		asm volatile ("mrc p13, 0, %0, c4, c0, 0" : "=r" (intsrc2));
		printk("\nnext int stat = %02x, %08x\n", int2, intsrc2);
		if (intsrc1 == intsrc2)
			panic("the CPLD interrupts aren't getting to the CPU");
	}
#endif
}

#else

/* 
 * Here the PMU cycle counter of the XS80200 is used as our timer source.
 * This means that the PMU cannot be used freely to any will...
 */

/* IRQs are disabled before entering here from do_gettimeofday() */
static unsigned long xs80200pmu_gettimeoffset (void)
{
	unsigned long elapsed, usec;

	/* We need elapsed timer ticks since last interrupt */
	/* 
	 * Read the CCNT value.  The returned value is 
	 * between -LATCH and 0, 0 corresponding to a full jiffy 
	 */
	asm volatile ("mrc p14, 0, %0, c1, c0, 0" : "=r" (elapsed));
	elapsed += LATCH;

	/* Now convert them to usec */
	usec = (unsigned long)(elapsed*tick)/LATCH;

	return usec;
}

static void xs80200pmu_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	long flags, cnt;
	
	/* Clear and disable CCNT overflow interrupt in the PMNC */
	asm volatile ("mcr p14, 0, %0, c0, c0, 0" : : "r" (0x00000701));

	do {
		save_flags_cli(flags);
		do_leds();
		do_timer(regs);
		/* get current count from CCNT */
		asm volatile ("mrc p14, 0, %0, c1, c0, 0" : "=r" (cnt));
		/* 
		 * Bring the count back for one timer tick.
		 * To be precise, we would need to consider the cycles 
		 * involved in this read-modify-write sequence as well.
		 */
		cnt -= LATCH;
		/* write back updated count into CCNT */
		asm volatile ("mcr p14, 0, %0, c1, c0, 0" : : "r" (cnt));
		restore_flags(flags);
		/* clear CCNT overflow in the PMNC */
		asm volatile ("mcr p14, 0, %0, c0, c0, 0" : : "r" (0x00000701));
	} while (cnt >= 0);
	
	/* Enable CCNT overflow interrupt in the PMNC */
	asm volatile ("mcr p14, 0, %0, c0, c0, 0" : : "r" (0x00000041));
}

static inline void setup_timer (void)
{
	gettimeoffset = xs80200pmu_gettimeoffset;
	timer_irq.handler = xs80200pmu_timer_interrupt;
	setup_arm_irq(IRQ_XS80200_PMU, &timer_irq);
	
	/* set the initial CCNT value */
	asm volatile ("mcr p14, 0, %0, c1, c0, 0" : : "r" (-LATCH));
	
	/* enable interrupt to occur on CCNT wraps in the PMNC */
	asm volatile ("mcr p14, 0, %0, c0, c0, 0" : : "r" (0x00000741));
}

#endif

