#ifndef __KGDB
#define __KGDB

/*
 * This file should not include ANY others.  This makes it usable
 * most anywhere without the fear of include order or inclusion.
 * Make it so!
 */

/* This function is called by main.c just before cpu_idle.  It should
 * allow gdb to interrupt into the debug stub by sending a ^C.
 * What has to be done is to set up the serial chip, if not done yet, 
 * get an interrupt vector from the system and set it up to call the
 * stubs interrupt handler.  The handler will check for control C and
 * execute a breakpoint if it finds one.  Other chars should be stored
 * to some limit and provide when GetDebugChar asks for them.
 */

extern void breakpoint(void);
#define INIT_KGDB_INTS kgdb_enable_ints()

#ifndef BREAKPOINT   
#define BREAKPOINT   asm("   int $3");
#endif
/*
 * GDB debug stub (or any debug stub) can point the 'linux_debug_hook'
 * pointer to its routine and it will be entered as the first thing
 * when a trap occurs.
 *
 * Return values are, at present, undefined.
 *
 * The debug hook routines does not necessarily return to its caller.
 * It has the register image and thus may choose to resume execution
 * anywhere it pleases. 
 */
struct pt_regs;

typedef int	gdb_debug_hook(int trapno,
			       int signo,
			       int err_code,
			       struct pt_regs *regs) ;
extern gdb_debug_hook	*linux_debug_hook ;
extern int in_kgdb(struct pt_regs *regs);
extern int remote_debug;
#endif /* __KGDB */

