/******************************************************************************
 *  ishoni Networks Confidential and Proprietary Copyright (c) 2000
 ******************************************************************************
 *  FILE:       bootcommon.h
 *
 *  ENGINEER:   Prodyut Hazarika
 *
 *  PROJECT:    Linux Bootloader	
 *
 *  REQUIREMENTS and FEATURES:
 *    This file contains the defines used by Linux bootloader and Flash driver.
 * 
 *    
 *  COMPILER REQUIREMENT:  
 *	== Compiler Name ===  ====== Options & Notes ==============================
 *
 *	RELEASE HISTORY: (latest on top)
 *	Rev   == Date ==  ========= Modification after first release ==============
 *	1.0   2002-MM-DD
 *
 ********** <Indent=4 Spaces> ************************************************/

#ifndef _BOOTCOMMON_H
#define _BOOTCOMMON_H

#define SUCCESS                         0

#ifndef ERROR
#define ERROR                           -1
#endif

#define BUSWIDTH CONFIG_MTD_PHYSMAP_BUSWIDTH
#define BOOT_PARAM_MAGIC_NUMBER  0x86421357

#if (ISHONI_FLASH == 4)  || (ISHONI_FLASH == 6)

/* Only the flash bootloader. Single 4MB flash */
#define FLASH_BOOT_DEV           "/dev/mtd0"
#define CRAMFS_DEV               "/dev/mtd2"
#define JFFS2_DEV                "/dev/mtd4"
#define FLASH_DEV                "/dev/mtd6"    /* Device for accessing bfc00000 flash fully */
#define DY_JFFS2_DEV             "/dev/mtd8" 
#define LOG_DEV                  "/dev/mtd10"

#define RFC1_DEV                "/dev/mtd12" 
#define RFC2_DEV                "/dev/mtd14" 

#define DYN_RFC_DEV             "/dev/mtd16" 
#define TOTAL_RFC_DEV           "/dev/mtd18" 

#define CRAMFS_BLK_DEV           "/dev/mtdblock1"
#define ROOT_DEV_NAME            "/dev/mtdblock1"
#define JFFS2_BLK_DEV            "/dev/mtdblock2"

#define WINDOW_ADDR              START_OF_BOOT_FLASH 

#if (ISHONI_FLASH == 4) /* 4MB */
    #define WINDOW_SIZE              0x400000 
#else /* 8MB */
    #define WINDOW_SIZE              0x800000
#endif /*#if (ISHONI_FLASH == 4) */

#define FLASH_BOOT_PART_SIZE     SIZEOF_FLASH_BOOTLDR
#define FLASH_BOOT_PART_OFFSET   0 

#if defined(ISHONI_RAWFLASHLOG) &&  defined(ISHONI_RAW_FLASH_CONFIG)
    #define JFFS2_PART_SIZE          0x0 /*JFFS2 is ZERO for RFC + Raw Flash Log*/
#else
    #define JFFS2_PART_SIZE          0x50000 /*64x5 =320k-Min needed by JFFS2 */
#endif/*defined(ISHONI_RAWFLASHLOG) &&  defined(ISHONI_RAW_FLASH_CONFIG)*/

#ifdef ISHONI_RAW_FLASH_CONFIG
    #define RFC_PARTITION_SZ        0x10000
#else
    #define RFC_PARTITION_SZ        0
#endif /*#ifdef ISHONI_RAW_FLASH_CONFIG*/

#ifdef ISHONI_RAWFLASHLOG
    #if (ISHONI_FLASH == 4) /* 4MB */
        #define FLASH_LOG_PART_SIZE      0x10000 /* 64 k */
    #else /* 8MB */
        #define FLASH_LOG_PART_SIZE      (0x10000 << 1 )/* 128 k */
    #endif/*if ISHONI_FLASH == 4*/
#else
    #define FLASH_LOG_PART_SIZE      0       /*Dummy partititions space */
#endif

#define CRAMFS_PART_SIZE         (WINDOW_SIZE - FLASH_BOOT_PART_SIZE - JFFS2_PART_SIZE - FLASH_LOG_PART_SIZE - RFC_PARTITION_SZ - RFC_PARTITION_SZ) /* rest is cramfs */

#define CRAMFS_PART_OFFSET       (FLASH_BOOT_PART_OFFSET + FLASH_BOOT_PART_SIZE) /* rest is cramfs */

#define JFFS2_PART_OFFSET        (CRAMFS_PART_OFFSET + CRAMFS_PART_SIZE)

#define FLASH_LOG_PART_OFFSET    (JFFS2_PART_OFFSET + JFFS2_PART_SIZE)

#define FIRST_CONFIG_PART_OFFSET ((FLASH_LOG_PART_OFFSET) + FLASH_LOG_PART_SIZE)

#define SECOND_CONFIG_PART_OFFSET  (FIRST_CONFIG_PART_OFFSET + RFC_PARTITION_SZ)



/*End of ISHONI_FLASH == 4 */
#elif ISHONI_FLASH == 5

/* Both the flash and network bootloader. Two 4MB flash - for development purposes */
#define NW_BOOT_DEV              "/dev/mtd0"
#define JFFS2_DEV                "/dev/mtd2"
#define FLASH_BOOT_DEV           "/dev/mtd4"
#define CRAMFS_DEV               "/dev/mtd6"
#define JFFS2_SEC_DEV            "/dev/mtd8"
#define FLASH_DEV                "/dev/mtd10"   /* Device for accessing bfc00000 flash fully */
#define ROOT_DEV_NAME            "/dev/mtdblock3"
#define JFFS2_SEC_BLK_DEV        "/dev/mtdblock1"
#define CRAMFS_BLK_DEV           "/dev/mtdblock3"
#define JFFS2_BLK_DEV            "/dev/mtdblock4"
#define WINDOW_ADDR              START_OF_BOOT_FLASH 
#define WINDOW_SIZE              0x800000 
#define FLASH_SIZE               0x400000 
#define NW_BOOT_PART_SIZE        0xA0000  /* 640 k */
#define NW_BOOT_PART_OFFSET      0        
#define JFFS2_SECOND_PART_SIZE   (FLASH_SIZE - NW_BOOT_PART_SIZE) 
#define JFFS2_SECOND_PART_OFFSET (NW_BOOT_PART_OFFSET + NW_BOOT_PART_SIZE)
#define FLASH_BOOT_PART_SIZE     SIZEOF_FLASH_BOOTLDR
#define FLASH_BOOT_PART_OFFSET   0x400000 /* 4MB */
#define JFFS2_FIRST_PART_SIZE    0x80000  /* 512 k */ 
#define CRAMFS_PART_SIZE         (FLASH_SIZE - FLASH_BOOT_PART_SIZE - JFFS2_FIRST_PART_SIZE) /* rest is cramfs */
#define CRAMFS_PART_OFFSET       (FLASH_BOOT_PART_OFFSET + FLASH_BOOT_PART_SIZE) 
#define JFFS2_FIRST_PART_OFFSET  (CRAMFS_PART_OFFSET + CRAMFS_PART_SIZE)

#else

#define ROOT_DEV_NAME            "/dev/mtdblock1"
#define JFFS2_BLK_DEV            "/dev/mtdblock1"
#define JFFS2_DEV                "/dev/mtd2"
#define NW_BOOT_DEV              "/dev/mtd0"

#endif /* ISHONI_FLASH */

/* Duplicate bootloader exists only when ISHONI_FLASH == 3 */
#if ISHONI_FLASH == 3
#define DUPLICATE_BOOT_DEV       "/dev/mtd4"
#endif

#define BUFFER_SIZE 1024
#ifndef ERROR
#define ERROR -1
#endif
#define SUCCESS 0

/* The error number must match the string description array bootErrMsg[] in bootui.c */
enum booterrno {    
ENONEXEC   =  1,           /* File is not an ELF executable file */
EREADINTERRUPT,            /* Read interrupted or EOF reached */
EBOOTINTERRUPT,            /* Boot interrupted by user */
EBADPARAM,                 /* Invalid configuration parameters */
ENOFILE                    /* File not found */

};

/* This Flash structure map will sit at 0xbfc04000 in the flash */
/* TODO -> Add partition table structure to this later          */
/* Flash partition Magic number defined in memory_map.h because */
/* it has to be included from assembly file */
typedef struct {
    unsigned int magic;                     /* Magic number for Ishoni Flash partition table */
    unsigned int numFlashChips;             /* Number of flash chips in the board 1 or 2*/
    unsigned int sizeofFlashChip1;          /* size of flash chip number 1 in the board */
    unsigned int sizeofFlashChip2;          /* size of flash chip number 2 in the board */
}flashMapStructure;

int JFFS2Init(void);
int getImageRelDir(char *);
int setImageRelDir(char *);
int getBootLoaderImage(char *);
int getRootFileSystemImage(char *);
int setBootLoaderImage(char *);
int setRootFileSystemImage(char *);
int getImageAbsolutePath(char *);
int getImagePath(char *);

int ftpFileOpen(char *);
int ftpFileReOpen(int , char *);
int ftpFileClose(void);
int ftpRootFileSystemUpgrade(char *, char *, unsigned int);

#endif  /* _BOOTCOMMON_H */
