/********************************************************************************************** 
 *
 *  FILE : ibeirq.h
 *
 * New file written for ishoni interrupt archirtecture support.
 * Originally taken from "arch/mips/lx/lxirq.h".
 * 05/15/2001, Nazim Khan.
 *
 */


#ifndef __ASM_LX_INTERRUPTS_H 
#define __ASM_LX_INTERRUPTS_H 

/*
 * Number of available Interrupts.
 */
#define NR_INTS	32	

/*
 * IBE Interrupts. 
 */

#define PIC_INT(x) (x-8) 
#define MAX_CPU_INTS 8

#define SWINT0   0 /* These will never be used. */ 
#define SWINT1   1

#define HWINT0   2 /* CPU Hardware interrupts. */
#define HWINT1   3
#define HWINT2   4 
#define HWINT3   5   
#define HWINT4   6 /* NetPIC.*/
#define HWINT5   7  

#define MEM_PROT_ERR_IRQ 8
#define MIPS0_IRQ        9   
#define MIPS1_IRQ       10 
#define MIPS2_IRQ       11 
#define MIPS3_IRQ       12 
#define PCI_IRQ         13 
#define PCI_EXT_IRQ     14           
#define HOST_IRQ        15           
#define ETH_CORE0_IRQ   16  
#define ETH_BM0_IRQ     17 
#define ETH_CORE1_IRQ   18  
#define ETH_BM1_IRQ     19 
#define WAN_CORE_IRQ    20 
#define WAN_BM_IRQ      21 
#define SCC_CORE_IRQ    22 
#define SCC_BM_IRQ      23 
#define TIMER0_IRQ      24 
#define TIMER1_IRQ      25 
#define TIMER2_IRQ      26 
#define USB_CORE_IRQ    27 
#define USB_BM_IRQ      28 
#define PIO_IRQ         29 
#define CODEC_IRQ       30
#define SPI_IRQ         31 

/**
 * Lexra vectored interrupt enable / cause bits.
 */
#define IBE_CPU_IRQ0         (1<<8)    	/* SW0 */
#define IBE_CPU_IRQ1         (1<<9)    	/* SW1 */
#define IBE_CPU_IRQ2         (1<<10)   	/* HW0 */
#define IBE_CPU_IRQ3         (1<<11)	/* HW1 */
#define IBE_CPU_IRQ4         (1<<12)	/* HW2 */
#define IBE_CPU_IRQ5         (1<<13) 	/* HW3 */
#define IBE_CPU_IRQ6         (1<<14) 	/* HW4 */
#define IBE_CPU_IRQ7         (1<<15) 	/* HW5 */

#define CPU0_REASON_REG 0xa1203020 /* for assembly code.*/

#ifndef __ASSEMBLY__

typedef struct {
    unsigned int ireq0;              /* 0xa1203000 */
    unsigned int ireq1;              /* 0xa1203004 */
    unsigned int ireq2;              /* 0xa1203008 */
    unsigned int ireq3;              /* 0xa120300c */
    unsigned int ien0;               /* 0xa1203010 */
    unsigned int ien1;               /* 0xa1203014 */
    unsigned int ien2;               /* 0xa1203018 */
    unsigned int ien3;               /* 0xa120301c */
    unsigned int reason0;            /* 0xa1203020 */
    unsigned int reason1;            /* 0xa1203024 */
    unsigned int reason2;            /* 0xa1203028 */
    unsigned int reason3;            /* 0xa120302c */
    unsigned int hp0;                /* 0xa1203030 */
    unsigned int hp1;                /* 0xa1203034 */
    unsigned int hp2;                /* 0xa1203038 */
    unsigned int hp3;                /* 0xa120303c */
    unsigned int ptp[4];       /* 0xa1203040 - 0xa120304c */
    unsigned int status;             /* 0xa1203050 */
    unsigned int pci;                /* 0xa1203054 */
} tPicReg;
#define PIC_BASE_ADDRESS           (tPicReg *)0xA1203000

#define GET_PIC_IEN() *((unsigned int *)&((PIC_BASE_ADDRESS)->ien0))
#define SET_PIC_IEN(ien) *((unsigned int *)&((PIC_BASE_ADDRESS)->ien0)) |= ien
#define RESET_PIC_IEN(ien) *((unsigned int *)&((PIC_BASE_ADDRESS)->ien0)) &= ien


/* To be removed laetr, when we do cleanup. Nazim Khan*/

/**
 * Variable for storage of the clock decrement value
 */
int LX_Dec_Value;


/* For timer */
typedef struct 
{
	volatile unsigned int	tm_control;
	volatile unsigned int	tm_constant;
	volatile unsigned int	tm_count;
	volatile unsigned int	tm_int;
} tTmReg;


/*
 * Data structure to hide the differences between the DECstation Interrupts
 *
 * If asic_mask == NULL, the interrupt is directly handled by the CPU.
 * Otherwise this Interrupt is handled the IRQ Controller.
 */

typedef struct
{
	unsigned int	cpu_mask;	/* checking and enabling interrupts in CP0	*/
	unsigned int	iemask;		/* enabling interrupts in IRQ Controller	*/
} decint_t;


/**
 * ibe_interrupt[NR_INTS]
 * Array of interrupt masks.
 *
 */
extern decint_t ibe_interrupt[NR_INTS];

/**
 * Interrupt mask tables for use with the assembly language interrupt handlers.
 */
extern unsigned long cpu_mask_tbl[8];
extern unsigned long cpu_irq_nr[8];
extern unsigned long cpu_ivec_tbl[8];
extern unsigned long netpic_irq_nr[32];


#define SET_TIMER_CONSTANT(tmReg, TM_const)  (tmReg)->tm_constant = TM_const
#define MY_TIMER_ADDRESS	(tTmReg *)0xA120C000
#define SET_TIMER_CONTROL(tmReg, control)    (tmReg)->tm_control = control
#define TM_CNTL_RELOAD_CONSTANT_START   2
#define TM_CNTL_START_SHF               0
#define TM_CNTL_ENABLE_INTERRUPT        1
#define TM_CNTL_INT_EN_SHF              2
#define TM_CNTL_AUTO_RELOAD_ON          1
#define TM_CNTL_AUTO_RELOAD_SHF         3

#define CLEAR_TIMER_INTERRUPT(tmReg) (tmReg)->tm_int = 0x00000001


#endif  /* _ASSEMBLY */

#endif 




