/*
 * PowerPC atomic operations
 */

#ifndef _ASM_PPC_ATOMIC_H_ 
#define _ASM_PPC_ATOMIC_H_

typedef struct { volatile int counter; } atomic_t;

extern void atomic_clear_mask(unsigned long mask, unsigned long *addr);
extern void atomic_set_mask(unsigned long mask, unsigned long *addr);

#define ATOMIC_INIT(i)	{ (i) }

#define atomic_read(v)		((v)->counter)

#ifdef CONFIG_ATOMIC_SYNC_FIX
#define ATOMIC_SYNC "	sync\n"
#else
#define ATOMIC_SYNC
#endif

static __inline__ void atomic_set(atomic_t *v, int a)
{
	int t;  

	__asm__ __volatile__("\n\
1:	lwarx   %0,0,%3\n\
	ori     %0,%2,0\n"\
	ATOMIC_SYNC\
"	stwcx.  %0,0,%3\n\
	bne-    1b"
	: "=&r" (t), "=m" (v->counter)
	: "r" (a), "r" (v), "m" (v->counter)
	: "cc");
}

static __inline__ int atomic_add_return(int a, atomic_t *v)
{
	int t;

	__asm__ __volatile__("\n\
1:	lwarx	%0,0,%3\n\
	add	%0,%2,%0\n"\
	ATOMIC_SYNC\
"	stwcx.	%0,0,%3\n\
	bne-	1b"
	: "=&r" (t), "=m" (v->counter)
	: "r" (a), "r" (v), "m" (v->counter)
	: "cc");

	return t;
}

static __inline__ int atomic_sub_return(int a, atomic_t *v)
{
	int t;

	__asm__ __volatile__("\n\
1:	lwarx	%0,0,%3\n\
	subf	%0,%2,%0\n"\
	ATOMIC_SYNC\
"	stwcx.	%0,0,%3\n\
	bne-	1b"
	: "=&r" (t), "=m" (v->counter)
	: "r" (a), "r" (v), "m" (v->counter)
	: "cc");

	return t;
}

static __inline__ int atomic_inc_return(atomic_t *v)
{
	int t;

	__asm__ __volatile__("\n\
1:	lwarx	%0,0,%2\n\
	addic	%0,%0,1\n"\
	ATOMIC_SYNC\
"	stwcx.	%0,0,%2\n\
	bne-	1b"
	: "=&r" (t), "=m" (v->counter)
	: "r" (v), "m" (v->counter)
	: "cc");

	return t;
}

static __inline__ int atomic_dec_return(atomic_t *v)
{
	int t;

	__asm__ __volatile__("\n\
1:	lwarx	%0,0,%2\n\
	addic	%0,%0,-1\n"\
	ATOMIC_SYNC\
"	stwcx.	%0,0,%2\n\
	bne	1b"
	: "=&r" (t), "=m" (v->counter)
	: "r" (v), "m" (v->counter)
	: "cc");

	return t;
}

#define atomic_add(a, v)		((void) atomic_add_return((a), (v)))
#define atomic_sub(a, v)		((void) atomic_sub_return((a), (v)))
#define atomic_sub_and_test(a, v)	(atomic_sub_return((a), (v)) == 0)
#define atomic_inc(v)			((void) atomic_inc_return((v)))
#define atomic_dec(v)			((void) atomic_dec_return((v)))
#define atomic_dec_and_test(v)		(atomic_dec_return((v)) == 0)

#endif /* _ASM_PPC_ATOMIC_H_ */
