/*
 *  linux/init/main.c
 *
 *  Copyright (C) 1991, 1992  Linus Torvalds
 *
 *  GK 2/5/95  -  Changed to support mounting root fs via NFS
 *  Added initrd & change_root: Werner Almesberger & Hans Lermen, Feb '96
 *  Moan early if gcc is old, avoiding bogus kernels - Paul Gortmaker, May '96
 *  Simplified starting of init:  Michael A. Griffith <grif@acm.org> 
 */

#ifndef __KERNEL_SYSCALLS__
#define __KERNEL_SYSCALLS__
#endif

#include <linux/config.h>
#include <linux/proc_fs.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/unistd.h>
#include <linux/string.h>
#include <linux/ctype.h>
#include <linux/delay.h>
#include <linux/utsname.h>
#include <linux/ioport.h>
#include <linux/init.h>
#include <linux/raid/md.h>
#include <linux/smp_lock.h>
#include <linux/blk.h>
#include <linux/hdreg.h>
#include <linux/iobuf.h>
#include <linux/bootmem.h>
#include <linux/mount.h>
#include <linux/fs.h>

#include <asm/io.h>
#include <asm/bugs.h>

#ifdef ISHONI_ENGINE
#include <asm/lexra/lx_defs.h>
#include <asm/ibe/bootcommon.h>
#include "lx_inc.h"
#include "mtdutils.h"
#include "memory_map.h"
#endif

#ifdef LINUX_VOICE
#include "global.h"
#include "rca.h"
#endif

#ifdef CONFIG_PCI
#include <linux/pci.h>
#endif

#ifdef CONFIG_DIO
#include <linux/dio.h> 
#endif

#ifdef CONFIG_ZORRO
#include <linux/zorro.h>
#endif

#ifdef CONFIG_MTRR
#  include <asm/mtrr.h>
#endif

#ifdef CONFIG_3215_CONSOLE
extern int con3215_activate(void);
#endif

#ifdef CONFIG_NUBUS
#include <linux/nubus.h>
#endif

#ifdef CONFIG_ISAPNP
#include <linux/isapnp.h>
#endif

#ifdef CONFIG_IRDA
#include <net/irda/irda_device.h>
#endif

#ifdef CONFIG_X86_IO_APIC
#include <asm/smp.h>
#endif

/*
 * Versions of gcc older than that listed below may actually compile
 * and link okay, but the end product can have subtle run time bugs.
 * To avoid associated bogus bug reports, we flatly refuse to compile
 * with a gcc that is known to be too old from the very beginning.
 */
#if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 91)
#error Sorry, your GCC is too old. It builds incorrect kernels.
#endif

extern char _stext, _etext;
extern char *linux_banner;

extern int console_loglevel;

static int init(void *);

extern void init_IRQ(void);
extern void init_modules(void);
extern void sock_init(void);
extern void fork_init(unsigned long);
extern void mca_init(void);
extern void sbus_init(void);
extern void ppc_init(void);
extern void sysctl_init(void);
extern void signals_init(void);
extern void bdev_init(void);
extern int init_pcmcia_ds(void);
extern void net_notifier_init(void);

extern void free_initmem(void);
extern void filesystem_setup(void);


#ifdef CONFIG_TC
extern void tc_init(void);
#endif

extern void ecard_init(void);

#if defined(CONFIG_SYSVIPC)
extern void ipc_init(void);
#endif
#if defined(CONFIG_QUOTA)
extern void dquot_init_hash(void);
#endif

#ifdef ISHONI_ENGINE
extern btParamStruct  flashParamStruct;
extern int saveFlashParamsFlag;
#ifdef NEW_FEATURES
extern flashMapStructure flashPartitionTblStruct;
extern int saveFlashPartitionTblFlag;
#endif
#ifdef ISHONI_KSHELL
int shell_main(int, char **);
#endif /* ISHONI_KSHELL */
#ifdef LINUX_VOICE
void init_voice(void);
extern int slac_setHwClkType(int); 
extern int registerAmdSlac();
#endif


#ifdef LINUX_BOOT_LOADER
extern int bs_main(void ); 
#endif

#endif  /* #ifdef ISHONI_ENGINE */

/*
 * Boot command-line arguments
 */
#define MAX_INIT_ARGS 8
#define MAX_INIT_ENVS 8

extern void time_init(void);
extern void softirq_init(void);

#ifdef ISHONI_ENGINE 
extern	int     ibeEnableNetPICinterrupt(void); /*  TIMER */
#ifndef LINUX_BOOT_LOADER
extern  void    init_fndriver(void);
#endif

#endif /* ifdef ISHONI_ENGINE */

int rows, cols;


#ifdef CONFIG_BLK_DEV_INITRD
kdev_t real_root_dev;
#endif
#ifdef ISHONI_ROOT_NFS
static kdev_t real_root_dev;
#endif

#ifndef ISHONI_ROOT_NFS
int root_mountflags = MS_RDONLY;
#else
int root_mountflags;
#endif

char *execute_command;
char root_device_name[64];


static char * argv_init[MAX_INIT_ARGS+2] = { "init", NULL, };
static char * envp_init[MAX_INIT_ENVS+2] = { "HOME=/", "TERM=linux", NULL, };

static int __init profile_setup(char *str)
{
    int par;
    if (get_option(&str,&par)) prof_shift = par;
	return 1;
}

__setup("profile=", profile_setup);


static struct dev_name_struct {
	const char *name;
	const int num;
} root_dev_names[] __initdata = {
	{ "nfs",     0x00ff },
	{ "hda",     0x0300 },
	{ "hdb",     0x0340 },
	{ "loop",    0x0700 },
	{ "hdc",     0x1600 },
	{ "hdd",     0x1640 },
	{ "hde",     0x2100 },
	{ "hdf",     0x2140 },
	{ "hdg",     0x2200 },
	{ "hdh",     0x2240 },
	{ "hdi",     0x3800 },
	{ "hdj",     0x3840 },
	{ "hdk",     0x3900 },
	{ "hdl",     0x3940 },
	{ "hdm",     0x5800 },
	{ "hdn",     0x5840 },
	{ "hdo",     0x5900 },
	{ "hdp",     0x5940 },
	{ "hdq",     0x5A00 },
	{ "hdr",     0x5A40 },
	{ "hds",     0x5B00 },
	{ "hdt",     0x5B40 },
	{ "sda",     0x0800 },
	{ "sdb",     0x0810 },
	{ "sdc",     0x0820 },
	{ "sdd",     0x0830 },
	{ "sde",     0x0840 },
	{ "sdf",     0x0850 },
	{ "sdg",     0x0860 },
	{ "sdh",     0x0870 },
	{ "sdi",     0x0880 },
	{ "sdj",     0x0890 },
	{ "sdk",     0x08a0 },
	{ "sdl",     0x08b0 },
	{ "sdm",     0x08c0 },
	{ "sdn",     0x08d0 },
	{ "sdo",     0x08e0 },
	{ "sdp",     0x08f0 },
	{ "ada",     0x1c00 },
	{ "adb",     0x1c10 },
	{ "adc",     0x1c20 },
	{ "add",     0x1c30 },
	{ "ade",     0x1c40 },
	{ "fd",      0x0200 },
	{ "md",      0x0900 },	     
	{ "xda",     0x0d00 },
	{ "xdb",     0x0d40 },
	{ "ram",     0x0100 },
	{ "scd",     0x0b00 },
	{ "mcd",     0x1700 },
	{ "cdu535",  0x1800 },
	{ "sonycd",  0x1800 },
	{ "aztcd",   0x1d00 },
	{ "cm206cd", 0x2000 },
	{ "gscd",    0x1000 },
	{ "sbpcd",   0x1900 },
	{ "eda",     0x2400 },
	{ "edb",     0x2440 },
	{ "pda",	0x2d00 },
	{ "pdb",	0x2d10 },
	{ "pdc",	0x2d20 },
	{ "pdd",	0x2d30 },
	{ "pcd",	0x2e00 },
	{ "pf",		0x2f00 },
	{ "apblock", APBLOCK_MAJOR << 8},
	{ "ddv", DDV_MAJOR << 8},
	{ "jsfd",    JSFD_MAJOR << 8},
#ifdef CONFIG_MDISK
        { "mnda", (MDISK_MAJOR << MINORBITS)},
        { "mndb", (MDISK_MAJOR << MINORBITS) + 1},
        { "mndc", (MDISK_MAJOR << MINORBITS) + 2},
        { "mndd", (MDISK_MAJOR << MINORBITS) + 3},
        { "mnde", (MDISK_MAJOR << MINORBITS) + 4},
        { "mndf", (MDISK_MAJOR << MINORBITS) + 5},
        { "mndg", (MDISK_MAJOR << MINORBITS) + 6},
        { "mndh", (MDISK_MAJOR << MINORBITS) + 7},
#endif
#ifdef CONFIG_DASD
	{ "dasda", (DASD_MAJOR << MINORBITS) },
	{ "dasdb", (DASD_MAJOR << MINORBITS) + (1 << 2) },
	{ "dasdc", (DASD_MAJOR << MINORBITS) + (2 << 2) },
	{ "dasdd", (DASD_MAJOR << MINORBITS) + (3 << 2) },
	{ "dasde", (DASD_MAJOR << MINORBITS) + (4 << 2) },
	{ "dasdf", (DASD_MAJOR << MINORBITS) + (5 << 2) },
	{ "dasdg", (DASD_MAJOR << MINORBITS) + (6 << 2) },
	{ "dasdh", (DASD_MAJOR << MINORBITS) + (7 << 2) },
#endif
#if defined(CONFIG_BLK_CPQ_DA) || defined(CONFIG_BLK_CPQ_DA_MODULE)
	{ "ida/c0d0p",0x4800 },
	{ "ida/c0d1p",0x4810 },
	{ "ida/c0d2p",0x4820 },
	{ "ida/c0d3p",0x4830 },
	{ "ida/c0d4p",0x4840 },
	{ "ida/c0d5p",0x4850 },
	{ "ida/c0d6p",0x4860 },
	{ "ida/c0d7p",0x4870 },
	{ "ida/c0d8p",0x4880 },
	{ "ida/c0d9p",0x4890 },
	{ "ida/c0d10p",0x48A0 },
	{ "ida/c0d11p",0x48B0 },
	{ "ida/c0d12p",0x48C0 },
	{ "ida/c0d13p",0x48D0 },
	{ "ida/c0d14p",0x48E0 },
	{ "ida/c0d15p",0x48F0 },
#endif
#if defined(CONFIG_BLK_CPQ_CISS_DA) || defined(CONFIG_BLK_CPQ_CISS_DA_MODULE)
    { "cciss/c0d0p",0x6800 },
    { "cciss/c0d1p",0x6810 },
    { "cciss/c0d2p",0x6820 },
    { "cciss/c0d3p",0x6830 },
    { "cciss/c0d4p",0x6840 },
    { "cciss/c0d5p",0x6850 },
    { "cciss/c0d6p",0x6860 },
    { "cciss/c0d7p",0x6870 },
    { "cciss/c0d8p",0x6880 },
    { "cciss/c0d9p",0x6890 },
    { "cciss/c0d10p",0x68A0 },
    { "cciss/c0d11p",0x68B0 },
    { "cciss/c0d12p",0x68C0 },
    { "cciss/c0d13p",0x68D0 },
    { "cciss/c0d14p",0x68E0 },
    { "cciss/c0d15p",0x68F0 },
#endif
#ifdef CONFIG_NFTL
    { "nftla", 0x5d00 },
#endif
#ifdef CONFIG_MTD
    { "mtdblock", 0x1f00 },
#endif
	{ NULL, 0 }
};

kdev_t __init name_to_kdev_t(char *line)
{
	int base = 0;

	if (strncmp(line,"/dev/",5) == 0) {
		struct dev_name_struct *dev = root_dev_names;
		line += 5;
		do {
			int len = strlen(dev->name);
			if (strncmp(line,dev->name,len) == 0) {
				line += len;
				base = dev->num;
				break;
			}
			dev++;
		} while (dev->name);
	}
	return to_kdev_t(base + simple_strtoul(line,NULL,base?10:16));
}

static int __init root_dev_setup(char *line)
{
	int i;
	char ch;

	ROOT_DEV = name_to_kdev_t(line);
	memset (root_device_name, 0, sizeof root_device_name);
	if (strncmp (line, "/dev/", 5) == 0) line += 5;
	for (i = 0; i < sizeof root_device_name - 1; ++i)
	{
	    ch = line[i];
	    if ( isspace (ch) || (ch == ',') || (ch == '\0') ) break;
	    root_device_name[i] = ch;
	}
	return 1;
}


__setup("root=", root_dev_setup);

static int __init checksetup(char *line)
{
	struct kernel_param *p;

	p = &__setup_start;
	do {
		int n = strlen(p->str);
		if (!strncmp(line,p->str,n)) {
			if (p->setup_func(line+n))
				return 1;
		}
		p++;
	} while (p < &__setup_end);
	return 0;
}

/* this should be approx 2 Bo*oMips to start (note initial shift), and will
   still work even if initially too large, it will just take slightly longer */
unsigned long loops_per_jiffy = (1<<12);   // Originally 1<<12

/* This is the number of bits of precision for the loops_per_second.  Each
   bit takes on average 1.5/HZ seconds.  This (like the original) is a little
   better than 1% */
#define LPS_PREC 8

void __init calibrate_delay(void)
{
	unsigned long ticks;
    int loopbit;
	int lps_precision = LPS_PREC;

	loops_per_jiffy = (1<<12);

	printk("Calibrating delay loop... ");
	while (loops_per_jiffy <<= 1) {
		/* wait for "start of" clock tick */
		ticks = jiffies;
		while (ticks == jiffies)
			/* nothing */;
		/* Go .. */
		ticks = jiffies;
		__delay(loops_per_jiffy);
		ticks = jiffies - ticks;
		if (ticks)
			break;
	}

/* Do a binary approximation to get loops_per_second set to equal one clock
   (up to lps_precision bits) */
	loops_per_jiffy >>= 1;
	loopbit = loops_per_jiffy;
	while ( lps_precision-- && (loopbit >>= 1) ) {
		loops_per_jiffy |= loopbit;
		ticks = jiffies;
		while (ticks == jiffies);
		ticks = jiffies;
		__delay(loops_per_jiffy);
		if (jiffies != ticks)	/* longer than 1 tick */
			loops_per_jiffy &= ~loopbit;
	}


/* Round the value and print it */	
	printk("%lu.%02lu BogoMIPS\n",
        loops_per_jiffy/(500000/HZ),
        (loops_per_jiffy/(5000/HZ)) % 100);
}

static int __init readonly(char *str)
{
	if (*str)
		return 0;
	root_mountflags |= MS_RDONLY;
	return 1;
}

static int __init readwrite(char *str)
{
	if (*str)
		return 0;
	root_mountflags &= ~MS_RDONLY;
	return 1;
}

static int __init debug_kernel(char *str)
{
	if (*str)
		return 0;
	console_loglevel = 10;
	return 1;
}

static int __init quiet_kernel(char *str)
{
	if (*str)
		return 0;
	console_loglevel = 4;
	return 1;
}

__setup("ro", readonly);
__setup("rw", readwrite);
__setup("debug", debug_kernel);
__setup("quiet", quiet_kernel);

/*
 * This is a simple kernel command line parsing function: it parses
 * the command line, and fills in the arguments/environment to init
 * as appropriate. Any cmd-line option is taken to be an environment
 * variable if it contains the character '='.
 *
 * This routine also checks for options meant for the kernel.
 * These options are not given to init - they are for internal kernel use only.
 */
static void __init parse_options(char *line)
{
	char *next,*quote;
	int args, envs;

    printk(KERN_DEBUG "******** inside parse options.*******\n");

	if (!*line)
		return;
	args = 0;
	envs = 1;	/* TERM is set to 'linux' by default */
	next = line;
	while ((line = next) != NULL) {
                quote = strchr(line,'"');
                next = strchr(line, ' ');
                while (next != NULL && quote != NULL && quote < next) {
                        /* we found a left quote before the next blank
                         * now we have to find the matching right quote
                         */
                        next = strchr(quote+1, '"');
                        if (next != NULL) {
                                quote = strchr(next+1, '"');
                                next = strchr(next+1, ' ');
                        }
                }
                if (next != NULL)
                        *next++ = 0;
		if (!strncmp(line,"init=",5)) {
			line += 5;
			execute_command = line;
			/* In case LILO is going to boot us with default command line,
			 * it prepends "auto" before the whole cmdline which makes
			 * the shell think it should execute a script with such name.
			 * So we ignore all arguments entered _before_ init=... [MJ]
			 */
			args = 0;
			continue;
		}
        /* Ishoni ipservice testing */
        printk(KERN_DEBUG "calling checksetup\n");
		if (checksetup(line))
			continue;
		
		/*
		 * Then check if it's an environment variable or
		 * an option.
		 */
		if (strchr(line,'=')) {
			if (envs >= MAX_INIT_ENVS)
				break;
			envp_init[++envs] = line;
		} else {
			if (args >= MAX_INIT_ARGS)
				break;
			if (*line)
				argv_init[++args] = line;
		}
	}
	argv_init[args+1] = NULL;
	envp_init[envs+1] = NULL;
}


extern void setup_arch(char **);
extern void cpu_idle(void);


#ifndef CONFIG_SMP

#ifdef CONFIG_X86_IO_APIC
static void __init smp_init(void)
{
	IO_APIC_init_uniprocessor();
}
#else
#define smp_init()	do { } while (0)
#endif

#else

/* Called by boot processor to activate the rest. */
static void __init smp_init(void)
{
	/* Get other processors into their bootup holding patterns. */
	smp_boot_cpus();
	smp_threads_ready=1;
	smp_commence();
}		

#endif

/*
 *	Activate the first processor.
 */
 
asmlinkage void __init start_kernel(void)
{
	char * command_line;
	unsigned long mempages;

/*
 * Interrupts are still disabled. Do necessary setups, then
 * enable them
 */
	lock_kernel();
#ifdef  ISHONI_ENGINE  /* raghav */
        /* On IBE we emulate many of the instructions such as lwl, lwr *
         * mult and div. These would start getting called early during *
         * the boot process. So the handler should be installed much   *
         * earlier.
         */
        trap_init();
#endif
	setup_arch(&command_line);


	printk(linux_banner);
    printk(KERN_DEBUG "Calling parse options....");
	parse_options(command_line);

#ifndef ISHONI_ENGINE
	trap_init();
#endif

	init_IRQ();
	sched_init();
	time_init();
	softirq_init();

	/*
	 * HACK ALERT! This is early. We're enabling the console before
	 * we've done PCI setups etc, and console_init() must be aware of
	 * this. But we do want output early, in case something goes wrong.
	 */
	console_init();
#ifdef CONFIG_MODULES
	init_modules();
#endif
	if (prof_shift) {
		unsigned int size;
		/* only text is profiled */
		prof_len = (unsigned long) &_etext - (unsigned long) &_stext;
		prof_len >>= prof_shift;
		
		size = prof_len * sizeof(unsigned int) + PAGE_SIZE-1;
		prof_buffer = (unsigned int *) alloc_bootmem(size);
	}

	kmem_cache_init();
	sti();
	calibrate_delay();
#ifdef CONFIG_BLK_DEV_INITRD
	if (initrd_start && !initrd_below_start_ok &&
			initrd_start < min_low_pfn << PAGE_SHIFT) {
		printk(KERN_CRIT "initrd overwritten (0x%08lx < 0x%08lx) - "
		    "disabling it.\n",initrd_start,min_low_pfn << PAGE_SHIFT);
		initrd_start = 0;
	}
#endif
#ifdef CONFIG_BINFMT_IRIX
	init_inventory();
#endif
	mem_init();
	kmem_cache_sizes_init();
#ifdef CONFIG_3215_CONSOLE
        con3215_activate();
#endif
#ifdef CONFIG_PROC_FS
	proc_root_init();
#endif
	mempages = num_physpages;

	fork_init(mempages);
	proc_caches_init();
	vfs_caches_init(mempages);
	buffer_init(mempages);
	page_cache_init(mempages);
	kiobuf_setup();
	signals_init();
	bdev_init();
	inode_init(mempages);
#if defined(CONFIG_SYSVIPC)
	ipc_init();
#endif
#if defined(CONFIG_QUOTA)
	dquot_init_hash();
#endif
	check_bugs();
	printk("POSIX conformance testing by UNIFIX\n");

	/* 
	 *	We count on the initial thread going ok 
	 *	Like idlers init is an unlocked kernel thread, which will
	 *	make syscalls (and thus be locked).
	 */
	smp_init();

#ifdef ISHONI_ENGINE /* NAZIM, TIMER */
	ibeEnableNetPICinterrupt();
#endif

	kernel_thread(init, NULL, CLONE_FS | CLONE_FILES | CLONE_SIGNAL);
	unlock_kernel();
	current->need_resched = 1;
 	cpu_idle();
}

#ifdef CONFIG_BLK_DEV_INITRD
static int do_linuxrc(void * shell)
{
	static char *argv[] = { "linuxrc", NULL, };

	close(0);close(1);close(2);
	setsid();
	(void) open("/dev/console",O_RDWR,0);
	(void) dup(0);
	(void) dup(0);
	return execve(shell, argv, envp_init);
}

#endif

struct task_struct *child_reaper = &init_task;

static void __init do_initcalls(void)
{
	initcall_t *call;

	call = &__initcall_start;
	do {
		(*call)();
		call++;
	} while (call < &__initcall_end);

    /* Make sure there is no pending stuff from the initcall sequence */
    flush_scheduled_tasks(); 
}

/*
 * Ok, the machine is now initialized. None of the devices
 * have been touched yet, but the CPU subsystem is up and
 * running, and memory and process management works.
 *
 * Now we can finally start doing some real work..
 */
static void __init do_basic_setup(void)
{
#ifdef CONFIG_BLK_DEV_INITRD
	int real_root_mountflags;
#endif

	/*
	 * Tell the world that we're going to be the grim
	 * reaper of innocent orphaned children.
	 *
	 * We don't want people to have to make incorrect
	 * assumptions about where in the task array this
	 * can be found.
	 */
	child_reaper = current;

#if defined(CONFIG_MTRR)	/* Do this after SMP initialization */
/*
 * We should probably create some architecture-dependent "fixup after
 * everything is up" style function where this would belong better
 * than in init/main.c..
 */
	mtrr_init();
#endif

#ifdef CONFIG_SYSCTL
	sysctl_init();
#endif

	/*
	 * Ok, at this point all CPU's should be initialized, so
	 * we can start looking into devices..
	 */
#ifdef CONFIG_PCI
	pci_init();
#endif
#ifdef CONFIG_SBUS
	sbus_init();
#endif
#if defined(CONFIG_PPC)
	ppc_init();
#endif
#ifdef CONFIG_MCA
	mca_init();
#endif
#ifdef CONFIG_ARCH_ACORN
	ecard_init();
#endif
#ifdef CONFIG_ZORRO
	zorro_init();
#endif
#ifdef CONFIG_DIO
	dio_init();
#endif
#ifdef CONFIG_NUBUS
	nubus_init();
#endif
#ifdef CONFIG_ISAPNP
	isapnp_init();
#endif
#ifdef CONFIG_TC
	tc_init();
#endif
#ifdef CONFIG_QTRONIX_KEYBOARD
    init_qtronix_990P_kbd();
#endif

	/* Networking initialization needs a process context */ 
    sock_init();

#ifdef CONFIG_BLK_DEV_INITRD
	real_root_dev = ROOT_DEV;
	real_root_mountflags = root_mountflags;
    if (initrd_start && mount_initrd) root_mountflags &= ~MS_RDONLY;
    else mount_initrd =0;
#endif

	start_context_thread();
#ifdef ISHONI_ROOT_NFS
	real_root_dev = ROOT_DEV;
#endif
    do_initcalls();

	/* .. filesystems .. */
	filesystem_setup();

#ifdef CONFIG_IRDA
	irda_device_init(); /* Must be done after protocol initialization */
#endif
#ifdef CONFIG_PCMCIA
	init_pcmcia_ds();		/* Do this last */
#endif


#if defined(ISHONI_ENGINE) && !defined(LINUX_BOOT_LOADER)
	ROOT_DEV = name_to_kdev_t(ROOT_DEV_NAME);
    printk(KERN_DEBUG "ROOT device is %x\n", ROOT_DEV);
#endif /* ISHONI_ENGINE */
	/* Mount the root filesystem.. */
    mount_root();

	mount_devfs_fs ();

#if defined(ISHONI_ENGINE) && !defined(LINUX_BOOT_LOADER)
#if ((ISHONI_FLASH == 4) || (ISHONI_FLASH == 5) || (ISHONI_FLASH == 6))
    {
        int status, Fd;
#if defined(ISHONI_RAW_FLASH_CONFIG) 
      /*Do Nothing : Dont Mount JFFS2, Mount RAMFS */
        if((status = sys_mount(NULL, "/ramfs","ramfs", MS_SYNCHRONOUS, NULL)) < 0)
            printk("Warning: Unable to mount RAMFS file system, status = %d\n", status);
        else {
        if ((status= sys_mkdir("/ramfs/tmp", 0777)) < 0)
            printk("Warning: Unable to mkdir /ramfs/tmp, status = %d\n", status);
        }
#else
        if((status = sys_mount(JFFS2_BLK_DEV,"/jffs2","jffs2", MS_SYNCHRONOUS , NULL)) < 0)
            printk("Warning: Unable to mount JFFS file system, status = %d\n", status);
#endif
        /* write the flash parameters */
        if (saveFlashParamsFlag == 1) {
		    if ((Fd = open(FLASH_BOOT_DEV, O_RDWR, 0)) < 0)
	           printk("\n\rWarning - Could not open flash device %s\n", FLASH_BOOT_DEV);
	        else {
	           if (eraseFlash0(Fd, START_OF_FLASH_PARAM_ADDR, 1) < 0) {
	              printk("\n\rWarning - Could not erase flash device %s\nFlash database save unsuccessful\n", 
                          FLASH_BOOT_DEV);
                  close(Fd);
               }
	           else {
	              if (programImage(FLASH_BOOT_DEV, START_OF_FLASH_PARAM_ADDR, 
	                (char *)&flashParamStruct, sizeof(btParamStruct)) < 0) {
	                    printk("\n\rWarning - Write of flash database unsuccessful\n");
                  }
                  close(Fd);
	           }
	        }
        }

#ifdef NEW_FEATURES
        /* write flash partition table */
        if (saveFlashPartitionTblFlag == 1) {
		    if ((Fd = open(FLASH_BOOT_DEV, O_RDWR, 0)) < 0)
	           printk("\n\rWarning - Could not open flash device %s\n", FLASH_BOOT_DEV);
	        else {
	           if (eraseFlash0(Fd, START_OF_FLASH_PARTITION_TBL, 1) < 0) {
	              printk("\n\rWarning - Could not erase flash device %s\nFlash partition table save unsuccessful\n", 
                          FLASH_BOOT_DEV);
                  close(Fd);
               }
	           else {
	              if (programImage(FLASH_BOOT_DEV, START_OF_FLASH_PARTITION_TBL, 
	                (char *)&flashPartitionTblStruct, sizeof(flashMapStructure)) < 0) {
	                    printk("\n\rWarning - Write of flash database unsuccessful\n");
                  }
                  close(Fd);
	           }
	        }
        }
#endif
    }
#endif
    LX_DBG("Initializing function driver ....\n");
    init_fndriver();
#endif  /* #ifdef ISHONI_ENGINE */

#ifdef CONFIG_BLK_DEV_INITRD
	root_mountflags = real_root_mountflags;
	if (mount_initrd && ROOT_DEV != real_root_dev
	    && MAJOR(ROOT_DEV) == RAMDISK_MAJOR && MINOR(ROOT_DEV) == 0) {
		int error;
		int i, pid;

		pid = kernel_thread(do_linuxrc, "/linuxrc", SIGCHLD);
		if (pid>0)
			while (pid != wait(&i));
		if (MAJOR(real_root_dev) != RAMDISK_MAJOR
		     || MINOR(real_root_dev) != 0) {
			error = change_root(real_root_dev,"/initrd");
			if (error)
				printk(KERN_ERR "Change root to /initrd: "
				    "error %d\n",error);
		}
	}
#endif
}

#ifdef ISHONI_ENGINE /* NAZIM, MULTIPLE_CPU_INT */
#ifndef ISHONI_RAW_FLASH_CONFIG_USPACE
extern void usrAppInit(void);
#endif
#endif

#ifdef TEST_FLASH
extern unsigned char initFlashAmd(void);
extern void amd29_reset(void *dv);
extern unsigned char amd29_read(void *dv,
                                int    sector,
                                int  offset,
                                void * buf,
                                int bufSize);
extern unsigned char amd29_write(void *dv,
                                 int    sector,
                                 int  offset,
                                 void * buf,
                                 int bufSize);
char fb[40];
char fb2[10] = "gggggg";

void flash_thread(void)
{
        void *dv;
        unsigned int fl_wait=10000;

        dv = initFlashAmd();
        amd29_write(dv,25,0x4540,fb2,strlen(fb2));
        printk("---------- before %s\n",fb);
        amd29_read(dv,25,0x4540,fb,strlen(fb2));
        printk("---------- got %s\n",fb);

        amd29_erase(dv,25);

        amd29_read(dv,25,0x4540,fb,strlen(fb2));
        printk("---------- after erase got %s\n",fb);
        strcpy(fb2,"egnahc");
        fl_wait = 1000000; while(fl_wait--);
        amd29_write(dv,25,0x4540,fb2,strlen(fb2));
        printk("---------- before %s\n",fb);
        fl_wait = 1000000; while(fl_wait--);
        amd29_read(dv,25,0x4540,fb,strlen(fb2));
        printk("\n");
        printk("---------- got %s\n",fb);
}

extern int init_amd29_mtd(int); 
#endif /* TEST_FLASH */

static void open_console (void)
{
#if defined(ISHONI_ENGINE) && defined(LINUX_BOOT_LOADER)
	extern long sys_ioctl(unsigned int ,unsigned int ,unsigned long );
	struct termio t;
	int fd;

	if ((fd = open("/dev/console", O_RDWR, 0)) < 0)
		printk("Warning: unable to open an initial console.\n");
	sys_ioctl(fd, TCGETA, (int)&t);
	t.c_cc[VERASE] = 0x8; /* back space */
	sys_ioctl(fd, TCSETA, (int)&t);
#else
	if (open("/dev/console", O_RDWR, 0) < 0)
		printk("Warning: unable to open an initial console.\n");
#endif
	(void) dup(0);
	(void) dup(0);
}

#ifdef ISHONI_ROOT_NFS

extern int __init ip_auto_config(void);
#define RAMFS_MOUNT_POINT	"/ramfs"
#define OLDROOT_MOUNT_POINT	"/oldroot"

static void __init detach_vfsmnt(struct vfsmount *mnt)
{
	list_del(&mnt->mnt_clash);
	list_del(&mnt->mnt_child);
	if (mnt->mnt_parent != mnt)
		mntput(mnt->mnt_parent);
	dput(mnt->mnt_mountpoint);
	/*list_del(&mnt->mnt_list);*/
}

static void __init graft_vfsmnt (struct nameidata *nd, struct vfsmount *mnt)
{
	mnt->mnt_mountpoint = dget(nd->dentry);
	mnt->mnt_parent = mntget(nd->mnt);
	list_add(&mnt->mnt_child, &nd->mnt->mnt_mounts);
	list_add(&mnt->mnt_clash, &nd->dentry->d_vfsmnt);
	/*list_add(&mnt->mnt_list, vfsmntlist.prev);*/
}

static int __init open_mountpoint (const char *dir_name, struct nameidata *nd)
{
	int retval = 0;

	if (path_init(dir_name,
		      LOOKUP_FOLLOW|LOOKUP_POSITIVE|LOOKUP_DIRECTORY, nd))
		retval = path_walk(dir_name, nd);

	if (retval)
		return retval;

	if (!nd->dentry->d_inode)
		return -ENOENT;

	return 0;
}

__init void ibe_mount_root_nfs (void)
{
	int error;
	struct nameidata nd;
	struct vfsmount *vfsmnt;
	
	if (real_root_dev && real_root_dev != ROOT_DEV) {
		printk ("nfsroot option specified on kernel command line.\n");
		/* Close console */
		close (0); close (1); close (2);

		if (open_mountpoint (RAMFS_MOUNT_POINT, &nd))
			panic ("Unable to get the ramfs mountpoint!\n"
				"Your root filesystem (on the flash)"
				" seems to be corrupted. \n");
		if (nd.dentry != nd.mnt->mnt_root)
			panic ("ramfs not mounted!\n");

		vfsmnt = nd.mnt;
		detach_vfsmnt (vfsmnt);
		path_release (&nd);

		ip_auto_config ();
		error = change_root (name_to_kdev_t ("nfs"), OLDROOT_MOUNT_POINT);
		if (error)
			printk ("ROOT NFS: change root error: %d\n", error);

		if (open_mountpoint (RAMFS_MOUNT_POINT, &nd))
			panic ("Unable to get the ramfs mountpoint!\n" 
				"Your **NFS root filesystem** seems to be corrupted\n");

		graft_vfsmnt (&nd, vfsmnt);
		path_release (&nd);
		open_console ();
	}
}
#endif

static int init(void * unused)
{

	lock_kernel();
	printk(KERN_DEBUG "****************** Starting basic setup *****************\n");
	do_basic_setup();
	
	/*
	 * Ok, we have completed the initial bootup, and
	 * we're essentially up and running. Get rid of the
	 * initmem segments and start the user-mode stuff..
	 */
#ifndef ISHONI_ROOT_NFS
	free_initmem();
#endif
	unlock_kernel();
	open_console ();

#ifdef ISHONI_ENGINE /* NAZIM, MULTIPLE_CPU_INT */
    /* Boot CPU1 now ! */
    /* Now we can call bootNextCpu(), anywhere.
     * Have to see whether it requires some other initialization
     * on CPU0.
     */
    /* TODO: correct ISHONI_LOCALCALL */

#ifdef BOOT_TIME_DEFAULTS
    {
        char resetCntStr[20];
        extern unsigned int resetCount; /* defined in arch/mips/lx.c */

        sprintf(resetCntStr, "%d", resetCount);
        /*
         * Keep this variable name in sync with the name in rfc_db.c
         * TODO: choose an appropriate GROUP ID for the variable
         */
        dbDeclare("resetCount", 0, "%d", resetCntStr, NULL, 0, 10);
    }
#endif /* BOOT_TIME_DEFAULTS */

#if defined(INCLUDE_ISHONI_BRIDGE) || defined(INCLUDE_ISHONI_EXPIP) || defined(ISHONI_LOCALCALL)
#ifndef ISHONI_RAW_FLASH_CONFIG_USPACE
    usrAppInit();
#endif
#ifdef ISHONI_ROOT_NFS
    free_initmem();
#endif
#endif 
#endif  /* ifdef ISHONI_ENGINE */

#ifdef ISHONI_ENGINE

#ifdef ISHONI_KSHELL
    printk("\n----Make configured with ISHONI-KSHELL-----\nInvoking KSHELL\n");
    shell_main(0, NULL);
#endif


#ifdef LINUX_BOOT_LOADER
    sys_syslog(7, 0, 0);

    /* kernel_thread(bs_main, NULL, CLONE_FS | CLONE_FILES | CLONE_SIGNAL); */
    /* Just jump to bs_main for now */
    bs_main();  
	panic("Kernel panic - returning from bootloader console");
#endif /* LINUX_BOOT_LOADER */


#endif  /* ISHONI_ENGINE */


#if defined(ISHONI_ENGINE) && defined(LINUX_VOICE)
#ifndef EMUL_BRD 
#ifndef ISHONI_RAW_FLASH_CONFIG_USPACE
    init_voice();
#endif
#endif
    /* ssv_main(0,NULL); */
#endif



	/*
	 * We try each of these until one succeeds.
	 *
	 * The Bourne shell can be used instead of init if we are 
	 * trying to recover a really broken machine.
	 */

	if (execute_command)
		execve(execute_command,argv_init,envp_init);
	execve("/sbin/init",argv_init,envp_init);
	execve("/etc/init",argv_init,envp_init);
	execve("/bin/init",argv_init,envp_init);
	execve("/bin/sh",argv_init,envp_init);

	panic("No init found.  Try passing init= option to kernel.");
}


#ifdef LINUX_VOICE
void init_voice(void)
{	
#ifndef SIGHLR_ON_CPU2
   registerAmdSlac();
#endif /* SIGHLR_ON_CPU2 */
   if (rca_Initialize(RCA_STD_IF_TYPE) == RCA_ERROR)/* RCA Iface is Standard */
    {
                printk(" rca_Initialize returned error");
                return;
    }
/*   slac_setHwClkType(1); */ 
    printk("Voice: initialzed. Ready for applications\n");
}
#endif

