#!/bin/sh

# Usage of this file:
# ishoni-isk.sh operation(COPY/BUILD) [name of file being operated on]

DEBUG()
{
	echo $1
}

# Function that creates the top level directory and then copies the file
make_and_copy()
{
	f=$1
        cd `dirname $f`
 	export t=`pwd -P`
	if [ -n "$VIEW_ROOT" ]
	then
		t2=`echo $t | sed "s:$VIEW_ROOT::g"`
	else
		t2=$t
	fi
	mkdir -p $STAGE_DIR/$t2
	cd -
	if [ ! -a $f ]
	then
		cp -df $f $STAGE_DIR/$t2
	fi
# Take care to copy the Original file if it is a link
	if [ -L $f ] 
	then
		DEBUG "Resolving the Link $f"
		copy_link_to_staging $f
	fi
}

copy_dir()
{
    local d=$1
    DEBUG "Copying Directory $d"
    if [ -n "$VIEW_ROOT" ]
    then
	    t=`echo $d | sed "s:$VIEW_ROOT::g"`
	    base=`dirname $t`
    else
	    base=`dirname $d`
    fi
    mkdir -p $STAGE_DIR/$base
    cp -rfd $d $STAGE_DIR/$base
    echo -n "Copying Links........."
    for file in `find $d -type l` ; do
        copy_link_to_staging $file
    done
    echo "Done."
}

# Small bug with this code. A "." before the last path  such as "xxx/yy/./x.c"
# will not remove the "." This does not cause any problem with the below
# code however
convert_to_absolute()
{
        fname=$1
# If the input name starts with "/", then it is absolute. Make absolute path
# as / and proceed with the rest
        if [ `echo $fname | cut -b1` == "/" ] ; then
        	absolute="/"
		fname=`echo $fname | cut -b2-`
	else
        	absolute=`pwd`
        fi
# Logic: Take the input name and keep on stripping the MSB path. Say you have
# ../tmp/xx. Then fpath will have ../tmp/xx, tmp/xx and finally xx(for this
# dirname will return ." comp will contain the MSB path .., then tmp.
# If ".." go back by doing a dirname on absolute, else if "." do nothing;
# else add comp to absolute.(absolute is made as pwd) At the end append the
# file name
        while [ `dirname $fname` != "." ] ; do
                comp=`echo $fname | cut -d"/" -f1`
                if [ "$comp" == ".." ]; then
                        absolute=`dirname $absolute`
                else if [ "$comp" != "." ]; then
                        absolute=$absolute/$comp
                fi
                fi
                fname=`echo $fname | cut -d"/" -f2-`
        done
        absolute=$absolute/$fname
#Strip 2 continous /
	absolute=`echo $absolute | sed 's/\/\//\//g'`
        echo $absolute
} 

copy_link_to_staging()
{
	file=$1
	if [ -n "$VIEW_ROOT" ]
	then
		file2=`echo $file | sed "s:$VIEW_ROOT::g"`
	else
		file2=$file
	fi
	cd $STAGE_DIR/`dirname $file2`
	LINKNAME=`ls -l $file | awk '{print $11}'`
	DIRNAME=`dirname $LINKNAME`
	mkdir -p $DIRNAME
	cp -Lf $file $DIRNAME
	cd -
}

copy_build_vmlinux()
{
	DEBUG "COPYING VMLINUX DIRECTORY TO STAGING AREA $STAGE_DIR!!!"
	mkdir -p $STAGE_DIR/ibe_linux/src/mvista

# Follow all symbolic links while copying. This is a must since we have symbolic
# links across VOBs.
	copy_dir ${VIEW_ROOT}/ibe_linux/src/mvista/linux-2.4.2_hhl20

	rm -rf $STAGE_DIR/ibe_linux/src/mvista/linux-2.4.2_hhl20/drivers/ibe/wireless
	rm -rf $STAGE_DIR/ibe_linux/src/mvista/linux-2.4.2_hhl20/drivers/ibe/802dot11
	rm -rf $STAGE_DIR/ibe_linux/src/mvista/linux-2.4.2_hhl20/drivers/ibe/centillium
	rm -rf $STAGE_DIR/ibe_linux/src/mvista/linux-2.4.2_hhl20/drivers/ibe/alcatel
	rm -rf $STAGE_DIR/ibe_linux/src/mvista/linux-2.4.2_hhl20/drivers/ibe/wlan_drv
	mkdir -p $STAGE_DIR/netvoice/project/netvoice/src/wireless
	mkdir -p $STAGE_DIR/netvoice/project/netvoice/src/wan/802dot11
	mkdir -p $STAGE_DIR/netvoice/project/netvoice/src/wlan_drv
	if [ $ISHONI_DSL = 1 ]
	then
		mkdir -p $STAGE_DIR/netvoice/project/netvoice/src/dslmodem/centillium
		mkdir -p $STAGE_DIR/netvoice/project/netvoice/src/dslmodem/alcatel
		ln -s ../../../../../../netvoice/project/netvoice/src/dslmodem/centillium $STAGE_DIR/ibe_linux/src/mvista/linux-2.4.2_hhl20/drivers/ibe
		ln -s ../../../../../../netvoice/project/netvoice/src/dslmodem/alcatel $STAGE_DIR/ibe_linux/src/mvista/linux-2.4.2_hhl20/drivers/ibe
	fi
	ln -s ../../../../../../netvoice/project/netvoice/src/wireless $STAGE_DIR/ibe_linux/src/mvista/linux-2.4.2_hhl20/drivers/ibe/wireless
	ln -s ../../../../../../netvoice/project/netvoice/src/wan/802dot11 $STAGE_DIR/ibe_linux/src/mvista/linux-2.4.2_hhl20/drivers/ibe/802dot11
	ln -s ../../../../../../netvoice/project/netvoice/src/wlan_drv $STAGE_DIR/ibe_linux/src/mvista/linux-2.4.2_hhl20/drivers/ibe/wlan_drv

	if [ "$GDK_TYPE" = "REGULAR" ]
	then
	    for i in $ISHONI_AREA_VMLINUX_CDIR; do
		DEBUG "Removing source files in $STAGE_DIR/$i"
		find $STAGE_DIR/$i -name "*.[cshoS]" -exec rm -f \{} \;
	    done
	    for i in $ISHONI_AREA_VMLINUX_HDIR; do
		DEBUG "Removing header files in $STAGE_DIR/$i"
		find $STAGE_DIR/$i -name "*.h" -exec rm -f \{} \;
	    done
	fi
# The following is a soft link that gets copied because we dereference links
	rm -rf $STAGE_DIR/ibe_linux/src/mvista/linux-2.4.2_hhl20/include/asm

# Copy all the files in VMLINUX_AREA_HEADER_FILES
	for i in $VMLINUX_AREA_HEADER_FILES; do
		make_and_copy ${VIEW_ROOT}$i
	done
	for i in $VMLINUX_AREA_SRC_FILES; do
		make_and_copy ${VIEW_ROOT}$i
	done
	for file in `find ${VIEW_ROOT}/ibe_linux/src/mvista/linux-2.4.2_hhl20 -follow -name Makefile`; do
		make_and_copy $file
	done
}

copy_build_cpu1()
{
	DEBUG "COPYING CPU1 DIRECTORY TO STAGING AREA $STAGE_DIR!!!"
	mkdir -p $STAGE_DIR/netvoice/project/netvoice/cpu1bld
	DEBUG "Copying the CPU1 Makefiles..."
	for file in `find ${VIEW_ROOT}/netvoice/project/netvoice/cpu1bld -name Makefile`; do
		make_and_copy $file
	done
	cp -d ${VIEW_ROOT}/netvoice/project/netvoice/cpu1bld/* $STAGE_DIR/netvoice/project/netvoice/cpu1bld/
        if [ "$ISHONI_CHIP" = "typhoon" ]
	then
		cp -d ${VIEW_ROOT}/netvoice/project/netvoice/cpu1bld/cpu1/cpu1.ld $STAGE_DIR/netvoice/project/netvoice/cpu1bld/cpu1
	fi
	if [ "$ISHONI_CHIP" = "tsunami" ]
	then
		cp -d ${VIEW_ROOT}/netvoice/project/netvoice/cpu1bld/cpu1/tsunami_cpu1.ld $STAGE_DIR/netvoice/project/netvoice/cpu1bld/cpu1
	fi
	cp -d ${VIEW_ROOT}/netvoice/project/netvoice/cpu1bld/ifast_path/sched.o $STAGE_DIR/netvoice/project/netvoice/cpu1bld/ifast_path
}

copy_build_rootfs()
{
	DEBUG "COPYING ROOTFS DIRECTORY TO STAGING AREA $STAGE_DIR!!!"
	mkdir -p $STAGE_DIR/ibe_linux/src

	DEBUG "Copying top level Makefiles and scripts"
	cp -d ${VIEW_ROOT}/ibe_linux/src/* $STAGE_DIR/ibe_linux/src/

	DEBUG "Copying IBE directory Makefiles"
	for file in `find ${VIEW_ROOT}/ibe_linux/src/ibe -name Makefile`; do
		make_and_copy $file
	done

	DEBUG "Copying thirdparty source Makefiles"
	for file in `find ${VIEW_ROOT}/ibe_linux/src/thirdparty -name Makefile`; do
		make_and_copy $file
	done
	for file in `find ${VIEW_ROOT}/tpsrc/net/802.1x/src -name Makefile`; do
		make_and_copy $file
	done

	DEBUG "Copying VPN source Makefiles"
	for file in `find ${VIEW_ROOT}/netvoice/project/netvoice/src/vpn -name Makefile`; do
		make_and_copy $file
	done

	if [ $OPENSSL = 1 ]
	then
		DEBUG "Copying OPENSSL source Makefiles"
		for file in `find ${VIEW_ROOT}/netvoice/project/netvoice/thirdpartysrc/openssl -name Makefile`; do
			make_and_copy $file
		done
	fi

	DEBUG "Copying nms web handler source Makefiles"
	for file in `find ${VIEW_ROOT}/netvoice/project/netvoice/src/nms/web/hdlr_v2 -name Makefile`; do
		make_and_copy $file
	done

	DEBUG "Copying Mvista source Makefiles"
	for file in `find ${VIEW_ROOT}/ibe_linux/src/mvista/lib -name Makefile`; do
		make_and_copy $file
	done

	for i in $ROOTFS_AREA_SRC_FILES; do
		make_and_copy ${VIEW_ROOT}$i
	done

	for i in $ROOTFS_AREA_DIR; do
		copy_dir ${VIEW_ROOT}$i 
	done

	mkdir $STAGE_DIR/ibe_linux/bin
	mkdir $STAGE_DIR/ibe_linux/binaries
	mkdir $STAGE_DIR/ibe_linux/idk
	mkdir $STAGE_DIR/ibe_linux/lib
	mkdir $STAGE_DIR/ibe_linux/rootfs

# Darned case of web being a soft link... SHould be made avoidable later...
	ln -s ../../../../netvoice/project/netvoice/src/nms/web/ $STAGE_DIR/ibe_linux/src/ibe/nms/web
	if [ $INCLUDE_ISHONI_MGCP = 1 ]
	then
		ln -s ../../../../../netvoice/project/netvoice/src/callctrl/Mgcp/config $STAGE_DIR/ibe_linux/src/ibe/callctrl/mgcp/config
		mkdir -p $STAGE_DIR/netvoice/project/netvoice/src/callctrl/Mgcp/config
		ln -s ../../../../../netvoice/project/netvoice/src/callctrl/localcallproc $STAGE_DIR/ibe_linux/src/ibe/callctrl/mgcp/localcallproc
		mkdir -p $STAGE_DIR/netvoice/project/netvoice/src/callctrl/localcallproc
		ln -s ../../../../../netvoice/project/netvoice/src/callctrl/locallinectrl $STAGE_DIR/ibe_linux/src/ibe/callctrl/mgcp/locallinectrl
		mkdir -p $STAGE_DIR/netvoice/project/netvoice/src/callctrl/locallinectrl
	fi
	make_and_copy ${VIEW_ROOT}/netvoice/project/netvoice/src/nms/web/Makefile
	make_and_copy ${VIEW_ROOT}/netvoice/project/netvoice/src/nms/web/hdlr_v2/Makefile
	make_and_copy ${VIEW_ROOT}/netvoice/project/netvoice/src/nms/web/goahead/Makefile
	mkdir -p $STAGE_DIR/netvoice/project/netvoice/src/nms/web/hdlr_v2
    mkdir -p $STAGE_DIR/ibe_linux/src/include/linux
	cp  -rfd ${VIEW_ROOT}/netvoice/project/netvoice/src/nms/web/hdlr_v2/*.h $STAGE_DIR/netvoice/project/netvoice/src/nms/web/hdlr_v2
	make_and_copy ${VIEW_ROOT}/netvoice/project/netvoice/thirdpartysrc/goahead/Makefile
	mkdir -p $STAGE_DIR/netvoice/project/netvoice/src/nms/web/web-ui-target/common/ro-basic/html
	mkdir -p $STAGE_DIR/netvoice/project/netvoice/src/nms/web/web-ui-target
	mkdir -p $STAGE_DIR/netvoice/project/netvoice/src/nms/web/web-ui_v2
	(cd ${VIEW_ROOT}/netvoice/project/netvoice/src/nms/web/web-ui_v2; tar cf - * | (cd $STAGE_DIR/netvoice/project/netvoice/src/nms/web/web-ui_v2/; tar xf -))
	mkdir -p $STAGE_DIR/ibe_linux/src/ibe/nms/ums/gen

	make_and_copy ${VIEW_ROOT}/ibe_linux/src/mvista/Makefile
	mkdir -p $STAGE_DIR/ibe_linux/src/mvista/
	copy_dir ${VIEW_ROOT}/ibe_linux/src/mvista/lib
}

# Function that appends $1 into $2. This will be used when doing a make clean
# at the customer site.
append_tar()
{
	abs_obj=`convert_to_absolute $obj`
	if [ -n "$VIEW_ROOT" ]
	then
	    objfile=`echo $abs_obj | sed "s:$VIEW_ROOT/::g"`
	else
	    objfile=$abs_obj
	fi
	cd $VIEW_ROOT/
	tarfile1=$STAGE_DIR/$TOPDIR/customer.tar
	if [ -n "$VIEW_ROOT" ]
	then
	    tarfile=`echo $tarfile1 | sed "s:$VIEW_ROOT::g"`
	else
	    tarfile=$tarfile1
	fi
	tar rvf $tarfile $objfile
	DEBUG "Stuffed $objfile into $tarfile"
	cd -
}

do_build()
{
	DEBUG "BUILDING $FILE"  
	case $ISHONI_AREA in
		"VMLINUX") CDIR=$ISHONI_AREA_VMLINUX_CDIR
			   HDIR=$ISHONI_AREA_VMLINUX_HDIR
		shift;;
		"ROOTFS")  CDIR=$ISHONI_AREA_ROOTFS_CDIR
			   HDIR=$ISHONI_AREA_ROOTFS_HDIR
		shift;;
		"CPU1")    CDIR=$ISHONI_AREA_CPU1_CDIR
			   HDIR=$ISHONI_AREA_CPU1_HDIR
		shift;;
	esac

	abs_path=`convert_to_absolute $OPERATE_FILE`
#	DEBUG "Absolute file of operating file is $abs_path"
	if [ "$GDK_TYPE" = "REGULAR" ]
	then
	    diff .ibemacros_with_flags .ibemacros_without_flags 1>/dev/null 2>&1

	    # If the file differs then we must take in the source code and the dependencies
	    if [ $? -ne 0 ] ; then
		DEBUG "Taking source for $OPERATE_FILE"
		make_and_copy `pwd`/$OPERATE_FILE

		# Clean up the dependency file so that only all the header files are there
		cat /dev/null > .ibe_depend.dup
		for path in `cat .ibe_depend`; do
			echo $path | grep "\.h" >> .ibe_depend.dup
		done

		# Take every header file in depend file and check if the initial path matches
		for h1 in `cat .ibe_depend.dup`; do
			h2=`convert_to_absolute $h1`
#			DEBUG "Header $h2 matches for $OPERATE_FILE"
			make_and_copy $h2
		done
		# Since there are no component dependencies just copy the object file alone
	    else
		DEBUG "Taking object for $OPERATE_FILE"
		obj=`echo $OPERATE_FILE | sed 's/\.[c|s]/\.o/g'`
		append_tar $obj $STAGE_DIR/$TOPDIR/customer.tar
	    fi
	elif [ "$GDK_TYPE" = "SOURCE" ]
	then
	    DEBUG "Taking SOURCE for $OPERATE_FILE"
	    make_and_copy `pwd`/$OPERATE_FILE
	    	    
	    # Clean up the dependency file so that only all the header files are there
	    cat /dev/null > .ibe_depend.dup
	    DEBUG "Checking Dependency"
	    for path in `cat .ibe_depend`; do
		echo $path | grep "\.h" >> .ibe_depend.dup
	    done

	    # Take every header file in depend file and check if the initial path matches
	    for h1 in `cat .ibe_depend.dup`; do
		h2=`convert_to_absolute $h1`
#		DEBUG "Header $h2 matches for $OPERATE_FILE"
		make_and_copy $h2
	    done
	fi
}

# If build type is
# *LOCAL* Do nothing. Just get out.
# *IDK* Check the operation type.
#	*COPY*  Get what needs to be copied using ISHONI_AREA
#		*VMLINUX* Copy all directories except ISHONI directories
#		*CPU1* Not implemented
#		*ROOTFS* Copy all directories without any source files
#	*BUILD* Get what needs to be built using ISHONI_AREA
#		*VMLINUX* If compilation is in IBE directory then copy the
#			  source file with dependencies; else copy object file.
#
#
OPERATION=$1

DEBUG "BUILD TYPE = $BUILD_TYPE"
DEBUG "OPERATION = $OPERATION"
DEBUG "ISHONI AREA = $ISHONI_AREA"
DEBUG "GDK TYPE = $GDK_TYPE"
DEBUG "VIEW ROOT = $VIEW_ROOT"
case $BUILD_TYPE in
#
	LOCAL)	 shift;;
#
	IDK)	 case $OPERATION in
#	
			"COPY")	 case $ISHONI_AREA in
#
				"VMLINUX") copy_build_vmlinux
					   shift;;
				"ROOTFS") copy_build_rootfs
					   shift;;
				"CPU1") copy_build_cpu1
					   shift;;
				esac
			shift;;
#
			"BUILD") OPERATE_FILE=$2 
				do_build
			shift;;
		esac

	shift;;
esac
	
DEBUG "End of ishoni_idk.sh"
