#ifndef CHIPCONFIG_H
#define	CHIPCONFIG_H
/****************************************************************************
 *  ishoni Networks Confidential and Proprietary Copyright (c) 2000
 ****************************************************************************
 *	FILE:  chipConfig.h
 *
 *  ENGINEER:  B. Cheang
 *
 *  PROJECT:  ishoni Broadband Engines
 *
 *  DESCRIPTION:
 *	    Contains definitions on chip registres.
 *
 *	RULES:
 *
 ***************************************************************************/
#include <chipfamily.h>

#define MY_PROCESSOR_ID()  			(*(unsigned int *)0xa1100000 & 0x00000003)  /* CPU_ID */
#define MY_PROCESSOR_REV()			((*(unsigned int *)0xa1100000 & 0xF00) >> 8)		/* CPU_REV */

#define PLL_BYPASS                  0
#define SYS_CLK_PLL_CNTL            1
#define PCI_CLK_PLL_CNTL            2
#define CODEC_CLK_PLL_CNTL          3
#define USB_CLK_PLL_CNTL            4
#define CODEC_CLK_DIV_CNTL          5
#define MAC_CLK_DIV_CNTL            6
#define UART_CLK_DIV_CNTL           7
#define MAC_CONFIG                  8
#define EJTAG_PIN_MUX_CNTL          9
#define UART_HDLC_PIN_MUX_CNTL      10
#define MAC1_USB_PIN_MUX_CNTL       11
#define HDLC_PIN_MUX_CNTL           12
#define DEBUG_PORT_SELECT           13
#define PCI_CONFIG                  14
#define STRAPPED_VALUE              16

#define FRAMER_CLK_2050_CNTL        15  /* 2050 */
#define STRAPPED_2050_VALUE         16  /* 2050 */
#define POWER_DOWN_2050_CNTL        17  /* 2050 */
#define USB_RNDIS_MODE_2050_CNTL    18  /* 2050 */
#define MIP_CACHE_POWER_2050_CNTL   19  /* 2050 */

#define DEVICE_2050                  4  /* bit 8-11 of 0xa1100000 */ 

#define DEVICE_DV2000                0  /* bit 8-11 of 0xa1100000 */

#define ENABLE_USB_RNDIS   setUSBCntlReg(0,1)
#define DISABLE_USB_RNDIS  setUSBCntlReg(0,0)

#define ENABLE_CLK_MIPS2   setPwCntlReg(0,0)
#define DISABLE_CLK_MIPS2  setPwCntlReg(0,1)

#define ENABLE_CLK_MIPS3   setPwCntlReg(1,0)
#define DISABLE_CLK_MIPS3  setPwCntlReg(1,1)

#define ENABLE_CLK_MAC0    setPwCntlReg(2,0)
#define DISABLE_CLK_MAC0   setPwCntlReg(2,1)

#define ENABLE_CLK_MAC1    setPwCntlReg(3,0)
#define DISABLE_CLK_MAC1   setPwCntlReg(3,1)

#define ENABLE_CLK_USB     setPwCntlReg(3,0)
#define DISABLE_CLK_USB    setPwCntlReg(3,1)

#define ENABLE_CLK_HDLC    setPwCntlReg(4,0)
#define DISABLE_CLK_HDLC   setPwCntlReg(4,1)

#define ENABLE_MIP1_PW_SAVE    setCachePwCntlReg(0,1)
#define DISABLE_MIP1_PW_SAVE   setCachePwCntlReg(0,0)


/* Chip IDs for Ishoni's various silicons.*/
#define NETVOICE_CHIP_ID            0
#define CYCLONE_CHIP_ID             1
#define SIROCCO_CHIP_ID             3
#define TYPHOON_CHIP_ID             4
#define TSUNAMI_CHIP_ID             5

/* cache memory sizes for these silicons.*/
#define NETVOICE_ICACHE_SIZE          4*1024 
#define TSUNAMI_ICACHE_SIZE           8*1024 
#define CYCLONE_ICACHE_SIZE          16*1024
#define TYPHOON_ICACHE_SIZE          16*1024

/* Moved to chipfamily.h 
*
*#ifdef CYCLONE
*#define     BOOT_CPU                   2
*#else
*#define     BOOT_CPU                   0
*#endif
*/

/* Kuldeep : Need to cleanup this code */
#ifndef WIN32
        /* Used by iBE */

#ifndef EMUL_BRD
#define     REG(x)             (volatile unsigned int)*(unsigned int *)(0xA120D000 + 4*x)

#else
/* Tsunami/typhoon/cyclone returns 3 for MAC1 instead of 0 ( unlike in dv2000).
 * Moreover Tsunami/typhoon Emulation board is not having the PLL registers ( a120d000 series ).
 * So hardcoding to 3 
 */
#ifndef LINUX_BOOT_LOADER
#define     REG(x)             (((x) == MAC1_USB_PIN_MUX_CNTL)?3:(volatile unsigned int)*(unsigned int *)(0xA120D000 + 4*x))
#else
#define     REG(x)             (volatile unsigned int)*(unsigned int *)(0xA120D000 + 4*x)
#endif
#endif   /* #ifndef TSUNAMI */

#else

        /* For debug use only */
static unsigned int regVal[16] =
{ 0, 40, 0, 0, 0, 0, 0, 0, 0, 0xc, 0, 0, 0, 0, 0 };
#define     REG(x)             (unsigned int)*(unsigned int *)((char *)&regVal[0] + 4*x)

#endif   /* #ifndef WIN32 */

/* ========== Published API Prototyping Start ============================== */

unsigned int getMyCPUId (void);                                              /*. */

void getMyCPURev (                                                       /*. */
int *chipTypeP, /*. output, chip id value 0~15 when chipIDP != 0             */
int *chipRevP,  /*. output, chip rev no 0~15  when chipRevP != 0             */
char *strP);     /*. output, chip and rev string when strP != 0               */

void getSysClkConfig(                                                    /*. */
int  *freqP,    /*. output, 0=External clock, >0 calculated frequency (MHz)  */
char *strP);     /*. output, description string. If pointer value = 0,        */
                /*. output is inhibited.                                     */

void getEJTAGConfig(                                                     /*. */
int  *ejtag0CPU,/*. output, CPU number connected EJTAG 0, value 0 ~ 3        */
int  *ejtag1CPU);/*. output, CPU number connected EJTAG 1, value 0 ~ 3        */
                /*. If output pointer = 0, output is inhibited.              */

void getUART_HDLCConfig(                                                 /*. */
int  *port0P,   /*. output, 0=UART 0 exist; 1=HDLC 0 exist                   */
int  *port1P,   /*. output, 0=UART 1 exist; 1=HDLC 1 exist                   */
char *strP);     /*. output, description string                               */
                /*. output pointer = 0, output is inhibited                  */

void getMAC1_USBConfig(                                                  /*. */
int  *typeP,    /*. output, 00 = MAC1 port;   01 = USB on MAC1               */
                /*.         10 = Slave host;  11 = USB on chip transceiver   */
char *strP);     /*. output, description string                               */
                /*. output pointer = 0, output is inhibited                  */

void getATMConfig(                                                       /*. */
int  *typeP,    /*. output, 00 = Utopia/ATM;  01 = T1/E1 Framer              */
                /*.         10 = Terayon;     11 = Framer + HDLC             */
char *strP);     /*. output, description string                               */
                /*. output pointer = 0, output is inhibited                  */

unsigned int getPwCntlReg(void);
                                               /*. */
void setPwCntlReg(
int device,                  /*. device ID           */  
int operation);              /*. 0=enable. 1=disable */

unsigned int getCachePwCntlReg(void);                                               /*. */
void setCachePwCntlReg(
int device,                  /*. device ID  (0..7)   */  
int operation);              /*. 0=disable. 1=enable */

unsigned int getUSBCntlReg(void);                                               /*. */
void setUSBCntlReg(
int device,                  /*. device ID  (0)      */  
int operation);              /*. 0=disable. 1=enable */

void getChipIDString(char *idStrP);

unsigned int getFmrCntlReg(void);                                               /*. */

unsigned int getStrapValReg(); 

int getEEPROM85xxDBStatus(void) ;                                           /*. */
int getBoardInfo(                                                       /*.  */
char *boardInfoP,               /* Board Part Number (max 30 bytes)          */
char *serialNumP,               /* Serial Number (max 30 bytes)              */
unsigned int *majorP,           /* 0-9999                                    */      
unsigned int *minorP); 

int setBoardInfo(char *boardInfoP); 

int getUARTResource(                                                     /*. */
int *uart0,                 /*. 1: allocated, 0: not allocated               */
int *uart1,                 /*. 1: allocated, 0: not allocated               */
int *uart2); 


/* ========== Published API Prototyping End   ============================== */

#endif /* CHIPCONFIG_H */
