#ifndef DRV_UART_H
#define DRV_UART_H
/****************************************************************************
 *  ishoni Networks Confidential and Proprietary Copyright (c) 2000
 ****************************************************************************
 *  FILE:  drv_uart.h
 *
 *  ENGINEER:  steve kao
 *
 *  PROJECT:  iBE DV2000
 *
 *  DESCRIPTION:
 *	This file contains the UART ioctl definition that are givin to user to 
 *  modify the UART operation. 
 *
 ****************************************************************************/
/*     UART SET/GET command */
#define 	UART_BAUD_SET           0x1003
#define 	UART_BAUD_GET           0x1004

#define     UART_HW_OPTS_SET        0x1005
#define     UART_HW_OPTS_GET        0x1006

#define 	UART_MODE_SET           0x1007
#define 	UART_MODE_GET           0x1008 
#define 	UART_AVAIL_MODES_GET    0x1009

#define     UART_HUP                0x100A
#define     UART_OPEN               0x100B

#define     UART_ECHO_MODE_SET      0x100E
#define     UART_ECHO_MODE_GET      0x100F

#define     UART_DE_REGISTER        0x1010
#define     UART_REGISTER           0x1011

/*  UART_BAUD_SET/GET parameter  */
#define BAUD_RATE_2400              2400
#define BAUD_RATE_9600              9600
#define BAUD_RATE_19200             19200

/*  UART_MODE_SET/GET  parameter  */
#define POLLING_MODE                1
#define INTERRUPT_MODE              2

#ifndef  LINUX
/*  UART_HW_OPTS_SET/GET bit pattern */
#define CLOCAL		0x1	/* ignore modem status lines */
#define CREAD		0x2	/* enable device reciever */

#define CSIZE		0xc	/* bits 3 and 4 encode the character size */
#define CS5		0x0	/* 5 bits */
#define CS6		0x4	/* 6 bits */
#define CS7		0x8	/* 7 bits */
#define CS8		0xc	/* 8 bits */

#define HUPCL		0x10	/* hang up on last close */
#define STOPB		0x20	/* send two stop bits (else one) */
#define PARENB		0x40	/* parity detection enabled (else disabled) */
#define PARODD		0x80	/* odd parity  (else even) */

#else
#ifdef __KERNEL__
#ifndef __ASM_MIPS_TERMBITS_H /* in case termbits .h is not included */
#define CLOCAL		0x1	/* ignore modem status lines */
#define CREAD		0x2	/* enable device reciever */
#define CSIZE		0xc	/* bits 3 and 4 encode the character size */
#define CS5		0x0	/* 5 bits */
#define CS6		0x4	/* 6 bits */
#define CS7		0x8	/* 7 bits */
#define CS8		0xc	/* 8 bits */
#define HUPCL		0x10	/* hang up on last close */
#define STOPB		0x20	/* send two stop bits (else one) */
#define PARENB		0x40	/* parity detection enabled (else disabled) */
#define PARODD		0x80	/* odd parity  (else even) */
#endif
#endif /* ifdef __KERNEL__ */
#define IBE_SERIAL_CLOCAL		0x1	/* ignore modem status lines */
#define IBE_SERIAL_CREAD		0x2	/* enable device reciever */

#define IBE_SERIAL_CSIZE		0xc	/* bits 3 and 4 encode the character size */
#define IBE_SERIAL_CS5		0x0	/* 5 bits */
#define IBE_SERIAL_CS6		0x4	/* 6 bits */
#define IBE_SERIAL_CS7		0x8	/* 7 bits */
#define IBE_SERIAL_CS8		0xc	/* 8 bits */

#define IBE_SERIAL_HUPCL		0x10	/* hang up on last close */
#define IBE_SERIAL_STOPB		0x20	/* send two stop bits (else one) */
#define IBE_SERIAL_PARENB		0x40	/* parity detection enabled (else disabled) */
#define IBE_SERIAL_PARODD		0x80	/* odd parity  (else even) */

/* defined some error flags */
#define IBE_SERIAL_OERR		0x2   /* overflow error */
#define IBE_SERIAL_PERR		0x4   /* parity error */
#define IBE_SERIAL_FERR		0x8   /* frame error */

#endif


/*  UART_ECHO_MODE_SET/GET parameter */
#define ECHO_OFF    0x00
#define ECHO_ON     0x01

/* ishoni uart def.*/
#define UART_CLK                    12500000
#define BAUD_RATE_MAX               (UART_CLK / 16)
#define DEFAULT_BAUD_RATE           BAUD_RATE_9600 

#define UART_FIFO_SIZE              16

#ifndef TSUNAMI
#define MAX_NOF_UART_PORTS          3
#else
#define MAX_NOF_UART_PORTS          1
#endif
#define MAX_NOF_Q                   MAX_NOF_UART_PORTS * 2  

#define MAX_NOF_Q_PER_PORT          2
#define TX_Q_ID                     0
#define RX_Q_ID                     1

#define POLLING_MODE                1
#define INTERRUPT_MODE              2

#define NONE_FLOWCTRL               0
#define HW_FLOWCTRL                 1
#define SW_FLOWCTRL                 2

#define NO_PARITY                   0
#define ODD_PARITY                  1
#define EVEN_PARITY                 2
  
#define MAX_Q_SIZE                  1024


/* ========== Published API Prototyping Start ============================== */

void enableTx(                                                           /*. */
int port);              /*. uart port number. ex : 0, 1, 2                   */
int isPortAllocated(                                                     /*. */    
int port);              /*. uart port number. ex : 0, 1, 2                   */
int registerPort(                                                        /*. */
int port);              /*. uart port number. ex : 0, 1, 2                   */
int deRegisterPort(                                                      /*. */
int port);              /*. uart port number. ex : 0, 1, 2                   */
int setUARTmode(                                                         /*. */
int port,               /*. uart port number. ex : 0, 1, 2                   */
int mode);       	    /*. POLLING_MODE, INTERRUPT_MODE */
unsigned int setBaudRate(                                                /*. */
int port,               /*.  uart port number. ex : 0, 1, 2                  */
int baudRate);          /*. 9600, 19200,...115200                            */
int setUARTopts(                                                         /*. */ 
int port,               /*. port number 0,1,2                                */
int dataBits,           /*. Number of data bits ex: 7,8                      */                        
int stopBits,           /*. Number of stop bits. 1,,2.                       */
int parity,             /*. NO_PARITY, ODD_PARITY, EVEN_PARITY               */
int handshake);         /*. HW_FLOWCTRL, SW_FLOWCTRL                         */
int initUARTport(                                                        /*. */ 
int port,               /*. UART port number : 0, 1, 2                       */
int baudRate,           /*. 9600, 19200, 38400, etc.,                        */
int parity,             /*. NO_PARITY, ODD_PARITY, EVEN_PARITY               */
int handshake,          /*. HW_FLOWCTRL, SW_FLOWCTRL                         */
int dataBits,           /*. Number of data bits ex: 7,8                      */                        
int stopBits,           /*. Number of stop bits. 1,,2.                       */
int uartMode);          /*. POLLING_MODE or INTERRUPT_MODE                   */
#ifndef LINUX
void initUARTDriver();                                                   /*. */  
#else
void initUARTDriver(void);
#endif


int setRxIntHandler(                                                     /*. */ 
int port,               /*. uart port number : 0, 1, 2                       */
#ifndef LINUX
void (*rxCallBk)(unsigned char data));/*. call back function address which   */ 
                        /*. load UART RX data into user buffer directly      */  
#else
void (*rxCallBk)(char, int, unsigned char, unsigned char));
#endif

int getRxIntHandler(                                                     /*. */ 
int port,               /*. uart port number : 0, 1, 2                       */
unsigned int *fAddr);   /*. storage for old callback function address        */ 
int put_char(                                                            /*. */
int port,               /*. port 0,1,2  ??? swap position                    */
int data);              /*. the output byte                                  */                 
int put_char_to_Q(                                                       /*. */ 
int  port,              /*. UART port number:0,1,2                           */
char data);             /*. the output byte                                  */                 
int put_string(                                                          /*. */
int port,               /*. port number 0,1,2                                */
unsigned char *string); /*. string end with NULL char                        */                 
int get_char(                                                            /*. */  
int port);              /*. UART port number : 0, 1, 2                       */
int get_string(                                                          /*. */                                 
int  port,              /*. UART port number : 0, 1, 2                       */
char *string,           /*. pointer to user's string location                */ 
int  maxLength);        /*. length of string + null char                     */
#ifndef LINUX
void txUARTdata( );                                                      /*. */
void rxUARTdata( );                                                      /*. */
#else
void txUARTdata(void);                                                      /*. */
void rxUARTdata(void);                                                      /*. */
#endif
void putStrPoll(                                                         /*. */
int  port,              /*.port 0,1,2                                        */
char *strP);            /*.pointer to NULL terminated string                 */
int getStrPoll(                                                          /*. */
int  port,              /*. port 0,1,2                                       */
char *strP,             /*.pointer to buffer to receive the data             */
int len);               /*.Max number of bytes that the buffer strP can hold */ 
int getUartPerf(                                                         /*. */
int  port,              /*. port 0,1,2                                       */
int  action,            /*. 0 = reset counter                                */
unsigned int *RxChar,   /*. number of byte received                          */
unsigned int *TxChar,   /*. number of byte sent                              */ 
unsigned int *RxError,  /*. number of receiving error                        */
unsigned int *RxQueFull);/*. receive Q status                                */
int getUartSetting(                                                      /*. */
int  port,              /*.port 0,1,2                                        */
unsigned int *BaudRate, /*.baudrate                                          */
unsigned int *Service,  /*.1 = in service                                    */ 
unsigned int *Mode,     /*.1 = polling, 2 = interrupt                        */
unsigned int *ChSize,   /*.7: 7 bits/char; 8: 8bits/char                     */
unsigned int *Parity,   /*."E":even, "O":odd, "N":no parity                  */
unsigned int *StopBits);/*.number of stop bits                               */
void moveCursor(                                                         /*. */
int port,               /*. Uart port number: 0,1,2 only                     */
int row,                /*. row number on the screen: 1..30                  */
int col);               /*. colum number on the screen 1..132                */
void clearScreen(                                                        /*. */
int port);              /*. Uart port number: 0,1,2 only                     */
void reverseVideoOn(                                                     /*. */
int port);              /*. Uart port number: 0,1,2 only                     */
void highLight(                                                          /*. */
int port);              /*. Uart port number: 0,1,2 only                     */
void normalVideo(                                                        /*. */
int port);              /*. Uart port number: 0,1,2 only                     */
void eraseLine(                                                          /*. */                
int port);              /*. Uart port number: 0,1,2 only                     */
int uartIoctl (         /*. see uart.h for definition                        */
int      port,          /*. port number: 0, OK, 2                            */
int      request,       /*. request code                                     */
int      *option);      /*. option code                                      */
int loopbackUARTtest(                                                    /*. */
int port);      /*. uart port number 0,1,2                                   */


int getUARTopts(                                                 /*. */ 
int port,       /*. port number 0,1,2                                */
int *dataBits,  /*. Number of data bits ex: 7,8                      */                        
int *stopBits,  /*. Number of stop bits. 1,,2.                       */
int *parity,    /*. 0=NO_PARITY, 1=ODD_PARITY, 2=EVEN_PARITY         */
int *handshake); 



/* ========== Published API Prototyping End   ============================== */
int uartConnectDir(void);

int dirFileToIshoniUartQ(char rxData);
#ifdef LINUX

void disableTx(int);
int processUartHWopts (int, unsigned int);
typedef struct
{
unsigned char *rd;                   /* queue read pointer */
unsigned char *wr;                   /* queue write pointer */
unsigned char *twr;                  /* queue temporary write pointer */
unsigned char buf[MAX_Q_SIZE];       /* the body of the queue */
}Q_STRUCT;


typedef struct
{                               
    unsigned int    phyAddr;  /* port physical address                         */

    unsigned int    chSize;   /* bits per char : 7: 7 bits/char; 8: 8bits/char */
    unsigned int    parity;   /* 0=NO_PARITY, 1=ODD_PARITY, 2=EVEN_PARITY      */                             
    unsigned int    stopBits; /* number of stop bits of the port               */
    unsigned int    options;  /* chSize + parity + stopBits                    */

    unsigned int    baudRate;
    unsigned int    mode    ; /* UART mode - POLLING_MODE INTERRUPT_MODE       */
    unsigned int    handshake;/* 0=NONE_FLOWCTRL, 1=HW_FLOWCTRL, 2=SW_FLOWCTRL */

    unsigned int    inService;/* identify service for multi-CPU/multi-UART sys */

    unsigned int    echoMode; /* 0: noEcho, 1: raw echo,  3: echo (special)    */ 
    unsigned int    echoHead;
    unsigned int    echoTail;
    unsigned char   echoBuf[UART_FIFO_SIZE];

    Q_STRUCT    queue[MAX_NOF_Q_PER_PORT]; 
                               /* stores the received data and data to
                                  to be xmitted out.
                                */
                                
    unsigned int    txQid;     /* the I/O queue id for tx                      */
    unsigned int    rxQid;     /* the I/O queue id for rx                      */
    
    void  (*txISRaddr)(int port);        /* ISR address for transmiting        */
    void  (*rxISRaddr)(int port);        /* ISR address for receiving          */
    void  (*errISRaddr)(int port);       /* to process interrupt due to errors */
    void  (*rxStatAddr)(int port);       /* ISR state machine function address */
#ifndef LINUX
    void  (*rxCallBk)(unsigned char data);/* ISR UART RX to user's function    */
#else
    void  (*rxCallBk)(char data, int port, unsigned char status, unsigned char eof);
#endif
                                
    unsigned int    rxChar;     /* No of characters received includes errors   */
    unsigned int    txChar;     /* No of characters xmitted                    */ 
    unsigned int    rxOvr;      /* Rx overrun count                            */
    unsigned int    rxPE;       /* Rx parity error count                       */
    unsigned int    rxFE;       /* Rx frame  error count                       */
    unsigned int    rxBrk;      /* Rx break signal count                       */
    unsigned int    rxQueFull;  /* Rx queue full                               */

    unsigned int    txIntEnabled;/* 1 - indicates that currently the txEmpty
                                  *     interrupt has been enabled.
                                  * 0 - indicates that currently txEmpty bit
                                  *     interrupt is not enabled.
                                 */

    unsigned int  pollingTxOn; /* 1 - data is being xmitted using Polling      */
                                 /* 0 - currently no data is being xmitted     */

}tPCB;

typedef union
{
    struct
    {
        char reserved       : 4;    
        char mdmStatus      : 1;     /* 1 - interrupt on change in Mdm signals */
        char rcvLineStatus  : 1;     /* 1 - interrupt on error on rcv          */
        char txEmpty        : 1;     /* 1 - interrupt on tx empty              */
        char rcvDataRdy     : 1;     /* 1 - interrupt if data received         */
    }_bits;

    struct
    {
        unsigned char value;
    }_byte;

}tIERreg;

typedef union
{
    struct
    {
        char fifoEnabled    : 2;    
        char reserved       : 2;       
#ifndef ISHONI_PATCH
        char status         : 4;      /* 0000 modem status                     */
#else
        unsigned char status         : 4;      /* 0000 modem status                     */
#endif
                                      /* 0010 Tx Holding Register Empty        */
                                      /* 1100 Character Timeout Ind            */
                                      /* 0100 Rx data available                */
                                      /* 0110 Rx error                         */
                                      /* 0001 none                             */
    }_bits;

    struct
    {
        unsigned char value;
    }_byte;

}tIIRreg;

typedef union 
{
    struct
    {
        char DLAB           : 1;       /* divisor latch bit                    */
        char setBreak       : 1;       /* 1 - generates break character        */
        char stickParity    : 1;       /* stick parity                         */
        char evenParity     : 1;       /* 1 - selects even parity.             */
        char parityEnable   : 1;       /* 1 - interrupt on tx empty            */
        char stopBits       : 1;       /* 0 - nof stop bits is 1               */
                                       /* 1 - nof stop bits is 2 if wordLength 
                                        *     is other than 5.
                                        */

        char wordLength     : 2;       /*
                                        * 10  - 7 bits
                                        * 11  - 8 bits
                                        */
    }_bits;

    struct
    {
        unsigned char value;
    }_byte;

}tLCRreg;

typedef union 
{
    struct
    {
        char errBit             : 1;  /* 0 in non-FIFO mode.
                                       * 1 if parity err, frm err or brk occurs*/
        char txEmpty            : 1;  /* 1 - tx shift and holding reg are empty*/
#ifndef LINUX
        char txHoldRegEmpty     : 1;  /* 1 - tx holding register is empty      */
#else
        volatile char txHoldRegEmpty     : 1;  /* 1 - tx holding register is empty      */
#endif
        char breakIndicator     : 1;    
        char frmErr             : 1;  /* 1 - framing err on byte rcvd          */
        char parityErr          : 1;  /* 1 - parity error occured              */
        char overrunErr         : 1;  /* 1 - over run error occured            */
#ifndef LINUX
        char dataRdy            : 1;  /* 1 - data received                     */     
#else
        volatile char dataRdy            : 1;  /* 1 - data received                     */     
#endif
    
    }_bits;

    struct
    {
        unsigned char value;
    }_byte;

}tLSRreg;

typedef union 
{
    struct
    {
        char reserved           : 3;                            
        char loop               : 1;    /* 1 - loop back feature enabled       */
        char out2               : 1;    /* controls nout2 o/p signal           */
        char out1               : 1;    /* controls nout1 o/p signal           */  
        char RTS                : 1;    /* controls the RTS o/p pin            */
        char DTR                : 1;    /* controls the DTR o/p pin            */
    }_bits;

    struct
    {
        unsigned char value;
    }_byte;

}tMCRreg;

typedef union
{
    struct
    {
        char DCD                : 1;    /* reflects state of ndcd i/p signal   */
        char RI                 : 1;    /* reflects state of nri i/p signal    */
        char DSR                : 1;    /* reflects state of ndsr i/p signal   */
        char CTS                : 1;    /* reflects state of ncts i/p signal   */
        char DDCD               : 1;    /* set whenever DCD changes state      */
        char TERI               : 1;    /* set whenever failling edge on RI    */
                                        /* is detected                         */
        char DDSR               : 1;    /* set whenever DSR changes state      */
        char DCTS               : 1;    /* set whenever CTS changes state      */      
    
    }_bits;

    struct
    {
        unsigned char value;
    }_byte;

}tMSRreg;

typedef union 
{
    struct
    {
        char triggerLevel       : 2;    /*  */
        char reserved           : 2;    /*  */
        char FCR3               : 1;    /*  */
        char FCR2               : 1;    /* 1 - clears all bytes in tx FIFO     */
        char FCR1               : 1;    /* 1 - clears all bytes in rx FIFO     */
        char FCR0               : 1;    /* 1 - enables tx and rx FIFO          */      
    
    }_bits;

    struct
    {
        unsigned char value;
    }_byte;

}tFCRreg;

#define IER 1
#define IIR 2
#define LCR 3
#define MCR 4
#define LSR 5
#define MSR 6


typedef struct Uart
{
    unsigned int driverID;      /*  unique ID that identifies this
                                 *  device driver. This could be
                                 *  the ID maintained by IO Service
                                 *  manager, to identify this driver
                                 */


    unsigned int nofUARTports;  /* Holds the number of UART ports
                                 * available on the chip.
                                 * This is not zero based. Ex: 1, 2, 3. 
                                 */
                           

    tPCB pcb[MAX_NOF_UART_PORTS];/* uart port control block */
                                 /* This pointer points to the array
                                  * of host interfaces. Each tHostInterface
                                  * structure holds information about
                                  * respective interface.
                                  * Currently we have only one HOST 
                                  * interface on NV.
                                 */

    unsigned int txOnPort;      /* Holds the UART port number for
                                 * which the data is being sent.
                                 * Once the complete frame is sent
                                 * out, this field is incrememted to
                                 * point to next UART port number
                                 */

    unsigned int rxOnPort;      /* Holds the UART port number on
                                 * which the data is being rcvd.
                                 */


    unsigned int driverInitialized;     
                                /* 0 -- driver is not yet initialized 
                                 * 1 -- driver has been initialized
                                 */ 

}tUARTDriver;

#endif
#endif
