/*
* cpx1024mon.c
*
* Driver to control access to the hardware monitor on the CPX1024 Quad
* Xeon processor machine built by Diversified Technology Inc.
* 
* For information on programming this driver, refer to
* Documentation/cpx1024.txt in the Linux kernel source code.
*
* Author: Johnnie Peters
*         jpeters@mvista.com
*
* Copyright 2000 MontaVista Softare Inc.
*
*  This program is free software; you can redistribute  it and/or modify it
*  under  the terms of  the GNU General  Public License as published by the
*  Free Software Foundation;  either version 2 of the  License, or (at your
*  option) any later version.
*
*  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR   IMPLIED
*  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
*  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
*  NO  EVENT  SHALL   THE AUTHOR  BE    LIABLE FOR ANY   DIRECT,  INDIRECT,
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
*  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
*  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
*  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
*  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*  You should have received a copy of the  GNU General Public License along
*  with this program; if not, write  to the Free Software Foundation, Inc.,
*  675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Define the ioctl ID interface for the driver */
/* Prepend 'C' for uniqueness */
#define CPX1024MON_MAGIC           'C'
#define CPX1024MON_GET_EVENTS      _IOWR(CPX1024MON_MAGIC, 1, sizeof(struct cpx1024mon_msg_ioctl))
#define CPX1024MON_SEND_ALL_EVENTS _IO(CPX1024MON_MAGIC, 2)
#define CPX1024MON_REG_EVENTS      _IO(CPX1024MON_MAGIC, 3)
#define CPX1024MON_UNREG_EVENTS    _IO(CPX1024MON_MAGIC, 4)
#define CPX1024MON_GET_TEMPS       _IOR(CPX1024MON_MAGIC, 5, sizeof(struct cpx1024mon_temps_ioctl))
#define CPX1024MON_GET_BOUND_TEMPS _IOR(CPX1024MON_MAGIC, 6, sizeof(struct cpx1024mon_bound_temps_ioctl))
#define CPX1024MON_SET_BOUND_TEMPS _IOW(CPX1024MON_MAGIC, 7, sizeof(struct cpx1024mon_bound_temps_ioctl))
#define   CPX1024MON_SET_BT_UPPER   0x1         /* Temperature for minor alarm */
#define   CPX1024MON_SET_BT_LOWER   0x2         /* Temperature for major alarm */
#define   CPX1024MON_SGT_ALARMB	    0x00	/* Alarm Board */
#define   CPX1024MON_SGT_CPU1	    0x01	/* CPU Temp 1 */
#define   CPX1024MON_SGT_CPU2	    0x02	/* CPU Temp 2 */
#define   CPX1024MON_SGT_CPU3	    0x03	/* CPU Temp 3 */
#define   CPX1024MON_SGT_CPU4	    0x04	/* CPU Temp 4 */
#define   CPX1024MON_SGT_IOB	    0x05	/* I/O Board */
#define   CPX1024MON_SGT_SYSTEM1    0x06	/* System Temp 1 */
#define   CPX1024MON_SGT_SYSTEM2    0x07	/* System Temp 2 */
#define   CPX1024MON_SGT_POWER	    0x08	/* Power Supply */
#define   CPX1024MON_SGT_REM1	    0x09	/* Remote Temp 1 */
#define   CPX1024MON_SGT_REM2	    0x0a	/* Remote Temp 2 */
#define   CPX1024MON_SGT_REM3	    0x0b	/* Remote Temp 3 */
#define CPX1024MON_SET_LED         _IOW(CPX1024MON_MAGIC, 8, sizeof(struct cpx1024mon_flash_ioctl))
#define   CPX1024MON_LED_ACTIVE      0x1        /* Active LED */
#define   CPX1024MON_LED_OOS         0x2        /* Out of service LED */
#define   CPX1024MON_LED_OFF         0x0
#define   CPX1024MON_LED_ON          0x1
#define   CPX1024MON_LED_FLASH       0x3
#define CPX1024MON_GET_OVERHEAT    _IOW(CPX1024MON_MAGIC, 9, sizeof(int))
#define CPX1024MON_SOFT_RESET      _IO(CPX1024MON_MAGIC, 10)
#define CPX1024MON_SET_TT_MODE     _IOW(CPX1024MON_MAGIC, 11, sizeof(int))
#define CPX1024MON_FORCE_GPO       _IOW(CPX1024MON_MAGIC, 12, sizeof(struct cpx1024mon_force_gpo_ioctl))
#define   CPX1024MON_FORCE_GPO_LOW      0x1
#define   CPX1024MON_FORCE_GPO_HIGH     0x2
#define CPX1024MON_POWERDOWN       _IO(CPX1024MON_MAGIC, 13)
#define CPX1024MON_GET_VOLT        _IOR(CPX1024MON_MAGIC, 14, sizeof(struct cpx1024mon_volt_ioctl))
#define     CPX1024MON_GVOLT_PLUS5      0x00 /* +5 Voltage */
#define     CPX1024MON_GVOLT_PLUS2_5    0x01 /* +5 Voltage */
#define     CPX1024MON_GVOLT_PLUS12     0x02 /* +2.5 Voltage */
#define     CPX1024MON_GVOLT_MINUS12    0x03 /* +12 Voltage */
#define     CPX1024MON_GVOLT_PLUS3_3    0x04 /* -12 Voltage */
#define     CPX1024MON_GVOLT_VTT        0x05 /* VTT Voltage */
#define     CPX1024MON_GVOLT_CPU1_CORE  0x06 /* CPU1 Core */
#define     CPX1024MON_GVOLT_CPU2_CORE  0x07 /* CPU2 Core */
#define     CPX1024MON_GVOLT_CPU3_CORE  0x08 /* CPU3 Core */
#define     CPX1024MON_GVOLT_CPU4_CORE  0x09 /* CPU4 Core */
#define     CPX1024MON_GVOLT_CPU1_LOG   0x0a /* CPU1 Logic */
#define     CPX1024MON_GVOLT_CPU2_LOG   0x0b /* CPU2 Logic */
#define     CPX1024MON_GVOLT_CPU3_LOG   0x0c /* CPU3 Logic */
#define     CPX1024MON_GVOLT_CPU4_LOG   0x0d /* CPU4 Logic */
#define CPX1024MON_GET_VTOLER      _IOR(CPX1024MON_MAGIC, 15, sizeof(struct cpx1024mon_vtoler_ioctl))
#define     CPX1024MON_TOLER_PLUS5      0x00 /* +5 Voltage */
#define     CPX1024MON_TOLER_PLUS2_5    0x01 /* +2.5 Voltage */
#define     CPX1024MON_TOLER_PLUS12     0x02 /* +12 Voltage */
#define     CPX1024MON_TOLER_MINUS12    0x03 /* -12 Voltage */
#define     CPX1024MON_TOLER_PLUS3_3    0x04 /* +3.3 Voltage */
#define     CPX1024MON_TOLER_VTT        0x05 /* +CPU or VTT */
#define CPX1024MON_SET_VTOLER      _IOW(CPX1024MON_MAGIC, 16, sizeof(struct cpx1024mon_vtoler_ioctl))
#define     CPX1024MON_FAN_1            0x00 /* Fan #1 */
#define     CPX1024MON_FAN_2            0x01 /* Fan #2 */
#define     CPX1024MON_FAN_3            0x02 /* Fan #3 */
#define     CPX1024MON_FAN_4            0x03 /* Fan #4 */
#define     CPX1024MON_FAN_5            0x04 /* Fan #5 */
#define     CPX1024MON_FAN_6            0x05 /* Fan #6 */
#define     CPX1024MON_FAN_STROBE       0x00 /* Strobe mode */
#define     CPX1024MON_FAN_LEVEL        0x01 /* Level mode */
#define     CPX1024MON_FAN_DISABLE      0x02 /* Fan disabled */
#define     CPX1024MON_FAN_LEVEL_LOW    0x00 /* Low level for Level sense mode */
#define     CPX1024MON_FAN_LEVEL_HIGH   0x01 /* High level */
#define CPX1024MON_GET_FAN_INFO    _IOR(CPX1024MON_MAGIC, 17, sizeof(struct cpx1024mon_fan_ioctl))
#define CPX1024MON_SET_FAN_INFO    _IOWR(CPX1024MON_MAGIC, 18, sizeof(struct cpx1024mon_fan_ioctl))
#define CPX1024MON_SET_GPO_POL     _IOW(CPX1024MON_MAGIC, 19, sizeof(struct cpx1024mon_gpo_pol_ioctl))
#define     CPX1024MON_GPO_0            0x00 /* GPO Number 1 */
#define     CPX1024MON_GPO_1            0x01 /* GPO Number 2 */
#define     CPX1024MON_GPO_2            0x02 /* GPO Number 3 */
#define     CPX1024MON_GPO_3            0x03 /* GPO Number 4 */
#define     CPX1024MON_GPO_LOW          0x00 /* Normally Low */
#define     CPX1024MON_GPO_HIGH         0x01 /* Normally High */
#define     CPX1024MON_GPO_FLOW         0x02 /* Forced Low */
#define     CPX1024MON_GPO_FHIGH        0x03 /* Forced High */
#define CPX1024MON_GET_GPO_POL     _IOR(CPX1024MON_MAGIC, 20, sizeof(struct cpx1024mon_gpo_pol_ioctl))
#define CPX1024MON_SET_GPI_POL     _IOW(CPX1024MON_MAGIC, 21, sizeof(struct cpx1024mon_gpi_pol_ioctl))
#define     CPX1024MON_GPI_0            0x00 /* GPI Number 1 */
#define     CPX1024MON_GPI_1            0x01 /* GPI Number 2 */
#define     CPX1024MON_GPI_2            0x02 /* GPI Number 3 */
#define     CPX1024MON_GPI_3            0x03 /* GPI Number 4 */
#define     CPX1024MON_GPI_4            0x04 /* GPI Number 5 */
#define     CPX1024MON_GPI_5            0x05 /* GPI Number 6 */
#define     CPX1024MON_GPI_6            0x06 /* GPI Number 7 */
#define     CPX1024MON_GPI_7            0x07 /* GPI Number 8 */
#define     CPX1024MON_GPI_POL_LOW      0x0
#define     CPX1024MON_GPI_POL_HIGH     0x1
#define     CPX1024MON_GPI_POL_DISABLE  0x2
#define CPX1024MON_GET_GPI_POL     _IOR(CPX1024MON_MAGIC, 22, sizeof(struct cpx1024mon_gpi_pol_ioctl))
#define CPX1024MON_GET_TEMPMASK    _IOR(CPX1024MON_MAGIC, 23, sizeof(int))
#define   CPX1024MON_TMASK_MEM1     0x1
#define   CPX1024MON_TMASK_MEM2     0x2
#define   CPX1024MON_TMASK_IO       0x4
#define   CPX1024MON_TMASK_ALARM    0x8
#define   CPX1024MON_TMASK_POWER    0x10
#define   CPX1024MON_TMASK_REM1     0x20
#define   CPX1024MON_TMASK_REM2     0x40
#define   CPX1024MON_TMASK_REM3     0x80
#define   CPX1024MON_TMASK_CPU1     0x100
#define   CPX1024MON_TMASK_CPU2     0x200
#define   CPX1024MON_TMASK_CPU3     0x400
#define   CPX1024MON_TMASK_CPU4     0x800
#define CPX1024MON_GET_CPUS        _IOR(CPX1024MON_MAGIC, 24, sizeof(int))
#define   CPX1024MON_CPUS_1         0x1
#define   CPX1024MON_CPUS_2         0x2
#define   CPX1024MON_CPUS_3         0x4
#define   CPX1024MON_CPUS_4         0x8
#define CPX1024MON_GET_MON_INFO    _IOR(CPX1024MON_MAGIC, 25, sizeof(struct cpx1024mon_info))
#define CPX1024MON_GET_AMPS        _IOR(CPX1024MON_MAGIC, 26, sizeof(struct cpx1024mon_amp_ioctl))
#define     CPX1024MON_AMPS_5_MOD1   0x00 /* +5 Module 1 */
#define     CPX1024MON_AMPS_5_MOD2   0x01 /* +5 Module 2 */
#define     CPX1024MON_AMPS_3_3      0x02 /* +3.3 Module */
#define CPX1024MON_UPDATE_EEPROM   _IO(CPX1024MON_MAGIC, 27)
#define CPX1024MON_SERIAL_ECHO     _IOW(CPX1024MON_MAGIC, 28, sizeof(int))
#define CPX1024MON_SERIAL_TRANSFER _IOW(CPX1024MON_MAGIC, 29, sizeof(int))
#define CPX1024MON_SERIAL_VERBOSE  _IOW(CPX1024MON_MAGIC, 30, sizeof(int))
#define CPX1024MON_SERIAL_PASSWD   _IOW(CPX1024MON_MAGIC, 31, sizeof(struct cpx1024mon_passwd_ioctl))
#define CPX1024MON_SERIAL_SETUID   _IOW(CPX1024MON_MAGIC, 32, sizeof(struct cpx1024mon_uid_ioctl))


/* A few sizes that need to be known */
#define CPX1024_MAX_MSG            10
#define CPX1024_TEMP_MAX           0xc
#define CPX1024_VOLT_SIZE          0xe
#define CPX1024_VOLT_TOL_SIZE      0x6
#define CPX1024_MAX_FAN            6
#define CPX1024_MAX_GPO            3
#define CPX1024_MAX_GPI            7
#define CPX1024_MAX_GPI_POL        2
#define CPX1024_AMP_SIZE           3


/* Format of returning termperatures */
struct cpx1024mon_temps_ioctl {
	int    temps[CPX1024_TEMP_MAX][2];
};

/* Format to get and set boundary temperatures */
struct cpx1024mon_bound_temps {
	int    flag;
	int    low[2];
	int    high[2];
};

struct cpx1024mon_bound_temps_ioctl {
	struct cpx1024mon_bound_temps bound_temps[CPX1024_TEMP_MAX];
};

/* Define the structure to hold event messages returned to calling program */
struct cpx1024mon_msg_buf {
	int    code;
	int    val;
};

struct cpx1024mon_msg_ioctl {
	int                       count;
	struct cpx1024mon_msg_buf msgs[CPX1024_MAX_MSG];
};

/* Set LED modes */
struct cpx1024mon_flash_ioctl {
	int    flags;
	int    active[2];
	int    out_of_service[2];
};

/* Force GPO connections */
struct cpx1024mon_force_gpo_ioctl {
	int    gpo;
	int    state;
};

/* Return voltage values */
struct cpx1024mon_volt_ioctl {
	int    volt[CPX1024_VOLT_SIZE][2];
};

/* Get and set voltage tolerances */
struct cpx1024mon_vtoler_ioctl {
	int    volt[CPX1024_VOLT_TOL_SIZE];
};

/* Get and set fan control values */
struct fan_info {
	int    mode;
	int    status;
	int    rps_level;
};

struct cpx1024mon_fan_ioctl {
	struct fan_info fan[CPX1024_MAX_FAN];
};

/* Get and set polarity value for the GPO interface */
struct cpx1024mon_gpo_pol_ioctl {
	int   polarity[CPX1024_MAX_GPO + 1];
};

/* Get and set polarity value for the GPI interface */
struct cpx1024mon_gpi_pol_ioctl {
	int   polarity[CPX1024_MAX_GPI + 1];
};

/* Retrieve various information about the monitor */
struct cpx1024mon_info {
	int mon_rev;
	int mon_patch;
	int bios_rev;
};

/* Retrieve amperage on power supplies */
struct cpx1024mon_amp_ioctl {
	int    line[CPX1024_AMP_SIZE][2];
};

/* Set serial port password */
struct cpx1024mon_passwd_ioctl {
	char   pw[4];
};

/* Set unit ID */
struct cpx1024mon_uid_ioctl {
	char   uid[11];
};

/* Alarms values returned in the code section of the message buffer. */
#define DTIMON_ALRM_VOLT_ON        1
#define DTIMON_ALRM_VOLT_OFF       2

#define DTIMON_ALRM_ONBTEMP_ON     3
#define DTIMON_ALRM_ONBTEMP_OFF    4

#define DTIMON_ALRM_OFFBTEMP_ON    5
#define DTIMON_ALRM_OFFBTEMP_OFF   6

#define DTIMON_ALRM_FAN_ON         7
#define DTIMON_ALRM_FAN_OFF        8

#define DTIMON_ALRM_GPI_ON         9
#define DTIMON_ALRM_GPI_OFF        10

#define DTIMON_ALRM_CPU_ON         11
#define DTIMON_ALRM_CPU_OFF        12

#define DTIMON_ALRM_MAJORLED_ON    13
#define DTIMON_ALRM_MAJORLED_OFF   14

#define DTIMON_ALRM_MINORLED_ON    15
#define DTIMON_ALRM_MINORLED_OFF   16


/*
 * Local variables:
 *  c-indent-level: 8
 *  c-basic-offset: 8
 *  tab-width: 8
 * End:
 */
